#!/usr/bin/env bash
# Author: 潘维吉
# Description: 检测项目是否启动成功 查看变更记录 钉钉机器人通知

start_time=$(date +'%Y-%m-%d %H:%M:%S')

DING_TALK_URL='https://oapi.dingtalk.com/robot/send?access_token=adfdfd9be880ba09235c64d822403d4ff6a076abcd8aa01b0f69df88bf79a156'
KEY_WORD='安泰科技'

PROJECT_TYPE=$1  # 1 前端 2 后端
DATE_TYPE=$2 # 1 发布前通知 2 发布后通知
PROJECT_NAME=$3  # 项目名
URL=$4 # 项目检测的URL
RELEASE_NAME=$5 # 发布人

function sendMsgBefore() {
  curl ${DING_TALK_URL} \
    -X POST \
    -H 'Content-Type: application/json' \
    -d '{"msgtype":"markdown",
    "markdown": {
    "title":"'${KEY_WORD}'CI/CD之前通知",
    "text":"### '${PROJECT_NAME}' \n\n ####  项目部署启动中, 请稍等...  ☕"
    },
    "at": {
    "atMobiles":[],
    "isAtAll":false}
    }'
}

function sendMsgAfter() {
  start_seconds=$(date --date="${start_time}" +%s)
  end_time=$(date +'%Y-%m-%d %H:%M:%S')
  end_seconds=$(date --date="${end_time}" +%s)

  if [[ ${PROJECT_TYPE} == 1 ]]; then
    curl ${DING_TALK_URL} \
      -X POST \
      -H 'Content-Type: application/json' \
      -d '{"msgtype":"markdown",
    "markdown": {
    "title":"'${KEY_WORD}'CI/CD之后通知",
    "text":"### '${PROJECT_NAME}' 前端 \n\n #### - Jenkins构建Docker部署成功👌 \n #### - 前端Web项目启动'$1' \n\n ###### 启动耗时: '$((end_seconds - start_seconds + 1))'s \n ###### 访问地址URL ['${URL}']('${URL}') \n ###### 发布人: '${RELEASE_NAME}'"

    },
    "at": {
    "atMobiles":["18863302302"],
    "isAtAll":false}
    }'
  elif [[ ${PROJECT_TYPE} == 2 ]]; then
    git_lab_url='http://git.antaikeji.top/anjia/an-plus/commits/dev'
    ding_dynamic_text="" # 注意钉钉参数不能有空字符串 如果有报错{"errcode":40035,"errmsg":"缺少参数 json"}
    if [[ ${PROJECT_NAME} =~ an-plus-app ]]; then
      ding_dynamic_text="Swagger文档:[查看](${URL/api/}doc.html)"
    fi
    if [[ ${PROJECT_NAME} =~ an-plus-job ]]; then
      ding_dynamic_text="Job中心:[查看](http://39.96.84.51:8081/xxl-job-admin/)"
    fi
    echo "动态文本: ${ding_dynamic_text}"
    curl ${DING_TALK_URL} \
      -X POST \
      -H 'Content-Type: application/json' \
      -d '{"msgtype":"markdown",
    "markdown": {
    "title":"'${KEY_WORD}'CI/CD之后通知",
    "text":"### '${PROJECT_NAME}' 后端 \n\n #### - Jenkins构建Docker部署成功👌 \n #### - Spring Boot项目启动'$1' \n\n ###### 启动耗时: '$((end_seconds - start_seconds + 1))'s \n ###### '${ding_dynamic_text}' \n ###### API 地址: '${URL}'  \n ###### Jenkins地址 [查看](http://39.96.84.51:8000/) \n ###### 版本变更记录 [查看全部]('${git_lab_url}') [新增功能]('${git_lab_url}'?utf8=%E2%9C%93&search=feat) [修复Bug]('${git_lab_url}'?utf8=%E2%9C%93&search=fix) \n ###### 发布人: '${RELEASE_NAME}'"
    },
    "at": {
    "atMobiles":["18863302302"],
    "isAtAll":false}
    }'
  fi
}

if [[ ${DATE_TYPE} == 1 ]]; then
  echo "发布部署之前通知"
  sendMsgBefore
elif [[ ${DATE_TYPE} == 2 ]]; then
  if [[ ${PROJECT_TYPE} == 1 ]]; then
    echo "前端Web项目检测启动健康状态"
    for i in $(seq 1 20); do
      if [[ ${i} == 20 ]]; then
        echo "前端Web启动失败 发送钉钉通知"
        sendMsgAfter "**失败❌**"
        break
      else
        echo "第$i次 前端Web启动检测"
        result=$(curl -I ${URL} | grep OK)
        if [[ ${result} =~ OK ]]; then
          echo "前端Web启动成功"
          sendMsgAfter "**成功✅**"
          break
        fi
        sleep 3s
      fi
    done
  elif [[ ${PROJECT_TYPE} == 2 ]]; then
    # 循环检测启动健康状态
    echo "后端项目检测启动健康状态"
    sleep 3s
    for i in $(seq 1 60); do
      if [[ ${i} == 60 ]]; then
        echo "Spring Boot启动失败 发送钉钉通知"
        sendMsgAfter "**失败❌**"
        break
      else
        sleep 3s
        echo "第$i次 Spring Boot启动检测"
        result1=$(curl ${URL} | grep 404)
        result2=$(curl ${URL} | grep 401)
        if [[ ${result1} =~ 404 || ${result2} =~ 401 ]]; then
          echo "Spring Boot启动成功"
          sendMsgAfter "**成功✅**"
          break
        fi
      fi
    done
  fi
fi

#log_file_path=/tmp/spring_boot_start.log
#
##  &表示输出转存到文件后，这条命令后台执行，没有&，下面的输出会看不到
#docker logs -f an-plus-app >${log_file_path} &
#
#while [ -f ${log_file_path} ]; do
#  result=$(grep ${log_file_path} | grep "Started AppApplication in")
#  if [[ "$result" != "" ]]; then
#    echo "Spring Boot 正在启动 ........."
#    sleep 3s
#  else
#    echo "Spring Boot 正在运行 ........."
#    break
#  fi
#done
#echo "Spring Boot 启动完成"
