#!/usr/bin/env bash
# Author: 潘维吉
# Description: 执行Jenkins Docker部署shell脚本

echo "执行Jenkins Docker部署shell脚本"
pwd
# cd /my && chmod +x jenkins-docker.sh 给shell脚本执行文件可执行权限
# 可采用$0,$1,$2..等方式获取脚本命令行传入的参数  执行脚本 cd /my && sudo ./jenkins-docker.sh project-name app 8080 8080 prod 5005
# 项目名称变量
project_name_prefix=$1
# 项目类型
project_type=$2
# 宿主机端口
host_port=$3
# 容器内暴露端口
expose_port=$4
# 环境模式 如 dev test prod等
env_mode=$5
# 远程调试端口
remote_debug_port=$6
# job执行器端口
job_port=$7

# 当前日期格式
date=$(date '+%Y%m%d-%H%M')
# docker镜像名称
docker_image_name=${project_name_prefix}/${project_type}-${env_mode}
# docker容器名称
docker_container_name=${project_name_prefix}-${project_type}-${env_mode}
# 项目全名称
project_name=${project_name_prefix}-${project_type}
# 部署文件夹
deploy_file=/my/${project_name}

# 检测是否存在Dockerfile文件 不存在退出执行
if [[ ! -f "Dockerfile" ]]; then
  echo "Dockerfile文件不存在 无法构建docker镜像 shell命令退出 构建镜像失败"
  exit
fi

# 检测是否存在部署文件夹 如果不存在创建一个
if [[ ! -d ${deploy_file} ]]; then
  mkdir -p ${deploy_file}
  echo "创建部署文件夹: ${deploy_file}"
fi

# 复制字体文件
cp -p msyh.ttc ${deploy_file}/

# 进入部署文件所在目录
cd ${deploy_file}

# docker构建和运行动态参数处理
build_expose_ports=${expose_port}
dynamic_run_args=""
if [[ ${job_port} ]]; then
  build_expose_ports="${expose_port} ${job_port}"
  dynamic_run_args="-p ${job_port}:${job_port} -e "JOB_PORT=${job_port}" "
fi

echo "构建暴露端口: ${build_expose_ports}"
echo "运行动态参数: ${dynamic_run_args}"

echo "构建镜像(无缓存构建)"
docker build -t ${docker_image_name} \
  --build-arg PROJECT_NAME=${project_name} --build-arg EXPOSE_PORT="${build_expose_ports}" \
  -f /my/Dockerfile . --no-cache

echo "停止指定的容器"
docker stop ${docker_container_name} --time=0
echo "删除指定的容器"
docker rm ${docker_container_name}

echo "启动运行docker容器 模式: ${env_mode} 映射端口: ${host_port}:${expose_port}"
docker run -d --restart=always -p ${host_port}:${expose_port} -p ${remote_debug_port}:5005 \
  -e "SPRING_PROFILES_ACTIVE=${env_mode}" -e "PROJECT_NAME=${project_name}" ${dynamic_run_args} \
  -e "JAVA_OPTS=-Xms512m -Xmx600m" -m 800m \
  -v /my/${project_name}/logs:/logs \
  --name ${docker_container_name} ${docker_image_name}

echo "删除名称或标签为none的镜像"
docker rmi $(docker images | grep "none" | awk '{print $3}') -f


#echo "删除三天之前的备份jar包"
#命令 >/dev/null 2>&1 常用来避免shell命令或者程序等运行中有内容输出
#find ${deploy_file} -name "*.jar" -type f -mtime +3 -exec rm -rf {} \; > /dev/null 2>&1

#echo "删除匹配的无效部署文件夹和文件"
#rm -rf *${project_name}*
#rm -f *${project_name}*

#  Jenkins单独指定模块构建 -pl指定项目名 -am 同时构建依赖项目模块
#  (mvn) clean install -pl app -am -Dmaven.test.skip=true  跳过单元测试打包
#  前端命令和shell命令都可使用 && 将多个命令连接执行
#  npm install && npm run build
