# 安家物业总服务端

#### 项目介绍
Spring Boot2.x模块化项目 服务端

#### 模块目录结构
- .ci : CI/CD 全自动化持续集成和交付 如jenkins构建部署脚本 动态传参的Dockerfile等
- _docker : docker自动化shell脚本、docker初始化安装以及常用服务镜像容器创建管理说明、docker compose容器编排yaml配置等
- _linux  : linux相关的自动化shell脚本、kubernetes(k8s)和常用命令说明等 
- _mysql  : mysql数据自动化定时备份shell脚本、主从复制读写分离配置、数据库表结构和初始化数据SQL语句等
- _nginx  : nginx做正反向代理、分布式、负载均衡、蓝绿部署(不停机)和SSL安全证书https配置等
- core : 核心模块 (与业务无关，可能80%项目依赖需要)
- common: 公共通用模块 (与业务无关，可能50%项目依赖需要)
- dao  : 数据库通用配置层和业务领域模型层模块
- constants  : 项目常量和通用yaml配置模块
- test : 测试 、实验性功能 、CRUD代码自动生成等模块 (与生产环境无关 可随便折腾😅)
- an-plus-common-app     : APP所有模块的业务通用模块 (业务模块)
- an-plus-common-admin     : Admin所有模块的业务通用模块 (业务模块)
- an-plus-app       : App服务端 (业务模块)
- an-plus-app-admin     : APP管理端的服务端 (业务模块)
- an-plus-admin : 后台管理系统服务端 (业务模块)
- an-plus-admin-control : 总控系统服务端 (业务模块)
- an-plus-job: 总任务调度服务端 (业务模块)
- an-plus-message : 消息服务端 (业务模块)

#### 特征&提供
- 最佳实践的项目结构、配置文件、精简的POM
- 统一响应结果封装及生成工具
- 统一异常处理
- JWT的接口签名认证
- 常用基础方法抽象封装
- 使用FastJsonHttpMessageConverter，提高JSON序列化速度
- 集成MyBatis、通用Mapper插件、PageHelper分页插件，实现单表业务零SQL
- 提供代码生成器根据表名生成对应的Model、Mapper、MapperXML、Service、ServiceImpl、Controller等基础代码，
  其中Controller模板默认提供RESTFul，根据需求在```CodeGenerator.genController(tableName)```方法中自己选择，
  代码模板可根据实际项目的需求来扩展，所以只提供了一些比较基础、通用的模板，**主要是提供一个思路**来减少重复代码的编写，使用模板也有助于保持团队代码风格的统一

#### 开发建议
- 表名，建议使用小写，多个单词使用下划线拼接
- Model内成员变量建议与表字段数量对应，如需扩展成员变量（比如连表查询）建议创建请求参数DTO类,响应数据VO类，否则需在扩展的成员变量上加```@Transient```注解
- 建议业务失败直接使用```ServiceException("message")```抛出，由统一异常处理器来封装业务失败的响应结果，比如```throw new ServiceException("该手机号已被注册")```，会直接被封装为```{"code":400,"msg":"该手机号已被注册"}```返回，无需自己处理，尽情抛出
- 需要工具类的话建议先从```apache-commons-*```和```guava```中找，实在没有再造轮子或引入类库，尽量精简项目
- 开发规范建议遵循阿里巴巴Java开发手册（[最新版下载](https://github.com/alibaba/p3c))

 
#### 技术选型&文档
- Spring Boot（[查看Spring Boot学习&使用指南](http://www.jianshu.com/p/1a9fd8936bd8)）
- MyBatis（[查看官方中文文档](http://www.mybatis.org/mybatis-3/zh/index.html)）
- MyBatis通用Mapper插件（[查看官方中文文档](https://mapperhelper.github.io/docs/)）
- MyBatis PageHelper分页插件（[查看官方中文文档](https://pagehelper.github.io/)）
- FastJson（[查看官方中文文档](https://github.com/Alibaba/fastjson/wiki/%E9%A6%96%E9%A1%B5)）


#### Git Commit Message格式说明一般包括三部分：Header、Body和Footer  
message信息格式采用目前主流的Angular规范，是目前使用最广的写法，比较合理和系统化，并且有配套的工具
在IDEA 可安装Git Commit Message插件 自动生成规范提交信息

##### Header
type(scope): subject

type: 用于说明commit的类别，规定为如下几种 
- feat: 新增功能
- fix: 修复 bug
- docs: 修改文档 比如 README, CHANGELOG, CONTRIBUTE等
- refactor: 代码重构，未新增任何功能和修复任何 bug
- build: 改变构建流程，新增依赖库、工具等（例如 webpack，maven等修改）
- style: 仅仅修改了空格、缩进、逗号等，不改变代码逻辑
- perf: 改善性能和体验的修改
- chore: 非源码和测试文件修改的杂项处理
- test: 测试用例的修改 包括单元测试、集成测试等
- ci: 自动化流程配置修改
- revert: 回滚到上一个版本
- scope: 【可选】用于说明 commit 的影响范围
- subject: commit 的简要说明，尽量简短

##### Body
对本次commit的详细描述，可分多行 

##### Footer
不兼容变动：需要描述相关信息
关闭指定Issue：输入Issue信息


#### 在File->Settings->File and Code Templates->Includes标签->File Header 添加如下注释信息，之后创建类会自动生成注释信息
/**
 * @author 潘维吉  
 * @date ${DATE} ${TIME}
 * @email 406798106@qq.com
 * @description 
 */

#### RESTFul API规范

- URL中不能有动词 宾语必须是名词 URL路径中首选小写字母 反例: /getAllCars /createNewCar /deleteAllRedCars等
- URL使用连字符”-”或使用下划线”_” 连接多单词 统一即可 尽量使用单单词
- 集合使用复数 URL 如: /articles（读取所有文章)
- 避免多级 URL 正例: /users/1?schools=2 反例: /users/1/schools/2
