# drop table if exists base_table;
/*create table if not exists base_table
(
    id         bigint auto_increment comment '主键'
        primary key,

    version    int      default 0                 null comment '数据版本号',
    deleted    tinyint  default 0                 null comment '删除标记',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '通用基础设计';*/

create table if not exists device
(
    id               bigint auto_increment comment '主键'
        primary key,
    community_id     bigint                             not null comment '社区id',
    device_type_id   bigint                             not null comment '设备类型id',
    new_device_id    bigint                             null comment '关联新设备id',
    code             varchar(32)                        not null comment '设备编码',
    name             varchar(32)                        not null comment '设备名称',
    unit_type        varchar(64)                        null comment '设备型号',
    manufacturer     varchar(64)                        null comment '生产厂商',
    image            varchar(255)                       null comment '设备图片',
    qr_code          varchar(128)                       null comment '二维码标识',
    qr_code_path     varchar(128)                       null comment '二维码地址',
    location         varchar(128)                       not null comment '位置',
    lon              decimal(10, 6)                     null comment '经度',
    lat              decimal(10, 6)                     null comment '纬度',
    install_date     datetime                           not null comment '安装日期',
    device_params    varchar(5000)                      null comment '设备参数',
    inspection_guide varchar(5000)                      null comment '巡检指南',
    maintain_guide   varchar(5000)                      null comment '保养指南',
    inspection_num   int      default 0                 null comment '巡检次数',
    maintain_num     int      default 0                 null comment '保养次数',
    repair_num       int      default 0                 null comment '维修次数',
    sign             tinyint  default 0                 null comment '设备标记 0: 正常 1: 报修',
    status           tinyint  default 0                 null comment '设备状态 0: 正常 1: 报废',
    scrap_reason     varchar(255)                       null comment '报废原因',
    remark           varchar(255)                       null comment '备注',
    version          int      default 0                 null comment '数据版本号',
    deleted          tinyint  default 0                 null comment '删除标记',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间',
    constraint unique_device_code_deleted
        unique (code, deleted)
)
    comment '设备信息';

create table if not exists device_category
(
    id         bigint auto_increment comment '主键'
        primary key,
    code       varchar(32)                        not null comment '类型编码',
    name       varchar(32)                        not null comment '类型名称',
    image      varchar(255)                       null comment '缩略图',
    remark     varchar(255)                       null comment '备注',
    version    int      default 0                 null comment '数据版本号',
    deleted    tinyint  default 0                 null comment '删除标记',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间',
    constraint unique_device_category_code
        unique (code)
)
    comment '设备父类型';

create table if not exists device_type
(
    id                 bigint auto_increment comment '主键'
        primary key,
    device_category_id bigint                             not null comment '设备父级id',
    community_ids      varchar(2000)                      not null comment '社区表ids 逗号,分割',
    code               varchar(32)                        not null comment '类型编码',
    name               varchar(32)                        not null comment '类型名称',
    image              varchar(255)                       null comment '缩略图',
    qr_code_size       tinyint  default 0                 null comment '二维码大小 0: 小码 1: 大码',
    remark             varchar(255)                       null comment '备注',
    version            int      default 0                 null comment '数据版本号',
    deleted            tinyint  default 0                 null comment '删除标记',
    ct_user_id         bigint                             null comment '创建人',
    ct_date            datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id         bigint                             null comment '最后修改人',
    lm_date            datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间',
    constraint unique_device_type_code
        unique (code)
)
    comment '设备子类型';

create table if not exists device_check_item
(
    id             bigint auto_increment comment '主键'
        primary key,
    type           tinyint                            not null comment '1.巡检 2.保养',
    device_type_id bigint                             not null comment '设备类型id',
    name           varchar(255)                       not null comment '检查项',
    version        int      default 0                 null comment '数据版本号',
    deleted        tinyint  default 0                 null comment '删除标记',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '巡检保养检查项';

create table if not exists device_overdue_config
(
    id            bigint auto_increment comment '主键'
        primary key,
    type          tinyint                              not null comment '1.巡检 2.保养',
    community_id  bigint                               not null comment '所属社区id',
    plan_type     varchar(16)                          not null comment '计划类型 day:天 month:月 year:年',
    overdue_time  decimal(10, 1)                       null comment '逾期时间（小时）',
    warn_count    tinyint                              null comment '警告次数',
    warn_interval decimal(10, 1)                       null comment '警告时间间隔',
    warn_content  varchar(255)                         null comment '警告内容',
    is_send_sms   tinyint(1) default 0                 null comment '是否发送短信提醒 0:否 1:是',
    version       int        default 0                 null comment '数据版本号',
    deleted       tinyint    default 0                 null comment '删除标记',
    ct_user_id    bigint                               null comment '创建人',
    ct_date       datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                               null comment '最后修改人',
    lm_date       datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养逾期配置';

create table if not exists device_route
(
    id           bigint auto_increment comment '主键'
        primary key,
    type         tinyint                            not null comment '1.巡检 2.保养',
    community_id bigint                             not null comment '社区id',
    route_name   varchar(128)                       not null comment '路线名称',
    device_ids   varchar(5000)                      not null comment '设备ids 逗号,分割',
    remark       varchar(255)                       null comment '备注',
    version      int      default 0                 null comment '数据版本号',
    deleted      tinyint  default 0                 null comment '删除标记',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'

)
    comment '设备巡检保养线路';

create index index_device_route_type
    on device_route (type);

create table if not exists device_plan_rule
(
    id              bigint auto_increment comment '主键'
        primary key,
    type            tinyint                            not null comment '1.巡检 2.保养',
    name            varchar(128)                       not null comment '计划名称',
    plan_type       varchar(16)                        not null comment '计划类型 day:天 month:月 year:年',
    user_id         bigint                             not null comment '巡检保养用户id',
    device_route_id bigint                             not null comment '路线id',
    start_time      datetime                           null comment '开始时间',
    end_time        datetime                           null comment '结束时间',
    execute_num     int                                null comment '执行次数',
    is_affirm       tinyint(1)                         null comment '是否确认 0: 否 1: 是',
    remark          varchar(255)                       null comment '备注',
    version         int      default 0                 null comment '数据版本号',
    deleted         tinyint  default 0                 null comment '删除标记',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划规则';

create table if not exists device_plan
(
    id              bigint auto_increment comment '主键'
        primary key,
    type            tinyint                                  not null comment '1.巡检 2.保养',
    user_id         bigint                                   not null comment '巡检保养用户id',
    device_id       bigint                                   not null comment '设备id',
    device_route_id bigint                                   null comment '路线id',
    plan_type       varchar(16)                              not null comment '计划类型 day:天 month:月 year:年',
    plan_date       date                                     not null comment '执行日期',
    electricity     decimal(10, 2) default 0.00              null comment '电流',
    water_pressure  decimal(10, 2) default 0.00              null comment '水压',
    voltage         decimal(10, 2) default 0.00              null comment '电压',
    temperature     decimal(10, 2) default 0.00              null comment '温度',
    total_num       int            default 0                 null comment '总次数',
    completed_num   int            default 0                 null comment '已完成次数',
    is_complete     tinyint(1)     default 0                 null comment '是否完成 0: 否 1: 是',
    is_suspend      tinyint(1)     default 0                 null comment '是否暂停 0: 否 1: 是',
    solve_time      datetime                                 null comment '解决时间',
    remark          varchar(255)                             null comment '备注',
    version         int            default 0                 null comment '数据版本号',
    deleted         tinyint        default 0                 null comment '删除标记',
    ct_user_id      bigint                                   null comment '创建人',
    ct_date         datetime       default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                                   null comment '最后修改人',
    lm_date         datetime       default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划';

create index index_device_plan_type
    on device_plan (type);
create index index_device_plan_user_id
    on device_plan (user_id);
create index index_device_plan_device_id
    on device_plan (device_id);

create table if not exists device_plan_detail
(
    id             bigint auto_increment comment '主键'
        primary key,
    device_plan_id bigint                                   not null comment '计划id',
    community_id   bigint                                   null comment '社区id',
    device_id      bigint                                   not null comment '设备id',
    user_id        bigint                                   not null comment '用户id',
    plan_date      date                                     not null comment '计划执行日期',
    electricity    decimal(10, 2) default 0.00              null comment '电流',
    water_pressure decimal(10, 2) default 0.00              null comment '水压',
    voltage        decimal(10, 2) default 0.00              null comment '电压',
    temperature    decimal(10, 2) default 0.00              null comment '温度',
    count          int                                      null comment '巡检保养次数标识',
    is_complete    tinyint(1)     default 0                 null comment '是否完成 0: 否 1: 是',
    status         tinyint(1)     default 0                 not null comment '状态 0: 正常 1: 异常',
    file_urls      varchar(6000)                            null comment '文件urls 逗号,分割',
    remark         varchar(255)                             null comment '备注',
    version        int            default 0                 null comment '数据版本号',
    deleted        tinyint        default 0                 null comment '删除标记',
    ct_user_id     bigint                                   null comment '创建人',
    ct_date        datetime       default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                                   null comment '最后修改人',
    lm_date        datetime       default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划明细';

create index index_device_plan_detail_device_plan_id
    on device_plan_detail (device_plan_id);

create table if not exists device_plan_detail_record
(
    id          bigint auto_increment comment '主键'
        primary key,
    type        tinyint                              not null comment '1.巡检 2.保养',
    detail_id   bigint                               not null comment '明细id',
    user_id     bigint                               not null comment '用户id',
    item_id     bigint                               not null comment '检查项id',
    item_status tinyint(1) default 1                 not null comment '检查项是否正常 0: 否 1: 是',
    version     int        default 0                 null comment '数据版本号',
    deleted     tinyint    default 0                 null comment '删除标记',
    ct_user_id  bigint                               null comment '创建人',
    ct_date     datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                               null comment '最后修改人',
    lm_date     datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养明细记录';

create table if not exists device_plan_subsidiary
(
    id             bigint auto_increment comment '主键'
        primary key,
    type           tinyint                              not null comment '1.巡检 2.保养',
    device_plan_id bigint                               not null comment '计划id',
    plan_type      varchar(16)                          not null comment '计划类型 day:天 month:月 year:年',
    user_id        bigint                               not null comment '巡检保养用户id',
    start_time     datetime                             null comment '开始时间',
    end_time       datetime                             null comment '结束时间',
    execution_date date                                 null comment '执行日期',
    count          int                                  null comment '巡检保养次数标识',
    is_complete    tinyint(1) default 0                 null comment '是否完成 0: 否 1: 是',
    version        int        default 0                 null comment '数据版本号',
    deleted        tinyint    default 0                 null comment '删除标记',
    ct_user_id     bigint                               null comment '创建人',
    ct_date        datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                               null comment '最后修改人',
    lm_date        datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划附属细则';
create index index_device_plan_subsidiary_device_plan_id
    on device_plan_subsidiary (device_plan_id);

create table if not exists device_plan_transfer_log
(
    id             bigint auto_increment comment '主键'
        primary key,
    device_plan_id bigint                             not null comment '计划id',
    device_id      bigint                             not null comment '设备id',
    before_user_id bigint                             not null comment '被转移用户id',
    now_user_id    bigint                             not null comment '当前用户id',
    execution_date date                               null comment '执行日期',
    remark         varchar(255)                       null comment '备注',
    version        int      default 0                 null comment '数据版本号',
    deleted        tinyint  default 0                 null comment '删除标记',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划转移日志';

create table if not exists device_repair_config
(
    id            bigint auto_increment comment '主键'
        primary key,
    community_id  bigint                               not null comment '社区id',
    name_code     varchar(16)                          null comment '配置环节名称(字典字段)',
    overdue_time  decimal(10, 1)                       null comment '逾期时间（小时）',
    is_send_sms   tinyint(1) default 0                 null comment '是否发送短信提醒 0: 否 1: 是',
    version       int        default 0                 null comment '数据版本号',
    deleted       tinyint    default 0                 null comment '删除标记',
    ct_user_id    bigint                               null comment '创建人',
    ct_date       datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                               null comment '最后修改人',
    lm_date       datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备报修配置';

create table if not exists device_repair_config_handle
(
    id         bigint auto_increment comment '主键'
        primary key,
    config_id  bigint                             not null comment '备维修配置id',
    type       tinyint                            null comment '处理类型 0.不报修 1.自修 2.派工维修',
    role_id    bigint                             null comment '角色id',
    user_id    bigint                             null comment '用户id',
    version    int      default 0                 null comment '数据版本号',
    deleted    tinyint  default 0                 null comment '删除标记',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备报修配置处理';

create table if not exists device_repair_handle_warn
(
    id            bigint auto_increment comment '主键'
        primary key,
    handle_id     bigint                               null comment '设备维修配置处理id',
    role_id       bigint                               null comment '角色id',
    warn_interval decimal(10, 1)                       null comment '警告时间间隔',
    warn_count    tinyint                              null comment '警告次数',
    msg           varchar(150)                         null comment '预警消息',
    is_send_sms   tinyint(1) default 0                 null comment '是否发送短信提醒 0: 否 1: 是',
    version       int        default 0                 null comment '数据版本号',
    deleted       tinyint    default 0                 null comment '删除标记',
    ct_user_id    bigint                               null comment '创建人',
    ct_date       datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                               null comment '最后修改人',
    lm_date       datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备报修处理预警';

create table if not exists device_repair_handle_user
(
    id         bigint auto_increment comment '主键'
        primary key,
    handle_id  bigint                             null comment '设备维修配置处理id',
    type       tinyint                            null comment '处理类型 0.不报修 1.自修 2.派工维修',
    role_id    bigint                             null comment '角色id',
    user_id    bigint                             null comment '用户id',
    version    int      default 0                 null comment '数据版本号',
    deleted    tinyint  default 0                 null comment '删除标记',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '报修处理人员表';

create index index_device_repair_handle_user_handle_id
    on device_repair_handle_user (handle_id);

create index index_device_repair_handle_user_role_id
    on device_repair_handle_user (role_id);

create index index_device_repair_handle_user_user_id
    on device_repair_handle_user (user_id);

create table if not exists device_repair_record
(
    id                   bigint auto_increment comment '主键'
        primary key,
    community_id         bigint                               not null comment '社区id',
    device_id            bigint                               not null comment '设备id',
    user_id              bigint                               not null comment '用户id',
    content              varchar(1024)                        not null comment '描述信息',
    repair_user_id       bigint                               null comment '当前维修人员',
    amount               decimal(16, 4)                       null comment '维保金额',
    file_urls            varchar(6000)                        null comment '文件urls 逗号,分割',
    current_in_date      datetime                             null on update CURRENT_TIMESTAMP comment '流程当前节点进入时间',
    current_overdue_date datetime                             null on update CURRENT_TIMESTAMP comment '流程当前节点逾期时间',
    is_once_overdue      tinyint(1) default 0                 null comment '曾经是否逾期 0: 否 1: 是',
    status               tinyint                              null comment '状态 0.待处理 1.已处理',
    version              int        default 0                 null comment '数据版本号',
    deleted              tinyint    default 0                 null comment '删除标记',
    ct_user_id           bigint                               null comment '创建人',
    ct_date              datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                               null comment '最后修改人',
    lm_date              datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备报修记录';

create table if not exists device_repair_handle
(
    id            bigint auto_increment comment '主键'
        primary key,
    record_id     bigint                               not null comment '设备报修记录id',
    user_id       bigint                               not null comment '提交人/处理人id',
    solve_time    datetime                             null comment '处理时间',
    file_urls     varchar(6000)                        null comment '文件urls 逗号,分割',
    status        tinyint                              null comment '状态 0.社区维修工 1.已完成',
    status_name   varchar(50)                          null comment '状态名称',
    overdue_count int                                  null comment '逾期次数',
    overdue_time  decimal(10, 1)                       null comment '逾期时间（小时）',
    overdue_date  datetime                             null on update CURRENT_TIMESTAMP comment '逾期时间',
    is_overdue    tinyint(1) default 0                 null comment '是否逾期 0: 否 1: 是',
    complete_time datetime                             null on update CURRENT_TIMESTAMP comment '完成时间',
    is_complete   tinyint(1) default 0                 null comment '是否完成 0: 否 1: 是',
    remark        varchar(2000)                        null comment '处理备注',
    version       int        default 0                 null comment '数据版本号',
    deleted       tinyint    default 0                 null comment '删除标记',
    ct_user_id    bigint                               null comment '创建人',
    ct_date       datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                               null comment '最后修改人',
    lm_date       datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备报修处理';

create index index_device_repair_handle_record_id
    on device_repair_handle (record_id);

create table if not exists device_location_history
(
    id         bigint auto_increment comment '主键'
        primary key,
    device_id  bigint                             not null comment '设备id',
    location   varchar(50)                        not null comment '设备位置',
    version    int      default 0                 null comment '数据版本号',
    deleted    tinyint  default 0                 null comment '删除标记',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备历史位置';

create table if not exists device_off_line
(
    id           bigint auto_increment comment '主键'
        primary key,
    community_id bigint                               not null comment '社区id',
    user_id      bigint                               not null comment '操作人id',
    file_name    varchar(50)                          not null comment '离线文件名',
    file_url     varchar(255)                         not null comment '离线文件url',
    status       tinyint(1) default 1                 null comment '是否上传 0: 否 1: 是',
    version      int        default 0                 null comment '数据版本号',
    deleted      tinyint    default 0                 null comment '删除标记',
    ct_user_id   bigint                               null comment '创建人',
    ct_date      datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                               null comment '最后修改人',
    lm_date      datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备离线';
