package com.anplus.admincontrol.annotation.user;

import com.anplus.admincontrol.mapper.sys.SysUserMapper;
import com.dao.model.sys.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;
import tk.mybatis.mapper.entity.Condition;

import java.util.List;

/**
 * @Author wangning
 * @Date 2019-10-21 15:12
 * @Email 1151089590@qq.com
 * @Description:
 */
@Component
public class CurrentUserResolver implements HandlerMethodArgumentResolver {

    @Autowired
    private SysUserMapper sysUserMapper;

    @Override
    public boolean supportsParameter(MethodParameter methodParameter) {
        //如果参数有CurrentUser注解则支持
        if (methodParameter.getParameterType().isAssignableFrom(SysUser.class) &&
                methodParameter.hasParameterAnnotation(CurrentUser.class)) {
            return true;
        }
        return false;
    }

    @Override
    public Object resolveArgument(MethodParameter methodParameter, ModelAndViewContainer modelAndViewContainer, NativeWebRequest nativeWebRequest, WebDataBinderFactory webDataBinderFactory) throws Exception {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        Condition condition = new Condition(SysUser.class);
        condition.createCriteria().andEqualTo("username", username);
        List<SysUser> userList = sysUserMapper.selectByCondition(condition);
        return userList.size() > 0 ? userList.get(0) : null;
    }
}
