package com.anplus.admincontrol.common.service;

import com.anplus.admincontrol.common.dto.SysMenuTree;
import com.anplus.admincontrol.controller.sys.vo.SysUserLoginInfoVO;
import com.anplus.admincontrol.mapper.sys.SysBusinessMapper;
import com.anplus.admincontrol.mapper.sys.SysBusinessOperateMapper;
import com.dao.model.sys.SysBusiness;
import com.dao.model.sys.SysBusinessOperate;
import com.dao.model.sys.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author wangning
 * @Date 2019-10-09 17:08
 * @Email 1151089590@qq.com
 * @Description:
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class MenuService {

    @Autowired
    private SysBusinessMapper sysBusinessMapper;

    @Autowired
    private SysBusinessOperateMapper sysBusinessOperateMapper;

    public SysUserLoginInfoVO getUserLoginInfo(SysUser sysUser) {

        sysUser.setPassword(null);
        SysUserLoginInfoVO apiRespUserLoginInfo = new SysUserLoginInfoVO();
        apiRespUserLoginInfo.setSysUser(sysUser);

        // 总控平台 1号用户（admin用户） 查询全部菜单和请求资源
        if (sysUser.getId() == 1) {
            List<SysBusiness> businessList = sysBusinessMapper.getAllBusiness();
            apiRespUserLoginInfo.setOptResList(getBusOptByBusinessIds(businessList));
            apiRespUserLoginInfo.setMenuList(getAllMenu(businessList));
        } else {
            // 查询用户拥有的操作资源
            apiRespUserLoginInfo.setOptResList(getBusOptByUserId(sysUser.getId()));
            apiRespUserLoginInfo.setMenuList(getUserMenu(sysUser.getId()));
        }

        return apiRespUserLoginInfo;

    }

    public List<SysMenuTree> getAllMenu(List<SysBusiness> businessList) {
        List<SysMenuTree> menuList = new ArrayList<>();
        Map<Long, SysMenuTree> menuMap = new HashMap<>();

        for (SysBusiness item : businessList) {
            menuMap.put(item.getId(), new SysMenuTree(item));
        }

        for (SysBusiness item : businessList) {
            if (menuMap.containsKey(item.getParentId())) {
                SysMenuTree sysMenuTree = menuMap.get(item.getParentId());
                if (sysMenuTree.getChildren() == null) {
                    sysMenuTree.setChildren(new ArrayList<>());
                }
                sysMenuTree.getChildren().add(menuMap.get(item.getId()));
            } else {
                menuList.add(menuMap.get(item.getId()));
            }
        }

        return menuList;
    }

    public List<SysBusinessOperate> getBusOptByUserId(Long userId) {
        return sysBusinessOperateMapper.getBusOptByUserId(userId);
    }

    /**
     * 根据用户查询菜单
     *
     * @param userId
     * @return
     */
    public List<SysMenuTree> getUserMenu(Long userId) {

        List<SysMenuTree> menuVoList = new ArrayList<>();
        List<SysBusiness> businessList = sysBusinessMapper.getUserMenu(userId);
        if (businessList != null && businessList.size() > 0) {

            // 查询所有菜单目录
            List<SysBusiness> allBusinessList = sysBusinessMapper.selectAll();
            Map<Long, SysBusiness> allBusinessMap = new HashMap<>();
            for (SysBusiness item : allBusinessList) {
                allBusinessMap.put(item.getId(), item);
            }

            // 遍历查询所有父节点
            Map<Long, SysBusiness> resultMenuMap = new HashMap<>();
            for (SysBusiness item : businessList) {
                pickMenuList(item, allBusinessMap, resultMenuMap);
            }

            // 构建Vo-Map
            Map<Long, SysMenuTree> menuMap = new HashMap<>();
            List<SysBusiness> busList = new ArrayList<>();
            for (SysBusiness item : resultMenuMap.values()) {
                menuMap.put(item.getId(), new SysMenuTree(item));
                busList.add(item);
            }
            // 排序
            sortListByShowOrder(busList);

            // 构建树
            for (SysBusiness item : busList) {
                if (menuMap.containsKey(item.getParentId())) {
                    SysMenuTree sysMenuTree = menuMap.get(item.getParentId());
                    if (sysMenuTree.getChildren() == null) {
                        sysMenuTree.setChildren(new ArrayList<>());
                    }
                    sysMenuTree.getChildren().add(menuMap.get(item.getId()));
                } else {
                    menuVoList.add(menuMap.get(item.getId()));
                }
            }

            return menuVoList;
        }
        return null;
    }

    /**
     * 查询业务下全部操作
     *
     * @param businessList
     * @author wangning
     * @date 2019-11-15 10:07
     */
    private List<SysBusinessOperate> getBusOptByBusinessIds(List<SysBusiness> businessList) {
        return sysBusinessOperateMapper.getBusOptByBusinessIds(businessList);
    }

    /**
     * 递归查找父菜单
     *
     * @param item
     * @param allMenuMap
     * @param resultMenuMap
     */
    private void pickMenuList(SysBusiness item, Map<Long, SysBusiness> allMenuMap, Map<Long, SysBusiness> resultMenuMap) {
        if (item.getParentId() != null) {
            if (!resultMenuMap.containsKey(item.getId())) {
                if (allMenuMap.containsKey(item.getParentId())) {
                    resultMenuMap.put(item.getId(), item);
                    pickMenuList(allMenuMap.get(item.getParentId()), allMenuMap, resultMenuMap);
                } else {
                    resultMenuMap.put(item.getId(), item);
                }
            }
        } else {
            if (!resultMenuMap.containsKey(item.getId())) {
                resultMenuMap.put(item.getId(), item);
            }
        }
    }

    /**
     * 按照showOrder字段排序
     *
     * @param list
     */
    private void sortListByShowOrder(List<SysBusiness> list) {
        list.sort((o1, o2) -> {
            if (o1.getShowOrder() == null) {
                return -1;
            }
            if (o2.getShowOrder() == null) {
                return 1;
            }
            // 升序
            if (o1.getShowOrder() > o2.getShowOrder()) {
                return 1;
            }
            if (o1.getShowOrder().equals(o2.getShowOrder())) {
                return 0;
            }
            return -1;
        });
    }

}
