package com.anplus.admincontrol.controller.holidays;

import com.anplus.admincontrol.annotation.user.CurrentUser;
import com.anplus.admincontrol.controller.holidays.dto.AddHolidaysDTO;
import com.anplus.admincontrol.controller.holidays.dto.HolidaysPageDTO;
import com.anplus.admincontrol.service.holidays.HolidaysService;
import com.core.result.Result;
import com.core.result.ResultGenerator;
import com.dao.common.page.PageInfoVO;
import com.dao.model.holidays.Holidays;
import com.github.pagehelper.PageHelper;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.List;

@RestController
@RequestMapping("/holidays")
public class HolidaysController {

    @Resource
    HolidaysService holidaysService;

    /**
     * 分页查询列表
     *
     * @param param 分页参数
     * @return 分页数据
     */
    @PostMapping("/page")
    public Result pageList(@RequestBody HolidaysPageDTO param) {
        PageHelper.startPage(param.getPage(), param.getSize());
        List<Holidays> list = holidaysService.getPage(param);
        PageInfoVO pageInfo = new PageInfoVO(list);
        return ResultGenerator.success(pageInfo, "获取分页列表");
    }

    @PostMapping("/synchronousVacation")
    public Result synchronousVacation(@RequestBody HolidaysPageDTO param, @CurrentUser Long userId) throws ParseException {
        holidaysService.synchronousVacation(param,userId);
        PageHelper.startPage(param.getPage(), param.getSize());
        List<Holidays> list = holidaysService.getPage(param);
        PageInfoVO pageInfo = new PageInfoVO(list);
        return ResultGenerator.success(pageInfo, "获取分页列表");
    }

    @DeleteMapping("/{id}")
    public Result deleteById(@PathVariable Long id) {
        holidaysService.deleteById(id);
        return ResultGenerator.success("删除成功");
    }

    @PostMapping("/batchDelete")
    public Result deleteBatch(@RequestBody List<Long> ids) {
        holidaysService.deleteByIds(ids);
        return ResultGenerator.success("批量删除") ;
    }

    @PostMapping("/add")
    public Result addAndUpdateHoliday(@RequestBody Holidays holidays,@CurrentUser Long userId){
        holidaysService.addAndUpdateHoliday(holidays,userId);
        return ResultGenerator.success("新增成功");
    }
}
