package com.anplus.admincontrol.controller.sys;

import com.anplus.admincontrol.common.dto.SysBusinessTree;
import com.anplus.admincontrol.service.sys.SysBusinessService;
import com.core.result.Result;
import com.core.result.ResultGenerator;
import com.dao.common.page.PageInfoVO;
import com.dao.common.page.PageParams;
import com.dao.model.sys.SysBusiness;
import com.github.pagehelper.PageHelper;
import lombok.NonNull;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Positive;
import java.util.List;

/**
 * @author wangning
 * @date 2019-10-11 09:02
 * SysBusinessController控制类
 */
@RestController
@RequestMapping("/sys/business")
@Validated
public class SysBusinessController {

    @Resource
    private SysBusinessService sysBusinessService;

    @GetMapping("/")
    public Result getAllList() {
        List<SysBusinessTree> busList = sysBusinessService.getAllList();
        return ResultGenerator.success(busList, "获取全部菜单列表");
    }

    /**
     * 分页查询列表
     *
     * @param page 分页参数
     * @return 分页数据
     */
    @GetMapping("/list")
    public Result pageList(@Validated PageParams page) {
        PageHelper.startPage(page.getPage(), page.getSize());
        List<SysBusiness> list = sysBusinessService.selectAll();
        PageInfoVO pageInfo = new PageInfoVO(list);
        return ResultGenerator.success(pageInfo, "获取分页列表");
    }

    /**
     * 根据id查询
     *
     * @param id 主键
     * @return 详情数据
     */
    @GetMapping("/{id}")
    public Result findById(@PathVariable @Positive Integer id) {
        SysBusiness sysBusiness = sysBusinessService.selectByPrimaryKey(id);
        return ResultGenerator.success(sysBusiness, "根据id获取详情");
    }

    /**
     * 新增保存对象
     *
     * @param sysBusiness 保存对象
     */
    @PostMapping
    public Result save(@RequestBody @NonNull @Validated SysBusiness sysBusiness) {
        sysBusinessService.insertSelective(sysBusiness);
        return ResultGenerator.success(sysBusiness, "新增成功");
    }

    /**
     * 更新修改对象
     *
     * @param sysBusiness 修改对象
     */
    @PutMapping
    public Result update(@RequestBody @NonNull @Validated SysBusiness sysBusiness) {
        sysBusinessService.updateByPrimaryKeySelective(sysBusiness);
        return ResultGenerator.success(sysBusiness, "更新成功");
    }

    /**
     * 根据id删除
     *
     * @param id 主键
     */
    @DeleteMapping("/{id}")
    public Result deleteById(@PathVariable @Positive Integer id) {
        sysBusinessService.deleteByPrimaryKey(id);
        return ResultGenerator.success("删除成功");
    }

}
