package com.anplus.admincontrol.controller.sys;

import com.anplus.admincontrol.annotation.user.CurrentUser;
import com.anplus.admincontrol.controller.sys.dto.SysCodePageDTO;
import com.anplus.admincontrol.controller.sys.dto.SysCodeSaveDTO;
import com.anplus.admincontrol.service.sys.SysCodeTypeService;
import com.core.result.Result;
import com.core.result.ResultGenerator;
import com.dao.common.page.PageInfoVO;
import com.dao.model.sys.SysCodeType;
import com.dao.model.sys.SysUser;
import com.github.pagehelper.PageHelper;
import lombok.NonNull;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author linxianghua
 * @date 2019-10-11 17:05
 * SysCodeTypeController控制类
 */
@RestController
@RequestMapping("/sys/code/type")
@Validated
public class SysCodeTypeController {

    @Resource
    private SysCodeTypeService sysCodeTypeService;

    /**
     * 分页查询列表
     *
     * @param param 分页参数
     * @return 分页数据
     */
    @PostMapping("/page")
    public Result pageList(@RequestBody SysCodePageDTO param) {
        PageHelper.startPage(param.getPage(), param.getSize());
        List<SysCodeType> list = sysCodeTypeService.getPage(param);
        PageInfoVO pageInfo = new PageInfoVO(list);
        return ResultGenerator.success(pageInfo, "获取分页列表");
    }

    /**
     * 根据id查询
     *
     * @param id 主键
     * @return 详情数据
     */
    @GetMapping("/{id}")
    public Result findById(@PathVariable Integer id) {
        return ResultGenerator.success(sysCodeTypeService.getDetail(id), "根据id获取详情");
    }

    /**
     * 保存对象
     *
     * @param param 保存对象
     */
    @PostMapping
    public Result save(@RequestBody SysCodeSaveDTO param, @CurrentUser SysUser sysUser) {
        return ResultGenerator.success(sysCodeTypeService.saveCode(param, sysUser), "保存通用代码");
    }

    /**
     * 根据code删除
     *
     * @param code
     */
    @DeleteMapping("/{code}")
    public Result deleteByCode(@PathVariable String code) {
        sysCodeTypeService.deleteByCodes(code);
        return ResultGenerator.success("删除成功");
    }

    @RequestMapping(value = "/batchDelete", method = RequestMethod.POST)
    public Result deleteBatch(@RequestBody String[] codes) {
        sysCodeTypeService.deleteByCodes(codes);
        return ResultGenerator.success("批量删除");
    }

    /**
     * 查询通用代码
     *
     * @param types
     * @return com.core.result.Result
     * @author wangning
     * @date 2019-11-12 10:54
     */
    @PostMapping("/code")
    public Result getCommonCode(@RequestBody @NonNull @Validated String[] types) {
        return ResultGenerator.success(sysCodeTypeService.getCodeByTypes(types), "查询成功");
    }

}
