package com.anplus.admincontrol.controller.task;

import com.anplus.admincontrol.controller.task.dto.ZtTaskEstimatePageDTO;
import com.anplus.admincontrol.controller.task.vo.AnnualConsumeData;
import com.anplus.admincontrol.service.task.ZtTaskestimateService;
import com.core.result.Result;
import lombok.NonNull;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.List;

import static com.core.result.ResultGenerator.success;

/**
 * @author 商天翔
 * @date 2021-03-02 17:28
 * ZtTaskestimateController控制类
 */
@RestController
@RequestMapping("/task-estimate")
@Validated
public class ZtTaskestimateController {

    @Resource
    private ZtTaskestimateService ztTaskestimateService;

    /**
     * 分页查询工时列表
     *
     * @param param 分页参数
     * @return 分页数据
     */
    @PostMapping("/consumed-page")
    public Result getConsumedPage(@RequestBody @NonNull @Validated ZtTaskEstimatePageDTO param) {
        return success(ztTaskestimateService.getConsumedPage(param), "获取分页列表");
    }

    /**
     * 导出工时列表
     * @param param 分页参数同分页列表接口
     */
    @PostMapping("/export")
    public void export(@RequestBody @NonNull @Validated ZtTaskEstimatePageDTO param,
                       HttpServletRequest request,
                       HttpServletResponse response) {
        ztTaskestimateService.exportExcel(param, request, response);
    }

    /**
     * 辅助信息，包括节假日和标准日工时
     * @param month
     * @return
     */
    @GetMapping("/auxiliary-info/{month}")
    public Result getAuxiliaryInfo(@PathVariable("month") String month) {
        return success(ztTaskestimateService.getAuxiliaryInfo(month));
    }

    @GetMapping("/annual-consume/{year}")
    public Result<List> getAnnualConsumeData(@PathVariable Integer year) {
        return success(ztTaskestimateService.getAnnualConsumeData(year));
    }


}
