package com.anplus.admincontrol.controller.zt;

import com.dao.model.zt.ZtUserWages;
import com.anplus.admincontrol.service.zt.ZtUserWagesService;

import com.core.result.Result;
import com.dao.common.page.PageParams;
import lombok.NonNull;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Positive;

import static com.core.result.ResultGenerator.success;
import static com.core.result.ResultGenerator.fail;

/**
 * @author 孟令君
 * @date 2021-03-02 16:46
 * ZtUserWagesController控制类
 */
@RestController
@RequestMapping("/zt/user/wages")
@Validated
public class ZtUserWagesController {

    @Resource
    private ZtUserWagesService ztUserWagesService;

    /**
     * 分页查询列表
     *
     * @param param 分页参数
     * @return 分页数据
     */
    @PostMapping("/page")
    public Result getPage(@RequestBody @NonNull @Validated PageParams param) {
        return success(ztUserWagesService.getPage(param), "获取分页列表");
    }

    /**
     * 根据id查询
     *
     * @param id 主键
     * @return 详情数据
     */
    @GetMapping("/{id}")
    public Result getDetails(@PathVariable @Positive Long id) {
        return success(ztUserWagesService.selectByPrimaryKey(id), "根据id获取详情");
    }

    /**
     * 新增保存对象
     *
     * @param ztUserWages 保存对象
     */
    @PostMapping
    public Result save(@RequestBody @NonNull @Validated ZtUserWages ztUserWages) {
        return success(ztUserWagesService.insertSelective(ztUserWages), "新增成功");
    }

    /**
     * 更新修改对象
     *
     * @param ztUserWages 修改对象
     */
    @PutMapping
    public Result update(@RequestBody @NonNull @Validated ZtUserWages ztUserWages) {
        return success(ztUserWagesService.updateByPrimaryKeySelective(ztUserWages), "更新成功");
    }

    /**
     * 根据id删除
     *
     * @param id 主键
     */
    @DeleteMapping("/{id}")
    public Result deleteById(@PathVariable @Positive Long id) {
        return success(ztUserWagesService.deleteByPrimaryKey(id), "删除成功");
    }

}
