package com.anplus.admincontrol.security.authorize;

import com.anplus.admincontrol.service.sys.SysBusinessOperateResourceService;
import com.dao.model.sys.SysBusinessOperateResource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.AccessDecisionManager;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Collection;
import java.util.List;

/**
 * @Author wangning
 * @Date 2019/10/05 15:08
 * @Email 1151089590@qq.com
 * @Description:
 */
@Service
public class CustomAccessDecisionManager implements AccessDecisionManager {

    @Autowired
    private SysBusinessOperateResourceService sysBusinessOperateResourceService;

    @Override
    public void decide(Authentication authentication, Object o, Collection<ConfigAttribute> collection) throws AccessDeniedException, InsufficientAuthenticationException {
        HttpServletRequest request = ((FilterInvocation) o).getHttpRequest();
        List<SysBusinessOperateResource> resourceList = sysBusinessOperateResourceService.getResourceListByUserName(authentication.getName());
        String method;
        AntPathRequestMatcher matcher;
        if (resourceList != null) {
            for (SysBusinessOperateResource item : resourceList) {
                matcher = new AntPathRequestMatcher("/api" + item.getOperateUrl());
                method = item.getOperateMethod() == null ? "ALL" : item.getOperateMethod().toUpperCase();
                if (matcher.matches(request)) {
                    //当权限表权限的method为ALL时表示拥有此路径的所有请求方式权利
                    if (method.equals(request.getMethod()) || "ALL".equals(method)) {
                        return;
                    }
                }
            }
        }

        throw new AccessDeniedException("access denied");
    }

    @Override
    public boolean supports(ConfigAttribute configAttribute) {
        return true;
    }

    @Override
    public boolean supports(Class<?> aClass) {
        return true;
    }
}
