package com.anplus.admincontrol.security.authorize;


import com.anplus.admincontrol.mapper.sys.SysBusinessOperateResourceMapper;
import com.dao.model.sys.SysBusinessOperateResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.access.SecurityConfig;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.access.intercept.FilterInvocationSecurityMetadataSource;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @Author wangning
 * @Date 2019/10/05 15:08
 * @Email 1151089590@qq.com
 * @Description:
 */
@Service
public class CustomMetadataSourceService implements FilterInvocationSecurityMetadataSource {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private SysBusinessOperateResourceMapper resourceMapper;

    private Map<String, Map<String, Collection<ConfigAttribute>>> filterUrlMap = null;

    @Override
    public Collection<ConfigAttribute> getAttributes(Object o) throws IllegalArgumentException {
        if (this.filterUrlMap == null) {
            this.filterUrlMap = initFilterUrl();
        }
        HttpServletRequest request = ((FilterInvocation) o).getHttpRequest();
        Map<String, Collection<ConfigAttribute>> methodFilterUrlMap = filterUrlMap.get(request.getMethod().toUpperCase());
        if (methodFilterUrlMap != null) {
            for (String url : methodFilterUrlMap.keySet()) {
                    AntPathRequestMatcher matcher = new AntPathRequestMatcher(url);
                if (matcher.matches(request)) {
                    logger.info(String.format("请求命中过滤清单URL=%s,METHOD=%s", request.getRequestURL().toString(), request.getMethod()));
                    return methodFilterUrlMap.get(url);
                }
            }
        }
        logger.info(String.format("请求未命中过滤清单URL=%s,METHOD=%s", request.getRequestURL().toString(), request.getMethod()));
        return null;
    }

    @Override
    public Collection<ConfigAttribute> getAllConfigAttributes() {
        return null;
    }

    @Override
    public boolean supports(Class<?> aClass) {
        return true;
    }

    /**
     * 加载权限表中所有权限
     */
    private Map<String, Map<String, Collection<ConfigAttribute>>> initFilterUrl() {
        logger.info("初始化请求过滤清单");
        List<SysBusinessOperateResource> operatorList = resourceMapper.selectAll();
        Map<String, Map<String, Collection<ConfigAttribute>>> configAttrMap = new HashMap<>(10);
        for (SysBusinessOperateResource item : operatorList) {
            if (!StringUtils.isEmpty(item.getOperateUrl())) {
                String method = item.getOperateMethod() == null ? "ALL" : item.getOperateMethod().toUpperCase();
                if (!configAttrMap.containsKey(method)) {
                    configAttrMap.put(method, new HashMap<>(50));
                }
                Collection<ConfigAttribute> configAttr = new ArrayList<>();
                configAttr.add(new SecurityConfig("null"));
                configAttrMap.get(method).put("/api" + item.getOperateUrl(), configAttr);
                logger.info(String.format("业务操作ID=%d,URL=%s,METHOD=%s 加入到过滤清单", item.getId(), item.getOperateUrl(), method));
            } else {
                logger.warn(String.format("业务资源ID=%d,没有配置URI不能加入资源过滤池",item.getId()));
            }
        }
        return configAttrMap;
    }
}
