package com.anplus.admincontrol.service.project;

import com.anplus.admincontrol.controller.project.dto.ProjectChargePageDTO;
import com.anplus.admincontrol.controller.project.dto.ProjectChargeSaveDTO;
import com.anplus.admincontrol.controller.project.vo.ProjectChargePageVO;
import com.dao.model.project.ProjectCharge;
import com.dao.mapper.project.ProjectChargeMapper;
import com.anplus.admincontrol.mapper.project.ProjectChargeBusinessMapper;

import com.dao.common.AbstractMapperService;
import com.dao.common.page.PageInfoVO;
import com.github.pagehelper.PageHelper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author 孟令君
 * @date 2021-03-04 10:53
 * ProjectChargeService服务类
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ProjectChargeService extends AbstractMapperService<ProjectCharge> {

    @Resource
    private ProjectChargeMapper projectChargeMapper;
    @Resource
    private ProjectChargeBusinessMapper projectChargeBusinessMapper;

    /**
     * 分页查询列表
     */
    public PageInfoVO getPage(ProjectChargePageDTO param) {
        PageHelper.startPage(param.getPage(), param.getSize());
        List<ProjectChargePageVO> list = projectChargeBusinessMapper.getPage(param);
        return new PageInfoVO(list);
    }

    public Byte updateStatistics(Long id) {
        ProjectCharge projectCharge = projectChargeBusinessMapper.findByProjectId(id);

        if (Objects.isNull(projectCharge)){
            projectCharge = ProjectCharge.builder()
                    .projectId(id)
                    .projectCharge(BigDecimal.valueOf(0))
                    .statisticsStatus((byte) 0)
                    .ctDate(new Date())
                    .build();
            projectChargeMapper.insertSelective(projectCharge);
        }else {
            projectCharge.setStatisticsStatus((byte) (projectCharge.getStatisticsStatus()==null?0:projectCharge.getStatisticsStatus()==1?0:1));
            projectChargeMapper.updateByPrimaryKeySelective(projectCharge);
        }

        return projectCharge.getStatisticsStatus();
    }

    public Long savProjectCharge(ProjectChargeSaveDTO dto) {
        ProjectCharge byProjectId = projectChargeBusinessMapper.findByProjectId(dto.getId());
        if (Objects.isNull(byProjectId)){
            byProjectId = ProjectCharge.builder()
                    .projectId(dto.getId())
                    .projectCharge(dto.getCharge())
                    .statisticsStatus((byte) 1)
                    .ctDate(new Date())
                    .build();
            projectChargeMapper.insertSelective(byProjectId);
        }else {
            byProjectId.setProjectCharge(dto.getCharge());
            byProjectId.setLmDate(new Date());
            projectChargeMapper.updateByPrimaryKeySelective(byProjectId);
        }
        return byProjectId.getId();
    }
}
