package com.anplus.admincontrol.service.report;

import com.anplus.admincontrol.controller.report.dto.ProjectCostReportDTO;
import com.anplus.admincontrol.controller.report.dto.StaffDailyReportDTO;
import com.anplus.admincontrol.controller.report.vo.ProjectCostReportVO;
import com.anplus.admincontrol.controller.report.vo.StaffDailyReportVO;
import com.anplus.admincontrol.controller.report.vo.UserInfoVO;
import com.anplus.admincontrol.mapper.report.ReportMapper;
import com.dao.mapper.zt.ZtUserWagesMapper;
import com.dao.model.zt.ZtUserWages;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class ReportService {
    @Resource
    private ReportMapper reportMapper;

    public List<StaffDailyReportVO> getStaffDailyReport(StaffDailyReportDTO params) {
        return reportMapper.getStaffDailyReport(params, "<br>");
    }

    public List<ProjectCostReportVO> getProjectCostReport(ProjectCostReportDTO params) {
        List<ProjectCostReportVO> result = new ArrayList<>();
        List<ProjectCostReportVO> baseList = reportMapper.getProjectCostReport(params);
        //获取最新年假期数量
        BigDecimal yearWorkHours = reportMapper.getYearWorkHours();
        //获取人员薪水
        List<UserInfoVO> userInfoList = reportMapper.getUserInfo();
        Map<Long,UserInfoVO> userInfoMap = new HashMap<>(userInfoList.size());
        for (UserInfoVO item : userInfoList) {
            userInfoMap.put(item.getId(),item);
            if(item.getWages() != null && item.getWages().compareTo(BigDecimal.ZERO) != 0){
                item.setBaseHoursCost(item.getWages().divide(yearWorkHours,1, RoundingMode.HALF_UP));
            }else{
                item.setBaseHoursCost(BigDecimal.ZERO);
            }
        }

        for (ProjectCostReportVO item : baseList) {
            if(result.size()==0 || result.get(result.size()-1).getProjectId() != item.getProjectId() ){
                result.add(ProjectCostReportVO.builder().
                        key(item.getProjectId().toString()).projectId(item.getProjectId()).name(item.getProjectName())
                        .hours(BigDecimal.ZERO).baseHoursCost(BigDecimal.ZERO).floatHoursCost(BigDecimal.ZERO)
                        .children(new ArrayList<>()).build());
            }
            ProjectCostReportVO project = result.get(result.size()-1);
            item.setKey(item.getProjectId() + "-" + item.getUserId());
            item.setName(item.getUserName());
            if(userInfoMap.containsKey(item.getUserId())){
                item.setBaseHoursCost(userInfoMap.get(item.getUserId()).getBaseHoursCost().multiply(item.getHours()).setScale(0,RoundingMode.HALF_UP));
            }else{
                item.setBaseHoursCost(BigDecimal.ZERO);
            }
            project.setBaseHoursCost(project.getBaseHoursCost().add(item.getBaseHoursCost()));
            project.getChildren().add(item);
            if(item.getHours()!=null){
                project.setHours(project.getHours().add(item.getHours()));
            }

        }
        return result;
    }
}
