package com.anplus.admincontrol.service.sys;

import com.anplus.admincontrol.mapper.sys.SysCodeDetailMapper;
import com.common.util.ArrayUtils;
import com.dao.common.AbstractMapperService;
import com.dao.model.sys.SysCodeDetail;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author linxianghua
 * @date 2019-10-11 17:05
 * SysCodeDetailService服务类
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SysCodeDetailService extends AbstractMapperService<SysCodeDetail> {

    @Resource
    private SysCodeDetailMapper sysCodeDetailMapper;

    public void updateTypeCode(String newCode, String oldCode) {
        sysCodeDetailMapper.updateTypeCode(newCode, oldCode);
    }

    /**
     * 获取通用代码详情列表
     *
     * @param code
     * @return
     */
    public List<SysCodeDetail> getCodeDetailList(String code) {
        if (Objects.isNull(code)) {
            return null;
        }
        return sysCodeDetailMapper.getSysCodeDetailList(code);
    }

    /**
     * 获取通用代码
     *
     * @param codeType
     * @return
     */
    public Map<String, List<SysCodeDetail>> getCodeByTypes(String... codeType) {
        if (ArrayUtils.isEmpty(codeType)) {
            return null;
        }
        List<SysCodeDetail> list = new ArrayList();
        for (String code : codeType) {
            List<SysCodeDetail> detailList = this.getCodeDetailList(code);
            if (ArrayUtils.isEmpty(detailList)) {
                continue;
            }
            list.addAll(detailList);
        }
        return this.process(list);
    }

    /**
     * 设置通用代码映射
     *
     * @param detailList
     * @return
     */
    private Map<String, List<SysCodeDetail>> process(List<SysCodeDetail> detailList) {
        Map<String, List<SysCodeDetail>> resultMap = null;
        if (!ArrayUtils.isEmpty(detailList)) {
            resultMap = new HashMap<>(detailList.size());
            for (SysCodeDetail item : detailList) {
                String typeCode = item.getTypeCode();
                if (!resultMap.containsKey(typeCode)) {
                    resultMap.put(typeCode, new ArrayList<>());
                }
                resultMap.get(typeCode).add(item);
            }
        }
        return resultMap;
    }

}
