package com.anplus.admincontrol.service.sys;

import com.anplus.admincontrol.constant.ErrorConstant;
import com.anplus.admincontrol.controller.sys.dto.SysCodePageDTO;
import com.anplus.admincontrol.controller.sys.dto.SysCodeSaveDTO;
import com.anplus.admincontrol.controller.sys.vo.SysCodeTypeDetailVO;
import com.anplus.admincontrol.mapper.sys.SysCodeTypeMapper;
import com.common.business.CommonSysCodeService;
import com.common.redis.RedisUtils;
import com.common.util.CommonUtils;
import com.constants.CodeConstants;
import com.constants.RedisKeys;
import com.core.exception.ServiceException;
import com.core.result.Result;
import com.dao.common.AbstractMapperService;
import com.dao.model.sys.SysCodeDetail;
import com.dao.model.sys.SysCodeType;
import com.dao.model.sys.SysUser;
import com.dao.util.EntityUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * @author linxianghua
 * @date 2019-10-11 17:05
 * SysCodeTypeService服务类
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SysCodeTypeService extends AbstractMapperService<SysCodeType> {

    @Resource
    private SysCodeTypeMapper sysCodeTypeMapper;

    @Resource
    private SysCodeDetailService sysCodeDetailService;

    @Resource
    private CommonSysCodeService commonSysCodeService;

    @Resource
    private RedisUtils redisUtils;

    /**
     * 分页查询
     *
     * @param sysCodePageDTO
     * @return com.anplus.admin.dto.SysCodePageDTO
     * @author linxianghua
     * @date 2019/10/12 10:39
     */
    public List<SysCodeType> getPage(SysCodePageDTO sysCodePageDTO) {
        return sysCodeTypeMapper.getPage(sysCodePageDTO);
    }

    /**
     * 分页查询
     *
     * @param id
     * @return com.anplus.admin.dto.SysCodePageDTO
     * @author linxianghua
     * @date 2019/10/12 10:39
     */
    public SysCodeTypeDetailVO getDetail(Integer id) {
        SysCodeTypeDetailVO dto = new SysCodeTypeDetailVO();
        dto.setCodeMap(sysCodeDetailService.getCodeByTypes(CodeConstants.SYS_CODE_TYPE));
        if (CommonUtils.isZero(id)) {
            dto.setSysCodeType(new SysCodeType());
        } else {
            SysCodeType sysCodeType = selectByPrimaryKey(id);
            dto.setSysCodeType(sysCodeType);
            dto.setCodeDetailList(sysCodeDetailService.getCodeDetailList(sysCodeType.getCode()));
        }
        return dto;
    }

    /**
     * @param param
     * @return java.lang.Integer
     * @author linxianghua
     * @date 2019/10/12 14:50
     */
    public Long saveCode(SysCodeSaveDTO param, SysUser sysUser) {
        SysCodeType sysCodeType = param.getSysCodeType();
        if (sysCodeTypeMapper.getRepeat(CommonUtils.toLong(sysCodeType.getId()), sysCodeType.getCode()) > 0) {
            throw new ServiceException(ErrorConstant.DUPLICATE_CODE);
        }

        //根据数据有无id判断是新建还是修改
        SysCodeType type = null;
        if (CommonUtils.isZero(sysCodeType.getId())) {
            sysCodeType.setCtUserId(sysUser.getId());
            insertSelective(sysCodeType);

        } else {
            type = this.selectByPrimaryKey(sysCodeType.getId());
            if (!type.getCode().equals(sysCodeType.getCode())) {
                sysCodeDetailService.updateTypeCode(sysCodeType.getCode(), type.getCode());
            }

            EntityUtils.setModifyBase(sysCodeType, sysUser.getId());
            updateByPrimaryKeySelective(sysCodeType);
        }

        //保存明细新增数据
        if (!CommonUtils.isEmpty(param.getInsertList())) {
            for (SysCodeDetail item : param.getInsertList()) {
                item.setId(null);
                item.setCtUserId(sysUser.getId());
                item.setTypeCode(sysCodeType.getCode());
                sysCodeDetailService.insertSelective(item);
            }
        }

        //保存明细修改数据
        if (!CommonUtils.isEmpty(param.getUpdateList())) {
            for (SysCodeDetail item : param.getUpdateList()) {
                EntityUtils.setModifyBase(item, sysUser.getId());
                item.setTypeCode(sysCodeType.getCode());
                sysCodeDetailService.updateByPrimaryKeySelective(item);
            }
        }

        //删除数据
        if (!CommonUtils.isEmpty(param.getDelIdList())) {
            sysCodeDetailService.deleteByIds(CommonUtils.integerListToStr(param.getDelIdList()));
        }

        //清缓存
        if (null != type) {
            this.cleanCache(type.getCode(), sysCodeType.getCode());
        } else {
            this.cleanCache(sysCodeType.getCode());
        }
        return sysCodeType.getId();
    }

    /**
     * 获取通用代码
     *
     * @param codeType
     * @return com.core.result.Result
     * @author wangning
     * @date 2019-10-14 14:15
     */
    public Result getCodeByTypes(String... codeType) {
        Result result = new Result();
        result.setData(commonSysCodeService.getCodeByTypes(codeType));
        return result;
    }

    /**
     * 根据code删除
     *
     * @param codes
     */
    public void deleteByCodes(String... codes) {
        Condition condition = new Condition(SysCodeType.class);
        condition.createCriteria().andIn("code", Arrays.asList(codes));
        this.deleteByCondition(condition);
        //清缓存
        this.cleanCache(codes);
    }

    /**
     * 清通用代码缓存
     *
     * @param codes
     */
    private void cleanCache(String... codes) {
        for (String code : codes) {
            redisUtils.remove(StringUtils.join(RedisKeys.SYS_CODE_DIR, "::", code));
        }
    }
}
