package com.anplus.admincontrol.service.sys;

import com.anplus.admincontrol.common.dto.SysBusinessTree;
import com.anplus.admincontrol.constant.ErrorConstant;
import com.anplus.admincontrol.controller.sys.dto.SysRolePageDTO;
import com.anplus.admincontrol.controller.sys.dto.SysRoleSaveDTO;
import com.anplus.admincontrol.controller.sys.vo.SysRoleDetailVO;
import com.anplus.admincontrol.mapper.sys.SysBusinessMapper;
import com.anplus.admincontrol.mapper.sys.SysRoleMapper;
import com.anplus.admincontrol.mapper.sys.SysRoleOperateMapper;
import com.common.business.CommonSysCodeService;
import com.common.util.CommonUtils;
import com.constants.CodeConstants;
import com.core.exception.ServiceException;
import com.dao.common.AbstractMapperService;
import com.dao.model.sys.*;
import com.dao.util.EntityUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangning
 * @date 2019-10-11 09:02
 * SysRoleService服务类
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SysRoleService extends AbstractMapperService<SysRole> {

    @Resource
    private SysRoleMapper sysRoleMapper;
    @Resource
    private SysBusinessMapper sysBusinessMapper;
    @Resource
    private SysBusinessOperateService sysBusinessOperateService;
    @Resource
    private SysRoleOperateMapper sysRoleOperateMapper;
    @Resource
    private SysRoleOperateService sysRoleOperateService;
    @Resource
    private CommonSysCodeService commonSysCodeService;

    static void getMenuList(List<SysBusinessTree> menuList, Map<Long, SysBusinessTree> menuMap, List<SysBusiness> businessList) {
        for (SysBusiness item : businessList) {
            if (menuMap.containsKey(item.getParentId())) {
                SysBusinessTree voMenu = menuMap.get(item.getParentId());
                if (voMenu.getChildren() == null) {
                    voMenu.setChildren(new ArrayList<>());
                }
                voMenu.getChildren().add(menuMap.get(item.getId()));
            } else {
                menuList.add(menuMap.get(item.getId()));
            }
        }
    }

    /**
     * 分页查询
     *
     * @param sysRolePageDTO
     * @return com.anplus.admin.dto.SysRolePageVo
     * @author linxianghua
     * @date 2019/10/12 10:39
     */
    public List<SysRole> getPage(SysRolePageDTO sysRolePageDTO) {
        return sysRoleMapper.getPage(sysRolePageDTO);
    }

    /**
     * 详情
     *
     * @param id
     * @return com.anplus.admin.dto.SysRolePageVo
     * @author linxianghua
     * @date 2019/10/12 10:39
     */
    public SysRoleDetailVO getDetail(Integer id) {
        SysRoleDetailVO vo = new SysRoleDetailVO();
        vo.setCodeMap(commonSysCodeService.getCodeByTypes(CodeConstants.SYS_AVAILABLE_STATUS));
        List<SysBusinessTree> menuList = new ArrayList<>();
        Map<Long, SysBusinessTree> menuMap = new HashMap<>();
        List<SysRoleOperate> operateSelectedList = new ArrayList<>();
        List<SysBusiness> businessList = sysBusinessMapper.getAllBusiness();
        List<SysBusinessOperate> operateList = sysBusinessOperateService.selectAll();

        if (CommonUtils.isZero(id)) {
            SysRole sysRole = new SysRole();
            sysRole.setAvailable(true);
            vo.setSysRole(sysRole);
        } else {
            vo.setSysRole(selectByPrimaryKey(id));
            Condition condition = new Condition(SysRoleOperate.class);
            condition.createCriteria().andCondition("role_id = " + Long.valueOf(id));
            operateSelectedList = sysRoleOperateService.selectByCondition(condition);
            Map<Long, SysBusinessOperate> operateMap = new HashMap<>();
            for (SysBusinessOperate operate : operateList) {
                operateMap.put(operate.getId(), operate);
            }
            for (SysRoleOperate roleOperate : operateSelectedList) {
                if (operateMap.containsKey(roleOperate.getId())) {
                    SysBusinessOperate businessOperate = operateMap.get(roleOperate.getBusinessOperateId());
                    businessOperate.setSelected(true);
                }
            }
        }
        for (SysBusiness item : businessList) {
            SysBusinessTree sysBusinessTree = new SysBusinessTree(item);
            sysBusinessTree.setSysBusinessOperateList(new ArrayList<>());
            for (SysBusinessOperate sysBusinessOperate : operateList) {
                if (sysBusinessOperate.getBusinessId().equals(item.getId())) {
                    if (operateSelectedList.size() > 0) {
                        for (SysRoleOperate sysRoleOperate : operateSelectedList) {
                            if (sysRoleOperate.getBusinessOperateId().equals(sysBusinessOperate.getId())) {
                                sysBusinessOperate.setSelected(true);
                            }
                        }
                    }
                    sysBusinessTree.getSysBusinessOperateList().add(sysBusinessOperate);
                }
            }
            menuMap.put(item.getId(), sysBusinessTree);
        }

        getMenuList(menuList, menuMap, businessList);

        vo.setSysBusinessTreeList(menuList);
        return vo;
    }

    /**
     * @param param
     * @return java.lang.Long
     * @author linxianghua
     * @date 2019/10/14 14:57
     */
    public Long saveRole(SysRoleSaveDTO param, SysUser sysUser) {
        SysRole sysRole = param.getSysRole();
        if (sysRoleMapper.getRepeat(CommonUtils.toLong(sysRole.getId()), sysRole.getCode()) > 0) {
            throw new ServiceException(ErrorConstant.DUPLICATE_CODE);
        }
        //根据数据有无id判断是新建还是修改
        if (CommonUtils.isZero(sysRole.getId())) {
            sysRole.setCtUserId(sysUser.getId());
            insertSelective(sysRole);
        } else {
            EntityUtils.setModifyBase(sysRole, sysUser.getId());
            updateByPrimaryKeySelective(sysRole);
            Condition condition = new Condition(SysRoleOperate.class);
            condition.createCriteria().andCondition("role_id=" + sysRole.getId());
            sysRoleOperateService.deleteByCondition(condition);
        }
        if (param.getSaveOperatedList().size() > 0) {
            sysRoleOperateMapper.insertBatch(sysRole.getId(), param.getSaveOperatedList());
        }
        return sysRole.getId();
    }

    /**
     * 查询所以可用角色
     *
     * @param
     * @return java.util.List<com.dao.model.sys.SysRole>
     * @author wangning
     * @date 2019-10-16 16:58
     */
    public List<SysRole> getAllAvailableRole() {
        return sysRoleMapper.getAllAvailableRole();
    }

    /**
     * 更改用户启用状态
     *
     * @param id
     * @return java.lang.Long
     * @author linxianghua
     * @date 2019/11/14 11:37
     */
    public Long changeStatus(Long id) {
        SysRole sysRole = selectByPrimaryKey(id);
        sysRole.setAvailable(!sysRole.getAvailable());

        if (!sysRole.getAvailable()) {
            // 如果是禁用，则校验是否已经关联用户
            if (sysRoleMapper.getRoleUserNum(id) > 0) {
                throw new ServiceException("该角色已经关联用户，不能禁用");
            }
        }
        updateByPrimaryKeySelective(sysRole);
        return sysRole.getId();
    }
}
