package com.anplus.admincontrol.service.task;

import com.anplus.admincontrol.mapper.task.ZtTaskestimateBusinessMapper;
import com.anplus.admincontrol.service.task.bo.UserDailyConsumedBO;
import com.common.util.MailUtil;
import com.dao.enums.EnvironmentEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;

/**
 * @author 商天翔
 * 工时预警job,每天检查一下昨天工时登记是否正常
 */
@Service
public class TaskConsumeWarningJobService {

    @Resource
    private ZtTaskestimateBusinessMapper bizMapper;
    @Resource
    private ZtTaskestimateService ztTaskestimateService;
    @Resource
    private MailUtil mailUtil;
    @Value("${spring.profiles.active}")
    private String env;

    /**
     * 每天7点检查一次
     */
    @Scheduled(cron = "0 0 7 * * ?")
    public void checkYesterday() {
        if (EnvironmentEnum.DEV.name().equals(env)) {
            return;
        }
        // 昨天日期
        var yesterday = LocalDate.now().minusDays(1);
        checkUserConsumed(yesterday);
    }

    public void checkUserConsumed(LocalDate date, String... specificAccounts) {
        // 获取标准工时
        Double standardDailyConsume = ztTaskestimateService.getStandardDailyConsume();

        List<UserDailyConsumedBO> warningList = bizMapper.getWarningUsers(date, standardDailyConsume, specificAccounts);
        System.out.println(warningList);

        warningList.forEach(w -> {
            if (!StringUtils.isBlank(w.getEmail())) {
                var formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                mailUtil.sendText(Map.of(
                        MailUtil.TITLE, "警告: 禅道工时登记",
                        MailUtil.CONTENT, "您" + date.format(formatter) + "登记的工时不足" + standardDailyConsume + "小时",
                        MailUtil.SEND_ADDR, w.getEmail()
                ));
            }
        });
    }
}
