package com.anplus.admincontrol.service.task;

import cn.afterturn.easypoi.excel.entity.params.ExcelExportEntity;
import cn.afterturn.easypoi.excel.entity.vo.BaseEntityTypeConstants;
import com.anplus.admincontrol.controller.task.dto.ZtTaskEstimatePageDTO;
import com.anplus.admincontrol.controller.task.vo.AnnualConsumeData;
import com.anplus.admincontrol.mapper.holidays.bo.HolidayBO;
import com.anplus.admincontrol.mapper.task.ZtTaskestimateBusinessMapper;
import com.anplus.admincontrol.service.holidays.HolidaysService;
import com.anplus.admincontrol.service.task.bo.UserTaskConsumeBO;
import com.common.util.DateUtils;
import com.common.util.ExcelUtils;
import com.constants.Constants;
import com.dao.common.AbstractMapperService;
import com.dao.common.page.PageInfoVO;
import com.dao.mapper.sys.SysParamMapper;
import com.dao.model.sys.SysParam;
import com.dao.model.zt.ZtTaskestimate;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.jni.Local;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author 商天翔
 * @date 2021-03-02 17:28
 * ZtTaskestimateService服务类
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ZtTaskestimateService extends AbstractMapperService<ZtTaskestimate> {

    @Resource
    private ZtTaskestimateBusinessMapper bizMapper;
    @Resource
    private SysParamMapper sysParamMapper;
    @Resource
    private HolidaysService holidaysService;

    /**
     * ----分页查询列表---begin
     */
    public PageInfoVO getConsumedPage(ZtTaskEstimatePageDTO param) {
        // 人员分页
        PageHelper.startPage(param.getPage(), param.getSize());
        List<Map<String, Object>> userList = bizMapper.getUserPage(param);
        var result = new PageInfoVO<>(userList);

        Calendar calendar = Calendar.getInstance();
        if (StringUtils.isEmpty(param.getMonth())) {
            var today = new Date();
            var month = DateUtils.formatDate(today, "yyyy-MM");
            param.setMonth(month);
            calendar.setTime(today);
        } else {
            calendar.setTime(DateUtils.parseDate(param.getMonth() + "-01"));
        }
        // 当月天数
        int dayOfTheMonth = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);

        // 根据人员id查询相关数据
        param.setIncludeUsers(userList.stream().map(m -> Long.parseLong(m.get("userId").toString())).toArray(Long[]::new));
        List<UserTaskConsumeBO> list = bizMapper.getTaskConsumed(param);


        List<Map<String, Object>> userConsumeList = new ArrayList<>(userList.size() * 2);
        Map<Long, List<Map<String, Object>>> userConsumeMap = new HashMap<>(userList.size());
        // 总合计行
        Map<String, Object> totalOfTotal = null;
        if (!ObjectUtils.isEmpty(param.getProjectId())) {
            totalOfTotal = new HashMap<>(38);
            totalOfTotal.put(REAL_NAME, "总合计");
            totalOfTotal.put(PROJECT_NAME, "总合计");
            totalOfTotal.put(ROW_KEY, TOTAL_COL + TOTAL_COL);
        }

        boolean addTotal = param.getProjectId() == null;
        Long id = 0L;
        int begin = 0;
        for (int i = 0; i < list.size(); i++) {
            var row = list.get(i);
            if (i == 0) {
                id = row.getUserId();
                continue;
            }
            if (!id.equals(row.getUserId())) {
                var subList = list.subList(begin, i);
                pivotOneUserData(subList, userConsumeMap, dayOfTheMonth, addTotal, totalOfTotal);
                begin = i;
                id = row.getUserId();
            }
        }
        if (list.size() > 0) {
            pivotOneUserData(list.subList(begin, list.size()), userConsumeMap, dayOfTheMonth, addTotal, totalOfTotal);
        }

        userList.forEach(u -> {
            var userSubList = userConsumeMap.get(Long.parseLong(u.get("userId").toString()));
            if (userSubList == null) {
                // 当指查询参数中指定了项目时就不再添加空行
                userConsumeList.add(emptyTotalRow(u));
            } else {
                userConsumeList.addAll(userSubList);
            }
        });

        result.setList(userConsumeList);
        if (!ObjectUtils.isEmpty(param.getProjectId())) {
            userConsumeList.add(totalOfTotal);
        }
        return result;
    }

    private final static String DAY_COL_NAME = "d";
    /**
     * map中的key名称
     */
    private final static String REAL_NAME = "realName";
    private final static String PROJECT_NAME = "projectName";
    private final static String PROJECT_ID = "projectId";
    private final static String ROW_SPAN = "rowspan";
    private final static String ROW_KEY = "rowKey";
    private final static String TOTAL = "合计";
    private final static String TOTAL_COL = "total";

    /**
     * 处理一个用户的数据
     */
    private void pivotOneUserData(List<UserTaskConsumeBO> userTaskList, Map<Long, List<Map<String, Object>>> map, int dayOfTheMonth, boolean addTotal, Map<String, Object> totalOfTotal) {
        // 判断共有几个项目
        List<Long> projectIdList = userTaskList.stream().map(UserTaskConsumeBO::getProjectId).distinct().collect(Collectors.toList());
        List<Map<String, Object>> result = new ArrayList<>(projectIdList.size() + 1);
        // 按照项目将单个用户的数据分为几行
        Map<Long, Integer> idIndexMap = new HashMap<>(projectIdList.size());
        for (int i = 0; i < projectIdList.size(); i++) {
            var id = projectIdList.get(i);
            // 随便找到一条记录
            UserTaskConsumeBO record = null;
            for (UserTaskConsumeBO bo : userTaskList) {
                if (bo.getProjectId().equals(id)) {
                    record = bo;
                    break;
                }
            }

            var row = new HashMap<String, Object>();
            row.put(REAL_NAME, record.getRealname());
            row.put(PROJECT_NAME, record.getProjectName());
            row.put(PROJECT_ID, id);
            row.put(ROW_KEY, id + "-" + userTaskList.get(0).getUserId());
            row.put(ROW_SPAN, 0);
            result.add(row);
            idIndexMap.put(id, i);
        }
        result.get(0).put(ROW_SPAN, addTotal ? projectIdList.size() + 1 : projectIdList.size());
        // 添加合计行
        if (addTotal) {
            var row = new HashMap<String, Object>();
            row.put(REAL_NAME, userTaskList.get(0).getRealname());
            row.put(PROJECT_NAME, TOTAL);
            row.put(ROW_SPAN, 0);
            result.add(row);
        }


        // 每日工时
        for (int i = 1; i <= dayOfTheMonth; i++) {
            // 每日全项目总计
            double dayTotal = 0;
            var key = DAY_COL_NAME + i;
            for (UserTaskConsumeBO ut : userTaskList) {
                if (ut.getDate() == i) {
                    var currentMap = result.get(idIndexMap.get(ut.getProjectId()));
                    currentMap.put(key, ut.getConsumed());
                    // 加到每行的总计列中
                    currentMap.put(TOTAL_COL, (currentMap.get(TOTAL_COL) == null ? 0d : Double.parseDouble(currentMap.get(TOTAL_COL).toString())) + ut.getConsumed());
                    dayTotal += ut.getConsumed();
                }
            }

            // 计算总计工时
            if (addTotal) {
                var totalRow = result.get(result.size() - 1);
                totalRow.put(key, dayTotal);
                totalRow.put(TOTAL_COL, (totalRow.get(TOTAL_COL) == null ? 0d : Double.parseDouble(totalRow.get(TOTAL_COL).toString())) + dayTotal);
                totalRow.put(ROW_KEY, TOTAL_COL + "-" + userTaskList.get(0).getUserId());
            } else {
                if (ObjectUtils.isNotEmpty(totalOfTotal)) {
                    totalOfTotal.put(key, (totalOfTotal.get(key) == null ? 0d : Double.parseDouble(totalOfTotal.get(key).toString())) + dayTotal);
                    totalOfTotal.put(TOTAL_COL, (totalOfTotal.get(TOTAL_COL) == null ? 0d : Double.parseDouble(totalOfTotal.get(TOTAL_COL).toString())) + dayTotal);
                }
            }


        }
        map.put(userTaskList.get(0).getUserId(), result);
    }

    /**
     * 返回一个空的合计行
     */
    private Map<String, Object> emptyTotalRow(Map<String, Object> bo) {
        Map<String, Object> map = new HashMap<>(3);
        map.put(REAL_NAME, bo.get("realname"));
        map.put(PROJECT_NAME, TOTAL);
        map.put(ROW_SPAN, 1);
        map.put(ROW_KEY, TOTAL_COL + "-" + bo.get("userId"));
        return map;
    }
    /**
     * ----分页查询列表---end
     */


    /**
     * 获取工时统计辅助信息， 当月节假日和标准工时数
     *
     * @param month 月份 yyyy-MM
     * @return 元组形式 [节假日数组, 标准工时]
     */
    public List getAuxiliaryInfo(String month) {
        List<String> rawHolidayData = bizMapper.getHolidaysByMonth(month);
        List<Integer> restDays = List.of();
        if (!rawHolidayData.isEmpty()) {
            restDays = rawHolidayData.stream()
                    .map(d -> Arrays.asList(d.split(",")))
                    .flatMap(Collection::stream)
                    .filter(d -> month.equals(StringUtils.trim(d).substring(0, 7)))
                    .map(d -> Integer.parseInt(StringUtils.trim(d).substring(8)))
                    .collect(Collectors.toList());
        }
        return List.of(restDays, getStandardDailyConsume());
    }

    /**
     * 导出工时明细  begin --------------------------------
     */
    public void exportExcel(ZtTaskEstimatePageDTO param, HttpServletRequest request, HttpServletResponse response) {
        param.setSize(9999);
        param.setPage(1);
        List<Map<String, Object>> data = getConsumedPage(param).getList();
        List<Map<String, Object>> exportData = data.stream().map(ZeroMap::new).collect(Collectors.toList());

        ExcelUtils.mapExportExcel(
                exportData,
                getExcelColumnList(param.getMonth()),
                param.getMonth() + "工时明细", param.getMonth(),
                param.getMonth() + "工时明细.xlsx",
                request,
                response
        );
    }

    private List<ExcelExportEntity> getExcelColumnList(String month) {
        // 计算天数
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(DateUtils.parseDate(month + "-01"));
        int dayOfTheMonth = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
        var colList = new ArrayList<ExcelExportEntity>(3 + dayOfTheMonth);
        // 姓名列
        var realNameCol = new ExcelExportEntity("姓名", "realName", 15);
        realNameCol.setMergeVertical(true);
        realNameCol.setType(BaseEntityTypeConstants.STRING_TYPE);
        realNameCol.setOrderNum(1);
        colList.add(realNameCol);
        // 项目列
        var projectCol = new ExcelExportEntity("项目", "projectName", 40);
        projectCol.setType(BaseEntityTypeConstants.STRING_TYPE);

        realNameCol.setOrderNum(2);
        colList.add(projectCol);
        var numFormat = "###.#";

        // 合计列
        var totalCol = new ExcelExportEntity("合计", "total", 8);
        totalCol.setNumFormat(numFormat);
        // 日消耗列
        for (int i = 1; i <= dayOfTheMonth; i++) {
            var dayConsumeCol = new ExcelExportEntity(i + "", DAY_COL_NAME + i, 8);
            dayConsumeCol.setNumFormat(numFormat);
            dayConsumeCol.setType(BaseEntityTypeConstants.DOUBLE_TYPE);
            colList.add(dayConsumeCol);
        }
        colList.add(totalCol);

        return colList;
    }

    /**
     * 获取人日标准工时
     */
    public Double getStandardDailyConsume() {
        var condition = new SysParam();
        condition.setCode(Constants.SysParamCode.STANDARD_DAILY_CONSUME);
        var sysParam = sysParamMapper.selectOne(condition);
        return Double.parseDouble(sysParam.getValueStr());
    }

    /**
     * 年度工时统计图数据
     * @return 元组[标准工时, 图表数据List ]
     */
    public List getAnnualConsumeData(Integer year) {
        List<HolidayBO> holidays = holidaysService.getByYear(year);
        // 计算标准工时

        long holidayCount = holidays.stream().mapToLong(h -> {
            // 防止出现跨年节假日
            if (h.getDuration() == 1) {
                return 1;
            }
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            List<LocalDate> vacationList = Arrays.stream(h.getVacation().split(","))
                    .map(d -> LocalDate.parse(d.trim(), fmt))
                    .filter(d -> d.getYear() == year).collect(Collectors.toList());
            h.setVacationList(vacationList);
            return vacationList.size();
        }).sum();

        /**
         * 计算当年有多少天
         */
        var now = LocalDate.now();
        LocalDate firstDay = LocalDate.of(year - 1, 12, 31);
        LocalDate lastDay = null;
        if(year == now.getYear()) {
            lastDay = now;
        } else {
            lastDay = LocalDate.of(year, 12, 31);
        }
        long dayOfYear = lastDay.toEpochDay() - firstDay.toEpochDay();

        DecimalFormat df = new DecimalFormat("##.#");
        double standardDailyConsume = getStandardDailyConsume();
        var standardConsume = Double.valueOf(df.format((dayOfYear - holidayCount) * standardDailyConsume));
        List<AnnualConsumeData> dataList = bizMapper.getAnnualConsume(year);

        dataList.forEach(r -> {
            r.setValue(Double.valueOf(df.format(r.getValue())));
            // 在当年入职的员工需要计算入职前的标准工时
            if(r.getJoinDate().getYear() == year) {
                // 当前用户入职前的天数
                long beforeJoinDays = r.getJoinDate().toEpochDay() - firstDay.toEpochDay();
                // 当前用户需要减掉的假期数
                long currentUserHolidayCount = holidays.stream()
                        // 结束日期大于入职日期
                        .filter(h -> h.getStartDate().isBefore(r.getJoinDate()))
                        .mapToLong(h -> {
                            if(h.getDuration() == 1) {
                                return 1;
                            }
                            return h.getVacationList().stream()
                                    .filter(d -> d.isBefore(r.getJoinDate()) || d.equals(r.getJoinDate()))
                                    .count();
                        }).sum();
                r.setBeforeJoin((beforeJoinDays - currentUserHolidayCount) * standardDailyConsume);
            }
        });

        return List.of(standardConsume, dataList);
    }

    /**
     * 处理工时null值
     */
    static class ZeroMap extends HashMap<String, Object> {
        public ZeroMap(Map m) {
            super(m);
        }

        @Override
        public Object get(Object key) {
            if (super.get(key) == null) {
                return 0;
            }
            return super.get(key);
        }
    }
    /**
     * 导出工时明细  end ----------------------
     */

}
