package com.anplus.admincontrol.service.zt;

import com.anplus.admincontrol.controller.zt.dto.ZtUserPageDTO;
import com.anplus.admincontrol.controller.zt.vo.ZtUserWagesPageVO;
import com.anplus.admincontrol.controller.zt.vo.ZtUserWagesSaveDTO;
import com.dao.model.zt.ZtUser;
import com.dao.mapper.zt.ZtUserMapper;
import com.anplus.admincontrol.mapper.zt.ZtUserBusinessMapper;

import com.dao.common.AbstractMapperService;
import com.dao.common.page.PageParams;
import com.dao.common.page.PageInfoVO;
import com.dao.model.zt.ZtUserWages;
import com.github.pagehelper.PageHelper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author 孟令君
 * @date 2021-03-02 08:34
 * ZtUserService服务类
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ZtUserService extends AbstractMapperService<ZtUser> {

    @Resource
    private ZtUserMapper ztUserMapper;
    @Resource
    private ZtUserBusinessMapper ztUserBusinessMapper;
    @Resource
    private ZtUserWagesService ztUserWagesService;

    /**
     * 分页查询列表
     */
    public PageInfoVO getPage(ZtUserPageDTO param) {
        PageHelper.startPage(param.getPage(), param.getSize());
        List<ZtUserWagesPageVO> userList = ztUserBusinessMapper.getPage(param);
        return new PageInfoVO(userList);
    }

    public Long saveUserWages(ZtUserWagesSaveDTO ztUser) {
        ZtUserWages ztUserWages = new ZtUserWages();
        //判断当前工资表是否已存在该用户信息
        if (CollectionUtils.isEmpty(ztUserWagesService.findByUserId(ztUser.getId()))) {
            //没有信息新建
            ztUserWages = ZtUserWages.builder().userId(ztUser.getId()).statisticsStatus(1).wages(ztUser.getWages()).deleted((byte) 0).build();
            ztUserWagesService.insertSelective(ztUserWages);
        } else {
            if (ztUserWagesService.findByUserId(ztUser.getId()).size() == 1) {
                ztUserWages = ztUserWagesService.findByUserId(ztUser.getId()).get(0);
                ztUserWages.setWages(ztUser.getWages());
                ztUserWages.setCtDate(new Date());
                ztUserWagesService.updateByPrimaryKeySelective(ztUserWages);
            } else {
                //list的数据大于1 删除所有重新添加
                Condition condition = new Condition(ZtUserWages.class);
                condition.createCriteria().andEqualTo("userId", ztUser.getId());
                ztUserWagesService.deleteByCondition(condition);
                ztUserWages.builder().userId(ztUser.getId()).wages(ztUser.getWages()).build();
                ztUserWagesService.insertSelective(ztUserWages);
            }
        }

        return ztUserWages.getId();
    }

    public Integer updateStatistics(Long id) {

        List<ZtUserWages> byUserId = ztUserWagesService.findByUserId(id);
        ZtUserWages ztUserWages = new ZtUserWages();
        if (CollectionUtils.isEmpty(byUserId)) {
            ztUserWages = ZtUserWages.builder().userId(id).ctDate(new Date()).wages(BigDecimal.valueOf(0)).statisticsStatus(0).deleted((byte) 0).build();

            ztUserWagesService.insertSelective(ztUserWages);
        } else {
            ztUserWages = byUserId.get(0);
            ztUserWages.setStatisticsStatus(ztUserWages.getStatisticsStatus() == null ? 0:ztUserWages.getStatisticsStatus() ==1?0:1);
            ztUserWagesService.updateByPrimaryKeySelective(ztUserWages);
        }
        return ztUserWages.getStatisticsStatus();
    }
}
