package com.anplus.common.admin.security.authentication;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.common.util.CommonUtils;
import com.core.security.jwt.JwtManager;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.Collections;

/**
 * @Author wangning
 * @Date 2019-10-04 17:20
 * @Email 1151089590@qq.com
 * @Description:
 */
@Component
@Slf4j
public class TokenService {
    /**
     * 默认一天有效时常 秒
     */
    @Value("${project.jwt.token-expire-time}")
    public long tokenExpireTime = 86_400;
    /**
     * 默认使用记住我模式10天token有效时长 秒
     */
    @Value("${project.jwt.token-expire-time-long}")
    public long tokenExpireTimeLong = 864_000;

    /**
     * request header中token存放的key值
     */
    public String headerTokenKey = JwtManager.AUTHORIZATION_HEADER;

    /**
     * token前缀
     */
    public String tokenPrefix = JwtManager.BEARER;

    private String user = "userName";
    private String company = "companyId";
    private String jsonObj = "userName";

    /**
     * 总控平台生成token
     **/
    public String addAuthentication(String username, boolean remeberMe) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(jsonObj, username);
        return builderToken(jsonObject, remeberMe);
    }

    /**
     * 公司平台生成token
     **/
    public String addAuthentication(String username, String companyId, boolean rememberMe) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(user, username);
        jsonObject.put(company, companyId);
        return builderToken(jsonObject, rememberMe);
    }


    /**
     * 公司平台 解析JWT 从令牌中获取数据声明
     *
     * @param request
     */
    public Authentication getAuthentication(HttpServletRequest request) {
        try {
            Claims claims = getClaim(request);
            if (claims != null) {
                // 如果token验证成功，将token对应的用户名
                String userName = (String) JSON.parseObject(claims.getSubject()).get(user);
                String companyId = (String) JSON.parseObject(claims.getSubject()).get(company);
                if (userName != null) {
                    return new UsernamePasswordAuthenticationToken(userName + CommonUtils.SEPARATOR + companyId, null, Collections.emptyList());
                }
            }
            return null;
        } catch (Exception e) {
            log.error("token解析错误:", e.getMessage());
            return null;
        }
    }

    /**
     * 总控平台 解析JWT 从令牌中获取数据声明
     *
     * @param request
     */
    public Authentication getControlAuthentication(HttpServletRequest request) {
        try {
            Claims claims = getClaim(request);
            if (claims != null) {
                // 如果token验证成功，将token对应的用户名
                String userName = (String) JSON.parseObject(claims.getSubject()).get(jsonObj);
                if (userName != null) {
                    return new UsernamePasswordAuthenticationToken(userName, null, Collections.emptyList());
                }
            }
            return null;
        } catch (Exception e) {
            log.error("token解析错误:", e.getMessage());
            return null;
        }
    }

    private Claims getClaim(HttpServletRequest request) {
        String token = request.getHeader(headerTokenKey);
        return JwtManager.parseToken(token.replace(tokenPrefix, ""));
    }

    private String builderToken(JSONObject jsonObject, boolean rememberMe) {
        return JwtManager.createToken(jsonObject, rememberMe == true ? tokenExpireTimeLong : tokenExpireTime);
    }
}
