package com.common.aliyun;

import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.afs.model.v20180112.AuthenticateSigRequest;
import com.aliyuncs.afs.model.v20180112.AuthenticateSigResponse;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;

@Slf4j
public class AfsUtils {
    @Value("${aliyun.afs.access-key-id}")
    private String accessKeyId;
    @Value("${aliyun.afs.access-key-secret}")
    private String accessKeySecret;
    @Value("${aliyun.afs.region-id}")
    private String regionId;
    @Value("${aliyun.afs.app-key}")
    private String appKey;
    @Value("${aliyun.afs.scene}")
    private String scene;

    /**
     * 阿里人机校验
     *
     * @param dto
     * @return
     */
    public Boolean validate(AfsDTO dto) {
        IClientProfile iClientProfile = DefaultProfile.getProfile(regionId, accessKeyId, accessKeySecret);
        IAcsClient client = new DefaultAcsClient(iClientProfile);
        DefaultProfile.addEndpoint(regionId, "afs", "afs.aliyuncs.com");
        AuthenticateSigRequest request = new AuthenticateSigRequest();
        request.setAppKey(appKey);
        request.setScene(scene);
        request.setSessionId(dto.getSessionId());
        request.setSig(dto.getSig());
        request.setToken(dto.getToken());
        request.setRemoteIp(dto.getIp());
        try {
            //response的code枚举：100验签通过，900验签失败
            AuthenticateSigResponse response = client.getAcsResponse(request);
            if (response.getCode() == 100) {
                return true;
            }
        } catch (Exception e) {
            log.error("人机校验失败-" + dto.getIp(), e);
            e.printStackTrace();
        }
        return false;
    }
}
