package com.common.business;

import com.common.util.ArrayUtils;
import com.dao.mapper.CommonMapper;
import com.dao.model.sys.SysCodeDetail;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author wang jianguo
 * @date 2020/1/2 11:13
 * @email qizhi1a440@outlook.com
 * @description:通用代码详情查询服务类
 */
@Service
public class CommonSysCodeService {

    @Resource
    private CommonMapper commonMapper;

    /**
     * 获取通用代码详情列表
     *
     * @param code
     * @return
     */
    public List<SysCodeDetail> getCodeDetailList(String code) {
        if (Objects.isNull(code)) {
            return null;
        }
        return commonMapper.getSysCodeDetailList(code);
    }

    /**
     * 获取通用代码
     *
     * @param codeType
     * @return
     */
    public Map<String, List<SysCodeDetail>> getCodeByTypes(String... codeType) {
        if (ArrayUtils.isEmpty(codeType)) {
            return null;
        }
        List<SysCodeDetail> list = new ArrayList();
        for (String code : codeType) {
            List<SysCodeDetail> detailList = this.getCodeDetailList(code);
            if (ArrayUtils.isEmpty(detailList)) {
                continue;
            }
            list.addAll(detailList);
        }
        return this.process(list);
    }

    /**
     * 设置通用代码映射
     *
     * @param detailList
     * @return
     */
    private Map<String, List<SysCodeDetail>> process(List<SysCodeDetail> detailList) {
        Map<String, List<SysCodeDetail>> resultMap = null;
        if (!ArrayUtils.isEmpty(detailList)) {
            resultMap = new HashMap<>(detailList.size());
            for (SysCodeDetail item : detailList) {
                String typeCode = item.getTypeCode();
                if (!resultMap.containsKey(typeCode)) {
                    resultMap.put(typeCode, new ArrayList<>());
                }
                resultMap.get(typeCode).add(item);
            }
        }
        return resultMap;
    }
}
