package com.common.log;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.common.util.Utils;
import com.constants.Constants;
import com.core.exception.ServiceException;
import com.core.result.Result;
import com.dao.mapper.CommonErrorLogMapper;
import com.dao.model.base.BaseErrorLog;
import com.google.common.eventbus.Subscribe;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.context.request.RequestAttributes;

import javax.annotation.Resource;

/**
 * @author 潘维吉
 * @date 2019-10-11
 * 定义EventBus事件消费者逻辑
 * 利用Guava的注解@Subscribe指定当某个事件发生时执行所制定的逻辑
 */
@Component
public class LogEventBusListener {

    @Value("${spring.profiles.active}")
    private String env;

    @Resource
    private NativeWebRequest nativeWebRequest;

    @Resource
    private CommonErrorLogMapper commonErrorLogMapper;

    /**
     * 当所注册到的事件总线post上发生Result消息时被订阅执行
     * MvcConfig 统一异常执行 事件发送 eventBus.post(result)
     * 要用于异常日志持久化存储查询
     */
    @Subscribe
    public void listener(Result result) {
        BaseErrorLog log = new BaseErrorLog();
        //当前用户Id
        Integer userId = (Integer) nativeWebRequest.getAttribute(Constants.CURRENT_USER_ID, RequestAttributes.SCOPE_REQUEST);
        //其他json信息
        JSONObject data = ((JSONObject) JSON.toJSON(result.getData()));
        //模块项目名称
        String projectName = data.getString("projectName") + "; 环境:" + env;
        //内容信息
        String content = result.getMsg();
        log.setUserId(userId == null ? null : Long.valueOf(userId));
        log.setProjectName(projectName);
        Integer code = result.getCode();
        log.setErrorCode(String.valueOf(code));
        log.setContent(content);
        log.setPlatform(Utils.osName());
        try {
            commonErrorLogMapper.insertSelective(log);
        } catch (Exception e) {
            throw new ServiceException("异常日志保存失败:" + result.getMsg());
        }
    }

}
