package com.common.util;

import com.fasterxml.jackson.core.JsonGenerationException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.io.StringWriter;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.regex.Pattern;

/**
 * @Author wangning
 * @Date 2019-10-12 16:33
 * @Email 1151089590@qq.com
 * @Description:
 */
public class CommonUtils {

    private static String LETTER_CHARS = "abcdefghijkmnopqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ0123456789";

    private static ObjectMapper objectMapper = new ObjectMapper();

    public static String SEPARATOR = "@";

    public static String integerListToStr(List<Integer> list) {
        String result = "";
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < list.size(); i++) {
            String s = list.get(i) + "";
            if (i < list.size() - 1) {
                sb.append(s).append(",");
            } else {
                sb.append(s);
                result = sb.toString();
            }
        }
        return result;
    }

    public static String longListToStr(List<Long> list) {
        String result = "";
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < list.size(); i++) {
            String s = list.get(i) + "";
            if (i < list.size() - 1) {
                sb.append(s).append(",");
            } else {
                sb.append(s);
                result = sb.toString();
            }
        }
        return result;
    }

    public static String integerArrayToStr(Integer[] list) {
        String result = "";
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < list.length; i++) {
            String s = list[i] + "";
            if (i < list.length - 1) {
                sb.append(s).append(",");
            } else {
                sb.append(s);
                result = sb.toString();
            }
        }
        return result;
    }

    public static boolean isZero(Integer value) {
        if (value == null || value == 0) {
            return true;
        }
        return false;
    }

    public static boolean isZero(Long value) {
        if (value == null || value == 0) {
            return true;
        }
        return false;
    }

    public static boolean isEmpty(List list) {
        if (list == null || list.size() == 0) {
            return true;
        }
        return false;
    }

    public static boolean isEmpty(String str) {
        if (str == null || StringUtils.isEmpty(str.trim())) {
            return true;
        }
        return false;
    }

    private static Pattern pattern = Pattern.compile("[0-9]*");
    private static Pattern patternIntegerOrLetter = Pattern.compile("[0-9a-zA-Z]*");

    public static boolean isInteger(String string) {
        if (string == null) {
            return false;
        }
        return pattern.matcher(string).matches();
    }

    public static boolean isIntegerOrLetter(String string) {
        if (string == null) {
            return false;
        }
        return patternIntegerOrLetter.matcher(string).matches();
    }

    //类型转换
    public static String toString(Object value) {
        return value != null ? value.toString() : "";
    }

    public static Double toDouble(Object value) {
        return value == null ? 0 : Double.valueOf(value.toString());
    }

    public static Long toLong(Object value) {
        return value == null || StringUtils.isEmpty(value.toString()) ? 0 : Math.round(Double.valueOf(value.toString()));
    }

    public static Integer toInteger(Object value) {
        return value == null || StringUtils.isEmpty(value.toString()) ? 0 : Math.round(Float.valueOf(value.toString()));
    }

    public static String generateRandomChars(int length) {
        char[] charsArray = LETTER_CHARS.toCharArray();
        Random random = new Random(System.currentTimeMillis());
        StringBuilder sb = new StringBuilder(length);
        for (int i = 0; i < length; i++) {
            sb.append(charsArray[random.nextInt(charsArray.length)]);
        }
        return sb.toString();
    }

    public static String[] getUserNameAndCompanyId(String userNameCompanyId) {
        String[] result = {"", ""};
        String[] strs = userNameCompanyId.split(SEPARATOR);
        if (strs.length > 1) {
            String companyId = strs[strs.length - 1];
            String username = "";
            // 处理名字中含有 @符号 的情况
            for (int i = 0; i < strs.length - 1; i++) {
                if (i == 0) {
                    username = strs[i];
                } else {
                    username += SEPARATOR + strs[i];
                }
            }
            result[0] = username;
            result[1] = companyId;

        }
        return result;
    }

    public static String getRandom(int len) {
        Random r = new Random();
        StringBuilder rs = new StringBuilder();
        for (int i = 0; i < len; i++) {
            rs.append(r.nextInt(10));
        }
        return rs.toString();
    }

    /**
     * 校验字符串能否转成数字
     *
     * @param str
     * @return
     */
    public static boolean canParseInt(String str) {
        //验证是否为空
        if (str == null) {
            return false;
        }
        // 处理表示负数的字符串
        if (str.length() > 1 && str.substring(0, 1).equals("-")) {
            str = str.substring(1);
        }
        //使用正则表达式判断该字符串是否为数字
        return str.matches("\\d+");
    }

    /**
     * 转化Map为json字符串
     *
     * @throws JsonGenerationException
     * @throws JsonMappingException
     * @throws IOException
     */
    public static String writeMap2Json(Map<String, ?> map)
            throws JsonGenerationException, JsonMappingException, IOException {

        StringWriter w = new StringWriter();
        objectMapper.writeValue(w, map);

        return w.toString();
    }

    /**
     * 生成月列表
     *
     * @param beginMonthStr
     * @param endMonthStr
     * @return
     */
    public static List<String> initMonthList(String beginMonthStr, String endMonthStr) {
        List<String> monthList = new ArrayList<>();
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("yyyy-MM");
        LocalDate beginMonth = LocalDate.parse(beginMonthStr + "-01", dayFormatter);
        LocalDate endMonth = LocalDate.parse(endMonthStr + "-01", dayFormatter);
        while (beginMonth.isBefore(endMonth)) {
            monthList.add(beginMonth.format(monthFormatter));
            beginMonth = beginMonth.plusMonths(1);
        }
        monthList.add(endMonth.format(monthFormatter));
        return monthList;
    }
}
