package com.core.security.jwt;

import lombok.Getter;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author 潘维吉
 * @date 2018-07-25
 * 不授权路径配置   不授权路径单独封装解耦配置
 * 注意 每个项目和模块路径都不一样  需要各自的application.yaml里单独修改配置
 * @Component自动装配Bean到IOC容器管理 用于获取application.yaml设置的非授权url路径
 */
@ConfigurationProperties(prefix = "project.jwt.path")
@Component
@Getter
public class JwtPathProperties {

    /** 等于path路径 忽略jwt授权路径集合 */
    private static List<String> equalsPaths;

    /** 包含匹配path路径 忽略jwt授权路径集合 */
    private static List<String> containsPaths;

    /**
     * 添加不授权路径  授权时不认证
     * 区分方法类型和是否包含或等于
     *
     * @param uri       请求path路径
     * @param uriMethod 请求的方法
     */
    public static boolean isAuth(String uri, String uriMethod) {
        //List修改操作 防止并发ConcurrentModificationException异常
        if (uri.startsWith("/api")) {
            uri = uri.replace("/api", "");
        }
        //路径等于匹配
        if (equalsPaths != null) {
            for (String path : equalsPaths) {
                if (path.equals(uri)) {
                    return true;
                }
            }
        }
        //路径包含匹配
        if (containsPaths != null) {
            for (String path : containsPaths) {
                if (uri.contains(path)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 因为@ConfigurationProperties只会调用 非静态的set方法
     */
    public void setEqualsPaths(List<String> equalsPaths) {
        JwtPathProperties.equalsPaths = equalsPaths;
    }

    public void setContainsPaths(List<String> containsPaths) {
        JwtPathProperties.containsPaths = containsPaths;
    }

}
