package com.dao.common.page;

import com.github.pagehelper.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * @author 潘维吉
 * @date 2019/10/31 16:05
 * @email 406798106@qq.com
 * @description APP分页VO 简化属性
 */
@ApiModel(value = "AppPageInfoVO", description = "APP分页接口响应数据")
@Getter
@Setter
public class AppPageInfoVO<T> implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "当前页", example = "1")
    private Integer page;

    @ApiModelProperty("是否有下一页")
    private Boolean hasNextPage;

    @ApiModelProperty("数据结果集")
    private List<T> list;

    public AppPageInfoVO() {
        this.page = 1;
        this.hasNextPage = false;
        this.list = new ArrayList<>();
    }

    public AppPageInfoVO(List<T> list) {
        if (list == null) {
            list = new ArrayList<>();
        }
        if (list instanceof Page) {
            Page page = (Page) list;
            this.page = page.getPageNum();
            this.hasNextPage = page.getPageNum() < page.getPages();
            this.list = page;
        } else if (list instanceof Collection) {
            this.page = 1;
            this.hasNextPage = false;
            this.list = list;
        }
    }
}
