package com.dao.model.holidays;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.util.Date;
import java.util.List;
import javax.persistence.*;

import lombok.*;
import tk.mybatis.mapper.annotation.LogicDelete;

@Getter
@Setter
@ToString
@Builder
@AllArgsConstructor
@NoArgsConstructor
@ApiModel("节假日")
public class Holidays {
    /**
     * 主键
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @ApiModelProperty("主键")
    private Long id;

    /**
     * 假期名称
     */
    @ApiModelProperty("假期名称")
    private String name;

    /**
     * 1节假日,2双休日
     */
    @ApiModelProperty("1节假日,2双休日")
    private Integer dayCode;

    /**
     * 开始日期
     */
    @Column(name = "start_date")
    @ApiModelProperty("开始日期")
    private Date startDate;

    /**
     * 结束日期
     */
    @Column(name = "end_date")
    @ApiModelProperty("结束日期")
    private Date endDate;

    /**
     * 年度
     */
    @ApiModelProperty("年度")
    private Long year;

    /**
     * 开始月份
     */
    @Column(name = "start_month")
    @ApiModelProperty("开始月份")
    private Integer startMonth;

    /**
     * 结束月份
     */
    @Column(name = "end_month")
    @ApiModelProperty("结束月份")
    private Integer endMonth;

    /**
     * 假期时长
     */
    @ApiModelProperty("假期时长")
    private Long duration;

    /**
     * 是否删除 0 未删除  1 逻辑删除
     */
    @ApiModelProperty("是否删除 0 未删除  1 逻辑删除")
    @LogicDelete
    private Byte deleted;

    /**
     * 创建人
     */
    @Column(name = "ct_user_id")
    @ApiModelProperty("创建人")
    private Long ctUserId;

    /**
     * 创建时间
     */
    @Column(name = "ct_date")
    @ApiModelProperty("创建时间")
    private Date ctDate;

    /**
     * 修改人
     */
    @Column(name = "lm_user_id")
    @ApiModelProperty("修改人")
    private Long lmUserId;

    /**
     * 修改时间
     */
    @Column(name = "lm_date")
    @ApiModelProperty("修改时间")
    private Date lmDate;

    /**
     * 假期详情
     */
    @ApiModelProperty("假期详情")
    private String vacation;

    @Transient
    private List<String> vacations;

    @Transient
    private List<String> remarks;
}