package com.test.controller;

import com.common.aliyun.OssUtils;
import com.common.util.QRCodeUtils;
import com.core.result.Result;
import com.core.result.ResultGenerator;
import com.dao.common.schedule.DynamicScheduleService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * @author 潘维吉
 * @date 2019/11/20 9:20
 * @email 406798106@qq.com
 * @description
 */
@RestController
@RequestMapping("/test")
@Validated
@Slf4j
public class TestController {

    @Resource
    private DynamicScheduleService dynamicScheduleService;

    @Resource
    private QRCodeUtils qrCodeUtils;

    @Resource
    private OssUtils ossUtils;

    /**
     * 调用工具类生成二维码并输出到输出流中
     */
    @GetMapping("/qr_code")
    public void qrCode(String codeContent, HttpServletResponse response) {
        try {
            System.out.println(qrCodeUtils.encodeDevice(codeContent,
                    Map.of("name", "消防栓", "code", "XHS-13-031", "location", "12#2-18层"),
                    1800, 900));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * oss下载多文打统一打zip包
     */
    @GetMapping("/zip")
    public void zip(HttpServletRequest request, HttpServletResponse response) {
        ossUtils.zipFilesDown("qr_code/748323084095721472.jpg , qr_code/748322617693310976.jpg", "test", request, response);
    }

    @GetMapping("/start")
    public Result start() {
        return ResultGenerator.success(dynamicScheduleService.add(1, () -> {
            // 调用具体Service方法执行的任务
            log.info("添加动态定时执行任务");
        }, "*/10 * * * * *"), "添加动态定时任务");
    }

    @GetMapping("/update")
    public Result update() {
        return ResultGenerator.success(dynamicScheduleService.update(1, () -> {
            // 调用具体Service方法执行的任务
            log.info("更新动态定时执行任务1");
        }, "*/15 * * * * *"), "更新动态定时任务");
    }

    @GetMapping("/stop")
    public Result stop() {
        return ResultGenerator.success(dynamicScheduleService.remove(1), "取消定时任务");
    }
}
