create table if not exists access_control_anjubao
(
    id             bigint auto_increment comment '主键'
        primary key,
    name           varchar(20)                        null comment '门禁名称',
    url            varchar(300)                       null comment '请求地址',
    community_id   bigint                             null comment '社区Id',
    community_code varchar(20)                        null comment '社区编号',
    building_code  varchar(20)                        null comment '楼栋编号',
    unit_code      varchar(20)                        null comment '单元编号',
    estate_code    varchar(20)                        null comment '房产编号',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '安居宝大门';

create table if not exists access_control_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    community_id bigint                             null comment '小区Id',
    ac_firm_id   bigint                             null comment '门禁厂商Id',
    access_code  varchar(50)                        null comment '门禁社区号',
    deleted      tinyint  default 0                 null comment '删除标记',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '门禁小区管理表';

create table if not exists access_control_community_unit
(
    id           bigint auto_increment comment '主键'
        primary key,
    community_id bigint null comment '小区Id',
    unit_id      bigint null comment '单元Id'
)
    comment '小区下门禁单元管理表';

create table if not exists access_control_firms
(
    id         bigint auto_increment comment '主键'
        primary key,
    code       varchar(20)                        null comment '厂商编号',
    name       varchar(20)                        null comment '厂商名称',
    phone      varchar(20)                        null comment '联系电话',
    introduce  varchar(500)                       null comment '厂商介绍',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '门禁厂商表';

create table if not exists access_control_record
(
    id           bigint auto_increment comment '主键'
        primary key,
    user_id      bigint                             null comment '用户Id',
    community_id bigint                             null comment '社区ID',
    door_id      bigint                             null comment '单元Id 或者 大门Id',
    ac_firm_id   bigint                             null comment '门禁厂商Id',
    visitor_id   bigint                             null comment '访客记录Id',
    open_type    int                                null comment '开门类型: 1业主开门, 2访客通行等',
    open_mode    int                                null comment '开门方式: 1蓝牙, 2二维码',
    door_type    int                                null comment '门禁类型:1 大门 2 单元门',
    pass_data    varchar(500)                       null comment '开门码',
    visitor_name varchar(20)                        null comment '访客姓名',
    firm_name    varchar(20)                        null comment '厂商名称',
    door_name    varchar(50)                        null comment '门禁名称',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '门禁开门记录';

create table if not exists access_control_visitor
(
    id             bigint auto_increment comment '主键'
        primary key,
    user_id        bigint                             null comment '用户Id',
    door_id        bigint                             null comment '单元Id 或者 大门Id',
    community_id   bigint                             null comment '社区Id',
    qrcode         varchar(50)                        null comment '二维码编号',
    visitor_name   varchar(20)                        null comment '访客姓名',
    visitor_sex    int                                null comment '访客性别 1:男 2:女 3:中性 4:未知',
    start_time     datetime                           null comment '开始时间',
    end_time       datetime                           null comment '结束时间',
    door_name      varchar(500)                       null comment '门禁名称',
    visitor_reason varchar(100)                       null comment '来访原因',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '访客记录表';

create table if not exists act_custom_process
(
    id               bigint auto_increment comment '主键'
        primary key,
    proc_def_id      varchar(200)                       null comment '流程定义Id',
    proc_def_key     varchar(200)                       null comment '路程定义key',
    business_type_id int                                null comment '业务类型Id 1.报修 2.投诉 3.集团验房 4.业主验房 5.家政服务 6.设备维修',
    is_current       bit                                null comment '最新版本',
    version          int      default 0                 null comment '数据版本号',
    deleted          tinyint  default 0                 null comment '删除标记',
    company_id       bigint                             null comment '公司id',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '流程表单';

create table if not exists act_custom_process_form_property
(
    id               bigint auto_increment comment '主键'
        primary key,
    business_type_id int          null comment '业务类型',
    property_id      varchar(200) null comment '属性id',
    property_name    varchar(100) null comment '属性名称'
)
    comment '流程表单';

create table if not exists act_custom_process_log
(
    id                 bigint auto_increment comment '主键'
        primary key,
    parent_id          bigint              null comment '前置审批节点id',
    business_type_id   int                 null comment '业务类型Id 1.报修 2.投诉 3.集团验房 4.业主验房 5.家政服务 6.设备维修',
    community_id       bigint              null comment '社区id',
    area_id            bigint default 0    null comment '区域id',
    proc_def_id        varchar(100)        null comment '流程定义id',
    proc_id            varchar(100)        null comment '流程实例Id',
    task_def_id        varchar(100)        null comment '任务Id',
    task_name          varchar(100)        null comment '当前任务名称',
    opreate            varchar(100)        null comment '操作',
    opreate_name       varchar(100)        null comment '操作名称',
    comment            varchar(1000)       null comment '审批说明',
    source_task_def_id varchar(100)        null comment '源头任务id',
    source_task_name   varchar(100)        null comment '源头任务名称',
    rejct_task_def_id  varchar(100)        null comment '驳回任务id',
    audit_user_id      bigint              null comment '审批人名称',
    audit_date         datetime            null comment '操作时间',
    is_current         bit                 null comment '是否最新流程',
    is_reject          bit                 null comment '是否驳回操作',
    is_finish          bit                 null comment '当前任务是否完成',
    task_limit_date    datetime            null comment '当前任务到期时间',
    is_overdue         bit                 null comment '是否超时',
    overdue_minute     int                 null comment '超时分钟数',
    ct_date            datetime            null comment '流程进入时间',
    deleted            bit    default b'0' null comment '删除标记'
)
    comment '流程审批日志';

create index idx_act_custom_process_log_audit_user_id
    on act_custom_process_log (audit_user_id);

create index idx_act_custom_process_log_business_type_id
    on act_custom_process_log (business_type_id);

create index idx_act_custom_process_log_community_id
    on act_custom_process_log (community_id);

create index idx_act_custom_process_log_is_finish
    on act_custom_process_log (is_finish);

create index idx_act_custom_process_log_proc_id
    on act_custom_process_log (proc_id);

create table if not exists act_custom_process_log_audit_object
(
    id          bigint auto_increment comment '主键'
        primary key,
    proc_id     varchar(100) null comment '流程实例id',
    log_id      bigint       null comment '日志id',
    object_type int          null comment '对象类型 1:人员 2:角色 3:外协部门',
    object_id   bigint       null comment '对象id',
    is_finish   bit          null comment '当前任务是否完成'
)
    comment '审批日志审批人列表';

create table if not exists act_custom_process_log_img
(
    id     bigint auto_increment comment '主键'
        primary key,
    log_id bigint       null comment '日志id',
    path   varchar(200) null comment '路径'
)
    comment '流程审批日志图片';

create table if not exists act_custom_process_log_remark
(
    id              bigint auto_increment comment '主键'
        primary key,
    log_id          bigint        null comment '日志id',
    content         varchar(1000) null comment '回复内容',
    company_user_id bigint        null comment '公司用户id',
    ct_date         datetime      null comment '创建时间'
)
    comment '流程内部备注';

create table if not exists act_custom_process_log_reminder
(
    id                bigint auto_increment comment '主键'
        primary key,
    log_id            bigint        null comment '日志id',
    community_user_id bigint        null comment '社区用户id',
    msg               varchar(1000) null comment '回复内容',
    ct_date           datetime      null comment '创建时间'
)
    comment '催单信息';

create table if not exists act_custom_process_log_reply
(
    id              bigint auto_increment comment '主键'
        primary key,
    log_id          bigint        null comment '日志id',
    msg             varchar(1000) null comment '回复内容',
    company_user_id bigint        null comment '公司用户id',
    image_list      varchar(2000) null comment '图片列表',
    ct_date         datetime      null comment '创建时间'
)
    comment '流程中对业主的回复';

create table if not exists act_custom_process_task
(
    id                     bigint auto_increment comment '主键'
        primary key,
    custom_process_id      bigint       null comment '自定义流程id',
    proc_def_id            varchar(200) null comment '流程定义Id',
    proc_task_def_id       varchar(200) null comment '流程任务定义Id',
    proc_task_def_name     varchar(200) null comment '流程任务定义名称',
    audit_user_rule        int          null comment '审批人规则 1.指定人选 2.默认配置  3:业主',
    audit_pre_task_def_id  varchar(200) null comment '前置审批节点',
    reject_audit_user_rule int          null comment '驳回审批人规则 1.审批人 2.默认配置',
    allow_reject           bit          null comment '当前流程是否允许驳回'
)
    comment '流程任务定义';

create table if not exists act_custom_process_task_audit_object
(
    id                bigint auto_increment comment '主键'
        primary key,
    task_community_id bigint                             null comment '社区任务id',
    object_type       int                                null comment '对象类型 1:人员 2:角色 3:外协部门',
    object_id         bigint                             null comment '对象id',
    company_id        bigint                             null comment '公司id',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '任务审批对象定义表';

create table if not exists act_custom_process_task_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    community_id bigint           null comment '社区id',
    task_id      bigint           null comment '任务id',
    time_limit   int default 0    null comment '时限(分钟)',
    send_sms     bit default b'1' null comment '是否发送短信提醒'
)
    comment '流程任务定义';

create table if not exists act_custom_process_task_out
(
    id                 bigint auto_increment comment '主键'
        primary key,
    proc_def_id        varchar(200)  null comment '流程定义Id',
    proc_task_def_id   varchar(200)  null comment '流程任务定义Id',
    target_task_def_id varchar(200)  null comment '目标流程id',
    out_id             varchar(200)  null comment '出口ID',
    out_name           varchar(200)  null comment '出口名称',
    is_forward         bit           null comment '是否为正向流程',
    show_order         int default 0 null comment '排序号'
)
    comment '任务出口';

create table if not exists act_custom_process_task_out_form
(
    id               bigint auto_increment comment '主键'
        primary key,
    task_out_id      bigint null comment '出口Id',
    form_property_id bigint null comment '表单属性Id',
    is_visiable      bit    null comment '是否可见',
    is_editable      bit    null comment '是否可编辑',
    is_required      bit    null comment '是否必填'
)
    comment '流程出口需要控制表单项';

create table if not exists act_evt_log
(
    LOG_NR_       bigint auto_increment
        primary key,
    TYPE_         varchar(64)       null,
    PROC_DEF_ID_  varchar(64)       null,
    PROC_INST_ID_ varchar(64)       null,
    EXECUTION_ID_ varchar(64)       null,
    TASK_ID_      varchar(64)       null,
    TIME_STAMP_   timestamp(3)      not null,
    USER_ID_      varchar(255)      null,
    DATA_         longblob          null,
    LOCK_OWNER_   varchar(255)      null,
    LOCK_TIME_    timestamp(3)      null,
    IS_PROCESSED_ tinyint default 0 null
)
    collate = utf8_bin;

create table if not exists act_ge_property
(
    NAME_  varchar(64)  not null
        primary key,
    VALUE_ varchar(300) null,
    REV_   int          null
)
    collate = utf8_bin;

create table if not exists act_hi_actinst
(
    ID_                varchar(64)             not null
        primary key,
    PROC_DEF_ID_       varchar(64)             not null,
    PROC_INST_ID_      varchar(64)             not null,
    EXECUTION_ID_      varchar(64)             not null,
    ACT_ID_            varchar(255)            not null,
    TASK_ID_           varchar(64)             null,
    CALL_PROC_INST_ID_ varchar(64)             null,
    ACT_NAME_          varchar(255)            null,
    ACT_TYPE_          varchar(255)            not null,
    ASSIGNEE_          varchar(255)            null,
    START_TIME_        datetime(3)             not null,
    END_TIME_          datetime(3)             null,
    DURATION_          bigint                  null,
    DELETE_REASON_     varchar(4000)           null,
    TENANT_ID_         varchar(255) default '' null
)
    collate = utf8_bin;

create index ACT_IDX_HI_ACT_INST_END
    on act_hi_actinst (END_TIME_);

create index ACT_IDX_HI_ACT_INST_EXEC
    on act_hi_actinst (EXECUTION_ID_, ACT_ID_);

create index ACT_IDX_HI_ACT_INST_PROCINST
    on act_hi_actinst (PROC_INST_ID_, ACT_ID_);

create index ACT_IDX_HI_ACT_INST_START
    on act_hi_actinst (START_TIME_);

create table if not exists act_hi_attachment
(
    ID_           varchar(64)   not null
        primary key,
    REV_          int           null,
    USER_ID_      varchar(255)  null,
    NAME_         varchar(255)  null,
    DESCRIPTION_  varchar(4000) null,
    TYPE_         varchar(255)  null,
    TASK_ID_      varchar(64)   null,
    PROC_INST_ID_ varchar(64)   null,
    URL_          varchar(4000) null,
    CONTENT_ID_   varchar(64)   null,
    TIME_         datetime(3)   null
)
    collate = utf8_bin;

create table if not exists act_hi_comment
(
    ID_           varchar(64)   not null
        primary key,
    TYPE_         varchar(255)  null,
    TIME_         datetime(3)   not null,
    USER_ID_      varchar(255)  null,
    TASK_ID_      varchar(64)   null,
    PROC_INST_ID_ varchar(64)   null,
    ACTION_       varchar(255)  null,
    MESSAGE_      varchar(4000) null,
    FULL_MSG_     longblob      null
)
    collate = utf8_bin;

create table if not exists act_hi_detail
(
    ID_           varchar(64)   not null
        primary key,
    TYPE_         varchar(255)  not null,
    PROC_INST_ID_ varchar(64)   null,
    EXECUTION_ID_ varchar(64)   null,
    TASK_ID_      varchar(64)   null,
    ACT_INST_ID_  varchar(64)   null,
    NAME_         varchar(255)  not null,
    VAR_TYPE_     varchar(255)  null,
    REV_          int           null,
    TIME_         datetime(3)   not null,
    BYTEARRAY_ID_ varchar(64)   null,
    DOUBLE_       double        null,
    LONG_         bigint        null,
    TEXT_         varchar(4000) null,
    TEXT2_        varchar(4000) null
)
    collate = utf8_bin;

create index ACT_IDX_HI_DETAIL_ACT_INST
    on act_hi_detail (ACT_INST_ID_);

create index ACT_IDX_HI_DETAIL_NAME
    on act_hi_detail (NAME_);

create index ACT_IDX_HI_DETAIL_PROC_INST
    on act_hi_detail (PROC_INST_ID_);

create index ACT_IDX_HI_DETAIL_TASK_ID
    on act_hi_detail (TASK_ID_);

create index ACT_IDX_HI_DETAIL_TIME
    on act_hi_detail (TIME_);

create table if not exists act_hi_identitylink
(
    ID_           varchar(64)  not null
        primary key,
    GROUP_ID_     varchar(255) null,
    TYPE_         varchar(255) null,
    USER_ID_      varchar(255) null,
    TASK_ID_      varchar(64)  null,
    PROC_INST_ID_ varchar(64)  null
)
    collate = utf8_bin;

create index ACT_IDX_HI_IDENT_LNK_PROCINST
    on act_hi_identitylink (PROC_INST_ID_);

create index ACT_IDX_HI_IDENT_LNK_TASK
    on act_hi_identitylink (TASK_ID_);

create index ACT_IDX_HI_IDENT_LNK_USER
    on act_hi_identitylink (USER_ID_);

create table if not exists act_hi_procinst
(
    ID_                        varchar(64)             not null
        primary key,
    PROC_INST_ID_              varchar(64)             not null,
    BUSINESS_KEY_              varchar(255)            null,
    PROC_DEF_ID_               varchar(64)             not null,
    START_TIME_                datetime(3)             not null,
    END_TIME_                  datetime(3)             null,
    DURATION_                  bigint                  null,
    START_USER_ID_             varchar(255)            null,
    START_ACT_ID_              varchar(255)            null,
    END_ACT_ID_                varchar(255)            null,
    SUPER_PROCESS_INSTANCE_ID_ varchar(64)             null,
    DELETE_REASON_             varchar(4000)           null,
    TENANT_ID_                 varchar(255) default '' null,
    NAME_                      varchar(255)            null,
    constraint PROC_INST_ID_
        unique (PROC_INST_ID_)
)
    collate = utf8_bin;

create index ACT_IDX_HI_PRO_INST_END
    on act_hi_procinst (END_TIME_);

create index ACT_IDX_HI_PRO_I_BUSKEY
    on act_hi_procinst (BUSINESS_KEY_);

create table if not exists act_hi_taskinst
(
    ID_             varchar(64)             not null
        primary key,
    PROC_DEF_ID_    varchar(64)             null,
    TASK_DEF_KEY_   varchar(255)            null,
    PROC_INST_ID_   varchar(64)             null,
    EXECUTION_ID_   varchar(64)             null,
    NAME_           varchar(255)            null,
    PARENT_TASK_ID_ varchar(64)             null,
    DESCRIPTION_    varchar(4000)           null,
    OWNER_          varchar(255)            null,
    ASSIGNEE_       varchar(255)            null,
    START_TIME_     datetime(3)             not null,
    CLAIM_TIME_     datetime(3)             null,
    END_TIME_       datetime(3)             null,
    DURATION_       bigint                  null,
    DELETE_REASON_  varchar(4000)           null,
    PRIORITY_       int                     null,
    DUE_DATE_       datetime(3)             null,
    FORM_KEY_       varchar(255)            null,
    CATEGORY_       varchar(255)            null,
    TENANT_ID_      varchar(255) default '' null
)
    collate = utf8_bin;

create index ACT_IDX_HI_TASK_INST_PROCINST
    on act_hi_taskinst (PROC_INST_ID_);

create table if not exists act_hi_varinst
(
    ID_                varchar(64)   not null
        primary key,
    PROC_INST_ID_      varchar(64)   null,
    EXECUTION_ID_      varchar(64)   null,
    TASK_ID_           varchar(64)   null,
    NAME_              varchar(255)  not null,
    VAR_TYPE_          varchar(100)  null,
    REV_               int           null,
    BYTEARRAY_ID_      varchar(64)   null,
    DOUBLE_            double        null,
    LONG_              bigint        null,
    TEXT_              varchar(4000) null,
    TEXT2_             varchar(4000) null,
    CREATE_TIME_       datetime(3)   null,
    LAST_UPDATED_TIME_ datetime(3)   null
)
    collate = utf8_bin;

create index ACT_IDX_HI_PROCVAR_NAME_TYPE
    on act_hi_varinst (NAME_, VAR_TYPE_);

create index ACT_IDX_HI_PROCVAR_PROC_INST
    on act_hi_varinst (PROC_INST_ID_);

create index ACT_IDX_HI_PROCVAR_TASK_ID
    on act_hi_varinst (TASK_ID_);

create table if not exists act_id_group
(
    ID_   varchar(64)  not null
        primary key,
    REV_  int          null,
    NAME_ varchar(255) null,
    TYPE_ varchar(255) null
)
    collate = utf8_bin;

create table if not exists act_id_info
(
    ID_        varchar(64)  not null
        primary key,
    REV_       int          null,
    USER_ID_   varchar(64)  null,
    TYPE_      varchar(64)  null,
    KEY_       varchar(255) null,
    VALUE_     varchar(255) null,
    PASSWORD_  longblob     null,
    PARENT_ID_ varchar(255) null
)
    collate = utf8_bin;

create table if not exists act_id_user
(
    ID_         varchar(64)  not null
        primary key,
    REV_        int          null,
    FIRST_      varchar(255) null,
    LAST_       varchar(255) null,
    EMAIL_      varchar(255) null,
    PWD_        varchar(255) null,
    PICTURE_ID_ varchar(64)  null
)
    collate = utf8_bin;

create table if not exists act_id_membership
(
    USER_ID_  varchar(64) not null,
    GROUP_ID_ varchar(64) not null,
    primary key (USER_ID_, GROUP_ID_),
    constraint ACT_FK_MEMB_GROUP
        foreign key (GROUP_ID_) references act_id_group (ID_),
    constraint ACT_FK_MEMB_USER
        foreign key (USER_ID_) references act_id_user (ID_)
)
    collate = utf8_bin;

create table if not exists act_re_deployment
(
    ID_             varchar(64)             not null
        primary key,
    NAME_           varchar(255)            null,
    CATEGORY_       varchar(255)            null,
    KEY_            varchar(255)            null,
    TENANT_ID_      varchar(255) default '' null,
    DEPLOY_TIME_    timestamp(3)            null,
    ENGINE_VERSION_ varchar(255)            null
)
    collate = utf8_bin;

create table if not exists act_ge_bytearray
(
    ID_            varchar(64)  not null
        primary key,
    REV_           int          null,
    NAME_          varchar(255) null,
    DEPLOYMENT_ID_ varchar(64)  null,
    BYTES_         longblob     null,
    GENERATED_     tinyint      null,
    constraint ACT_FK_BYTEARR_DEPL
        foreign key (DEPLOYMENT_ID_) references act_re_deployment (ID_)
)
    collate = utf8_bin;

create table if not exists act_re_model
(
    ID_                           varchar(64)             not null
        primary key,
    REV_                          int                     null,
    NAME_                         varchar(255)            null,
    KEY_                          varchar(255)            null,
    CATEGORY_                     varchar(255)            null,
    CREATE_TIME_                  timestamp(3)            null,
    LAST_UPDATE_TIME_             timestamp(3)            null,
    VERSION_                      int                     null,
    META_INFO_                    varchar(4000)           null,
    DEPLOYMENT_ID_                varchar(64)             null,
    EDITOR_SOURCE_VALUE_ID_       varchar(64)             null,
    EDITOR_SOURCE_EXTRA_VALUE_ID_ varchar(64)             null,
    TENANT_ID_                    varchar(255) default '' null,
    constraint ACT_FK_MODEL_DEPLOYMENT
        foreign key (DEPLOYMENT_ID_) references act_re_deployment (ID_),
    constraint ACT_FK_MODEL_SOURCE
        foreign key (EDITOR_SOURCE_VALUE_ID_) references act_ge_bytearray (ID_),
    constraint ACT_FK_MODEL_SOURCE_EXTRA
        foreign key (EDITOR_SOURCE_EXTRA_VALUE_ID_) references act_ge_bytearray (ID_)
)
    collate = utf8_bin;

create table if not exists act_re_procdef
(
    ID_                     varchar(64)             not null
        primary key,
    REV_                    int                     null,
    CATEGORY_               varchar(255)            null,
    NAME_                   varchar(255)            null,
    KEY_                    varchar(255)            not null,
    VERSION_                int                     not null,
    DEPLOYMENT_ID_          varchar(64)             null,
    RESOURCE_NAME_          varchar(4000)           null,
    DGRM_RESOURCE_NAME_     varchar(4000)           null,
    DESCRIPTION_            varchar(4000)           null,
    HAS_START_FORM_KEY_     tinyint                 null,
    HAS_GRAPHICAL_NOTATION_ tinyint                 null,
    SUSPENSION_STATE_       int                     null,
    TENANT_ID_              varchar(255) default '' null,
    ENGINE_VERSION_         varchar(255)            null,
    constraint ACT_UNIQ_PROCDEF
        unique (KEY_, VERSION_, TENANT_ID_)
)
    collate = utf8_bin;

create table if not exists act_procdef_info
(
    ID_           varchar(64) not null
        primary key,
    PROC_DEF_ID_  varchar(64) not null,
    REV_          int         null,
    INFO_JSON_ID_ varchar(64) null,
    constraint ACT_UNIQ_INFO_PROCDEF
        unique (PROC_DEF_ID_),
    constraint ACT_FK_INFO_JSON_BA
        foreign key (INFO_JSON_ID_) references act_ge_bytearray (ID_),
    constraint ACT_FK_INFO_PROCDEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_)
)
    collate = utf8_bin;

create index ACT_IDX_INFO_PROCDEF
    on act_procdef_info (PROC_DEF_ID_);

create table if not exists act_ru_execution
(
    ID_                   varchar(64)             not null
        primary key,
    REV_                  int                     null,
    PROC_INST_ID_         varchar(64)             null,
    BUSINESS_KEY_         varchar(255)            null,
    PARENT_ID_            varchar(64)             null,
    PROC_DEF_ID_          varchar(64)             null,
    SUPER_EXEC_           varchar(64)             null,
    ROOT_PROC_INST_ID_    varchar(64)             null,
    ACT_ID_               varchar(255)            null,
    IS_ACTIVE_            tinyint                 null,
    IS_CONCURRENT_        tinyint                 null,
    IS_SCOPE_             tinyint                 null,
    IS_EVENT_SCOPE_       tinyint                 null,
    IS_MI_ROOT_           tinyint                 null,
    SUSPENSION_STATE_     int                     null,
    CACHED_ENT_STATE_     int                     null,
    TENANT_ID_            varchar(255) default '' null,
    NAME_                 varchar(255)            null,
    START_TIME_           datetime(3)             null,
    START_USER_ID_        varchar(255)            null,
    LOCK_TIME_            timestamp(3)            null,
    IS_COUNT_ENABLED_     tinyint                 null,
    EVT_SUBSCR_COUNT_     int                     null,
    TASK_COUNT_           int                     null,
    JOB_COUNT_            int                     null,
    TIMER_JOB_COUNT_      int                     null,
    SUSP_JOB_COUNT_       int                     null,
    DEADLETTER_JOB_COUNT_ int                     null,
    VAR_COUNT_            int                     null,
    ID_LINK_COUNT_        int                     null,
    constraint ACT_FK_EXE_PARENT
        foreign key (PARENT_ID_) references act_ru_execution (ID_)
            on delete cascade,
    constraint ACT_FK_EXE_PROCDEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_),
    constraint ACT_FK_EXE_PROCINST
        foreign key (PROC_INST_ID_) references act_ru_execution (ID_)
            on update cascade on delete cascade,
    constraint ACT_FK_EXE_SUPER
        foreign key (SUPER_EXEC_) references act_ru_execution (ID_)
            on delete cascade
)
    collate = utf8_bin;

create table if not exists act_ru_deadletter_job
(
    ID_                  varchar(64)             not null
        primary key,
    REV_                 int                     null,
    TYPE_                varchar(255)            not null,
    EXCLUSIVE_           tinyint(1)              null,
    EXECUTION_ID_        varchar(64)             null,
    PROCESS_INSTANCE_ID_ varchar(64)             null,
    PROC_DEF_ID_         varchar(64)             null,
    EXCEPTION_STACK_ID_  varchar(64)             null,
    EXCEPTION_MSG_       varchar(4000)           null,
    DUEDATE_             timestamp(3)            null,
    REPEAT_              varchar(255)            null,
    HANDLER_TYPE_        varchar(255)            null,
    HANDLER_CFG_         varchar(4000)           null,
    TENANT_ID_           varchar(255) default '' null,
    constraint ACT_FK_DEADLETTER_JOB_EXCEPTION
        foreign key (EXCEPTION_STACK_ID_) references act_ge_bytearray (ID_),
    constraint ACT_FK_DEADLETTER_JOB_EXECUTION
        foreign key (EXECUTION_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_DEADLETTER_JOB_PROCESS_INSTANCE
        foreign key (PROCESS_INSTANCE_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_DEADLETTER_JOB_PROC_DEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_)
)
    collate = utf8_bin;

create table if not exists act_ru_event_subscr
(
    ID_            varchar(64)                               not null
        primary key,
    REV_           int                                       null,
    EVENT_TYPE_    varchar(255)                              not null,
    EVENT_NAME_    varchar(255)                              null,
    EXECUTION_ID_  varchar(64)                               null,
    PROC_INST_ID_  varchar(64)                               null,
    ACTIVITY_ID_   varchar(64)                               null,
    CONFIGURATION_ varchar(255)                              null,
    CREATED_       timestamp(3) default CURRENT_TIMESTAMP(3) not null,
    PROC_DEF_ID_   varchar(64)                               null,
    TENANT_ID_     varchar(255) default ''                   null,
    constraint ACT_FK_EVENT_EXEC
        foreign key (EXECUTION_ID_) references act_ru_execution (ID_)
)
    collate = utf8_bin;

create index ACT_IDX_EVENT_SUBSCR_CONFIG_
    on act_ru_event_subscr (CONFIGURATION_);

create index ACT_IDC_EXEC_ROOT
    on act_ru_execution (ROOT_PROC_INST_ID_);

create index ACT_IDX_EXEC_BUSKEY
    on act_ru_execution (BUSINESS_KEY_);

create table if not exists act_ru_job
(
    ID_                  varchar(64)             not null
        primary key,
    REV_                 int                     null,
    TYPE_                varchar(255)            not null,
    LOCK_EXP_TIME_       timestamp(3)            null,
    LOCK_OWNER_          varchar(255)            null,
    EXCLUSIVE_           tinyint(1)              null,
    EXECUTION_ID_        varchar(64)             null,
    PROCESS_INSTANCE_ID_ varchar(64)             null,
    PROC_DEF_ID_         varchar(64)             null,
    RETRIES_             int                     null,
    EXCEPTION_STACK_ID_  varchar(64)             null,
    EXCEPTION_MSG_       varchar(4000)           null,
    DUEDATE_             timestamp(3)            null,
    REPEAT_              varchar(255)            null,
    HANDLER_TYPE_        varchar(255)            null,
    HANDLER_CFG_         varchar(4000)           null,
    TENANT_ID_           varchar(255) default '' null,
    constraint ACT_FK_JOB_EXCEPTION
        foreign key (EXCEPTION_STACK_ID_) references act_ge_bytearray (ID_),
    constraint ACT_FK_JOB_EXECUTION
        foreign key (EXECUTION_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_JOB_PROCESS_INSTANCE
        foreign key (PROCESS_INSTANCE_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_JOB_PROC_DEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_)
)
    collate = utf8_bin;

create table if not exists act_ru_suspended_job
(
    ID_                  varchar(64)             not null
        primary key,
    REV_                 int                     null,
    TYPE_                varchar(255)            not null,
    EXCLUSIVE_           tinyint(1)              null,
    EXECUTION_ID_        varchar(64)             null,
    PROCESS_INSTANCE_ID_ varchar(64)             null,
    PROC_DEF_ID_         varchar(64)             null,
    RETRIES_             int                     null,
    EXCEPTION_STACK_ID_  varchar(64)             null,
    EXCEPTION_MSG_       varchar(4000)           null,
    DUEDATE_             timestamp(3)            null,
    REPEAT_              varchar(255)            null,
    HANDLER_TYPE_        varchar(255)            null,
    HANDLER_CFG_         varchar(4000)           null,
    TENANT_ID_           varchar(255) default '' null,
    constraint ACT_FK_SUSPENDED_JOB_EXCEPTION
        foreign key (EXCEPTION_STACK_ID_) references act_ge_bytearray (ID_),
    constraint ACT_FK_SUSPENDED_JOB_EXECUTION
        foreign key (EXECUTION_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_SUSPENDED_JOB_PROCESS_INSTANCE
        foreign key (PROCESS_INSTANCE_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_SUSPENDED_JOB_PROC_DEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_)
)
    collate = utf8_bin;

create table if not exists act_ru_task
(
    ID_               varchar(64)             not null
        primary key,
    REV_              int                     null,
    EXECUTION_ID_     varchar(64)             null,
    PROC_INST_ID_     varchar(64)             null,
    PROC_DEF_ID_      varchar(64)             null,
    NAME_             varchar(255)            null,
    PARENT_TASK_ID_   varchar(64)             null,
    DESCRIPTION_      varchar(4000)           null,
    TASK_DEF_KEY_     varchar(255)            null,
    OWNER_            varchar(255)            null,
    ASSIGNEE_         varchar(255)            null,
    DELEGATION_       varchar(64)             null,
    PRIORITY_         int                     null,
    CREATE_TIME_      timestamp(3)            null,
    DUE_DATE_         datetime(3)             null,
    CATEGORY_         varchar(255)            null,
    SUSPENSION_STATE_ int                     null,
    TENANT_ID_        varchar(255) default '' null,
    FORM_KEY_         varchar(255)            null,
    CLAIM_TIME_       datetime(3)             null,
    constraint ACT_FK_TASK_EXE
        foreign key (EXECUTION_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_TASK_PROCDEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_),
    constraint ACT_FK_TASK_PROCINST
        foreign key (PROC_INST_ID_) references act_ru_execution (ID_)
)
    collate = utf8_bin;

create table if not exists act_ru_identitylink
(
    ID_           varchar(64)  not null
        primary key,
    REV_          int          null,
    GROUP_ID_     varchar(255) null,
    TYPE_         varchar(255) null,
    USER_ID_      varchar(255) null,
    TASK_ID_      varchar(64)  null,
    PROC_INST_ID_ varchar(64)  null,
    PROC_DEF_ID_  varchar(64)  null,
    constraint ACT_FK_ATHRZ_PROCEDEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_),
    constraint ACT_FK_IDL_PROCINST
        foreign key (PROC_INST_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_TSKASS_TASK
        foreign key (TASK_ID_) references act_ru_task (ID_)
)
    collate = utf8_bin;

create index ACT_IDX_ATHRZ_PROCEDEF
    on act_ru_identitylink (PROC_DEF_ID_);

create index ACT_IDX_IDENT_LNK_GROUP
    on act_ru_identitylink (GROUP_ID_);

create index ACT_IDX_IDENT_LNK_USER
    on act_ru_identitylink (USER_ID_);

create index ACT_IDX_TASK_CREATE
    on act_ru_task (CREATE_TIME_);

create table if not exists act_ru_timer_job
(
    ID_                  varchar(64)             not null
        primary key,
    REV_                 int                     null,
    TYPE_                varchar(255)            not null,
    LOCK_EXP_TIME_       timestamp(3)            null,
    LOCK_OWNER_          varchar(255)            null,
    EXCLUSIVE_           tinyint(1)              null,
    EXECUTION_ID_        varchar(64)             null,
    PROCESS_INSTANCE_ID_ varchar(64)             null,
    PROC_DEF_ID_         varchar(64)             null,
    RETRIES_             int                     null,
    EXCEPTION_STACK_ID_  varchar(64)             null,
    EXCEPTION_MSG_       varchar(4000)           null,
    DUEDATE_             timestamp(3)            null,
    REPEAT_              varchar(255)            null,
    HANDLER_TYPE_        varchar(255)            null,
    HANDLER_CFG_         varchar(4000)           null,
    TENANT_ID_           varchar(255) default '' null,
    constraint ACT_FK_TIMER_JOB_EXCEPTION
        foreign key (EXCEPTION_STACK_ID_) references act_ge_bytearray (ID_),
    constraint ACT_FK_TIMER_JOB_EXECUTION
        foreign key (EXECUTION_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_TIMER_JOB_PROCESS_INSTANCE
        foreign key (PROCESS_INSTANCE_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_TIMER_JOB_PROC_DEF
        foreign key (PROC_DEF_ID_) references act_re_procdef (ID_)
)
    collate = utf8_bin;

create table if not exists act_ru_variable
(
    ID_           varchar(64)   not null
        primary key,
    REV_          int           null,
    TYPE_         varchar(255)  not null,
    NAME_         varchar(255)  not null,
    EXECUTION_ID_ varchar(64)   null,
    PROC_INST_ID_ varchar(64)   null,
    TASK_ID_      varchar(64)   null,
    BYTEARRAY_ID_ varchar(64)   null,
    DOUBLE_       double        null,
    LONG_         bigint        null,
    TEXT_         varchar(4000) null,
    TEXT2_        varchar(4000) null,
    constraint ACT_FK_VAR_BYTEARRAY
        foreign key (BYTEARRAY_ID_) references act_ge_bytearray (ID_),
    constraint ACT_FK_VAR_EXE
        foreign key (EXECUTION_ID_) references act_ru_execution (ID_),
    constraint ACT_FK_VAR_PROCINST
        foreign key (PROC_INST_ID_) references act_ru_execution (ID_)
)
    collate = utf8_bin;

create index ACT_IDX_VARIABLE_TASK_ID
    on act_ru_variable (TASK_ID_);

create table if not exists activity
(
    id                    bigint auto_increment comment '主键'
        primary key,
    title                 varchar(100)                         null comment '标题',
    activity_type         int                                  null comment '活动类型  码表',
    summary               varchar(500)                         null comment '摘要',
    thumbnail             varchar(255)                         null comment '列表缩略图',
    content               text                                 null comment '内容',
    content_img           varchar(255)                         null comment '内容主图',
    phone                 varchar(20)                          null comment '活动联系电话',
    address               varchar(200)                         null comment '活动地址',
    top                   tinyint(1) default 0                 null comment '是否置顶',
    allow_comment         tinyint(1) default 1                 null comment '是否允许评论 0：不能评论 1：可以评论',
    need_confirm          tinyint(1) default 0                 null comment '是否需要确认 0：不需要 1：需要',
    allow_push            tinyint(1) default 0                 null comment '是否允许推送 0：不推送 1：推送',
    allow_enroll          tinyint(1) default 0                 null comment '是否允许报名',
    allow_enroll_max_num  int        default 0                 null comment '允许最大报名人数,0为不限制',
    allow_arrearage       tinyint(1) default 1                 null comment '是否允许拖欠物业费报名',
    allow_arrearage_mouth int        default 0                 null comment '允许欠费报名月数,默认0,不允许欠费',
    start_time            datetime                             null comment '活动开始时间',
    end_time              datetime                             null comment '活动结束时间',
    enroll_start_time     datetime                             null comment '报名开始时间',
    enroll_end_time       datetime                             null comment '报名结束时间',
    enroll_spend_integral int        default 0                 null comment '报名所需积分',
    enroll_type           int                                  null comment '报名类型 1:人 2:组',
    enroll_num            int        default 0                 null comment '已报名数量',
    view_num              int        default 0                 null comment '浏览数',
    comment_num           int        default 0                 null comment '评论数',
    praise_num            int        default 0                 null comment '点赞数',
    status                int        default 0                 null comment '状态 0：新建 1：已发布 2：已撤销',
    remark                varchar(1000)                        null comment '备注',
    show_order            int        default 0                 null comment '排序号',
    deleted               tinyint    default 0                 null comment '删除标记',
    version               int        default 0                 null comment '数据版本号',
    company_id            bigint                               null comment '公司id',
    ct_user_id            bigint                               null comment '创建人',
    ct_date               datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id            bigint                               null comment '最后修改人',
    lm_date               datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '活动表
';

create table if not exists activity_comment
(
    id                bigint auto_increment comment '主键'
        primary key,
    activity_id       bigint                               null comment '活动id',
    type              int                                  null comment '类型 1:评论 2:回复',
    source            int                                  null comment '评论用户类型 1:业主用户 2:物业用户',
    base_comment_id   bigint                               null comment '基础评论id',
    reply_num         int        default 0                 null comment '基础评论被回复数量',
    praise_num        int        default 0                 null comment '点赞数',
    reply_source      int                                  null comment '回复用户类型 1:业主用户 2:物业用户',
    reply_comment_id  bigint                               null comment '回复评论id',
    reply_user_id     bigint                               null comment '回复人id',
    content           varchar(500)                         null comment '评论内容',
    hide              tinyint(1) default 0                 null comment '是否隐藏 0：不隐藏 1：隐藏',
    delete_type       int        default 0                 null comment '删除类型 1：业主删除 2：管理员删除',
    deleted           tinyint    default 0                 null comment '删除标记',
    version           int        default 0                 null comment '数据版本号',
    community_id      bigint                               null comment '社区id',
    community_user_id bigint                               null comment '用户',
    company_user_id   bigint                               null comment '公司人员',
    company_id        bigint                               null comment '公司id',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    delete_user_id    bigint                               null comment '公司用户最后操作人',
    lm_date           datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '活动评论';

create table if not exists activity_comment_praise_record
(
    id         bigint auto_increment comment '主键'
        primary key,
    comment_id bigint                             null comment '评论id',
    deleted    tinyint  default 0                 null comment '逻辑删除',
    ct_user_id bigint                             null comment '点赞人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '点赞时间'
)
    comment '评论点赞记录';

create table if not exists activity_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    activity_id  bigint            null comment '活动id',
    community_id bigint            null comment '社区id',
    deleted      tinyint default 0 null comment '删除标记',
    company_id   bigint            null comment '公司id'
)
    comment '活动发布社区';

create table if not exists activity_enroll
(
    id                bigint auto_increment comment '主键'
        primary key,
    activity_id       bigint                                null comment '活动id',
    community_id      bigint                                null comment '社区id',
    community_user_id bigint                                null comment '报名业主id',
    name              varchar(20)                           null comment '报名业主姓名',
    phone             varchar(20) default '0'               null comment '报名业主联系电话',
    enroll_num        int                                   null comment '报名人数',
    has_child         tinyint(1)  default 1                 null comment '是否携带儿童0：不携带 1：携带',
    child_num         int                                   null comment '儿童人数',
    enroll_remark     varchar(1000)                         null comment '报名人员备注',
    remark            varchar(1000)                         null comment '后台人员备注',
    status            int         default 0                 null comment '报名状态 1:未确认 2:已确认 3:已取消',
    deleted           tinyint     default 0                 null comment '删除标记',
    version           int         default 0                 null comment '数据版本号',
    company_id        bigint                                null comment '公司id',
    ct_user_id        bigint                                null comment '创建人',
    ct_date           datetime    default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                                null comment '最后修改人',
    lm_date           datetime    default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '活动报名
';

create table if not exists activity_view_praise_record
(
    id          bigint auto_increment comment '主键'
        primary key,
    type        int                                null comment '类型 1：查看 2：点赞',
    activity_id bigint                             null comment '活动id',
    deleted     tinyint  default 0                 null comment '逻辑删除',
    ct_user_id  bigint                             null comment '创建人',
    ct_date     datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '活动查看点赞记录';

create table if not exists aio_menu
(
    id         bigint auto_increment comment '主键'
        primary key,
    name       varchar(100)                       null comment '名称',
    ename      varchar(100)                       null comment '英文名',
    color      int                                null comment '颜色 1 purple 2 green 3 orange 4 Khaki 5 red 6 blue 7 light_yellow 8 light_blue',
    remark     varchar(100)                       null comment '备注',
    version    int      default 0                 null comment '数据版本号',
    deleted    tinyint  default 0                 null comment '删除标记',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '一体机菜单';

create table if not exists aio_pay_channel
(
    id            bigint auto_increment comment '主键'
        primary key,
    merchant_name varchar(100)                       null comment '商户名称',
    company_id    bigint                             null comment '公司id',
    appid         varchar(100)                       null comment 'app id',
    mchid         varchar(100)                       null comment '商户id',
    app_key       varchar(200)                       null comment '秘钥',
    remark        varchar(100)                       null comment '备注',
    version       int      default 0                 null comment '数据版本号',
    deleted       tinyint  default 0                 null comment '删除标记',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '一体机商户表';

create table if not exists aio_pay_channel_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    channel_id   bigint                             null comment '渠道id',
    community_id bigint                             null comment '社区id',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '一体机支付渠道与社区关联表';

create table if not exists aio_terminal
(
    id           bigint auto_increment comment '主键'
        primary key,
    company_id   bigint                               null comment '公司id',
    community_id bigint                               null comment '社区id',
    name         varchar(100)                         null comment '终端名称',
    bank_mid     varchar(100)                         null comment '银行卡商户号',
    bank_tid     varchar(100)                         null comment '银行卡支付终端号',
    qrcode_tid   varchar(100)                         null comment '二维码支付终端号',
    size         int                                  null comment '尺寸：1、19寸  2 、55寸',
    is_active    tinyint(1) default 0                 null comment '是否使用  0：未使用 1：已使用',
    remark       varchar(100)                         null comment '备注',
    version      int        default 0                 null comment '数据版本号',
    deleted      tinyint    default 0                 null comment '删除标记',
    ct_user_id   bigint                               null comment '创建人',
    ct_date      datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                               null comment '最后修改人',
    lm_date      datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '一体机终端';

create table if not exists aio_terminal_community_record
(
    id           bigint auto_increment comment '主键'
        primary key,
    terminal_id  bigint                             null comment '终端id',
    community_id bigint                             null comment '社区id',
    company_id   bigint                             null comment '公司id',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '一体机关联社区历史记录';

create table if not exists aio_third_log
(
    id              bigint auto_increment comment '主键'
        primary key,
    type            int                                null comment '请求类型 1 支付回调  2 退款回调 3 查询支付 4 退款',
    third_sys_name  varchar(100)                       null comment '支付系统名称',
    out_trade_no    varchar(100)                       null comment '外部单号',
    callback_status varchar(100)                       null comment '回调状态',
    trade_no        varchar(100)                       null comment '交易单号',
    content         text                               null comment '详细信息',
    remark          text                               null comment '备注',
    company_id      bigint                             null comment '公司id',
    deleted         tinyint  default 0                 null comment '删除标记',
    version         int      default 0                 null comment '数据版本号',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '一体机第三方支付日志';

create table if not exists aio_union_response
(
    id          bigint auto_increment comment '主键'
        primary key,
    bill_no     varchar(100)                       null comment '交易单号',
    rsp_code    varchar(100)                       null comment '交易状态标准，00表示成功',
    rsp_chin    varchar(100)                       null comment '返回码描述。交易结果的中文描述',
    card_no     varchar(100)                       null comment '19位卡号，消费、撤销和退货返回屏蔽卡号，其它交易无',
    amount      varchar(100)                       null comment '交易金额，精确到分，不足左补0。比如1.23元应填写000000000123。',
    bank_code   varchar(100)                       null comment '发卡行代码',
    bank_name   varchar(100)                       null comment '发卡行中文名',
    trace_no    varchar(100)                       null comment '交易凭证号',
    batch_no    varchar(100)                       null comment '交易批次号',
    ref_no      varchar(100)                       null comment '系统参考号',
    auth_no     varchar(100)                       null comment '授权号:预授权类返回，其它无',
    settle_date varchar(100)                       null comment '清算日期',
    merch_id    varchar(100)                       null comment '商户号',
    term_id     varchar(100)                       null comment '终端号',
    trans_date  varchar(100)                       null comment '交易日期:MMDD',
    trans_time  varchar(100)                       null comment '交易时间:HHMMMSS',
    deleted     tinyint  default 0                 null comment '删除标记',
    version     int      default 0                 null comment '数据版本号',
    ct_user_id  bigint                             null comment '创建人',
    ct_date     datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                             null comment '最后修改人',
    lm_date     datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '银行卡响应记录';

create table if not exists aio_upgrade_record
(
    id         bigint auto_increment comment '主键'
        primary key,
    name       varchar(100)                         null comment '名称',
    version    varchar(100)                         null comment '版本号',
    url        varchar(255)                         null comment '文件路径',
    content    varchar(600)                         null comment '更新内容',
    size       int                                  null comment '尺寸：1、19寸  2 、55寸',
    is_publish tinyint(1) default 0                 null comment '是否发布 0.否 1.是',
    deleted    tinyint    default 0                 null comment '删除标记',
    ct_user_id bigint                               null comment '创建人',
    ct_date    datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                               null comment '最后修改人',
    lm_date    datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '一体机升级记录';

create table if not exists base_app_version
(
    id          bigint auto_increment comment '主键'
        primary key,
    app_type    tinyint                              not null comment '1.用户端APP 2.管理端APP',
    client_type tinyint                              not null comment '1.android 2.ios',
    version     varchar(20)                          not null comment '版本号x.y.z',
    content     varchar(600)                         null comment '更新内容',
    url         varchar(300)                         null comment '下载URL',
    is_force    tinyint(1) default 0                 null comment '0.不强制 1.强制',
    ct_date     datetime   default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment 'APP版本升级';

create table if not exists base_area
(
    id         bigint auto_increment comment '主键'
        primary key,
    code       varchar(20)                        null comment '行政单位编码',
    name       varchar(100)                       null comment '行政单位名称',
    parent_id  bigint                             null comment '父节点id',
    level      int                                null comment '处于第几层',
    leaf       tinyint(1)                         null comment '是否叶子节点',
    id_path    varchar(100)                       null comment 'ID全路径',
    name_path  varchar(200)                       null comment '名称全路径',
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '行政单位管理';

create table if not exists base_error_log
(
    id           bigint auto_increment comment '主键id'
        primary key,
    user_id      bigint                             null comment '操作用户id',
    project_name varchar(50)                        null comment '项目名称',
    error_code   varchar(10)                        null comment '错误码',
    content      text                               not null comment '日志异常信息',
    platform     varchar(50)                        null comment '平台类型',
    version      varchar(50)                        null comment '软件版本号',
    deleted      tinyint  default 0                 null comment '是否删除 0 未删除  1 逻辑删除',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '日志表';

create table if not exists base_template
(
    id         bigint auto_increment comment '主键'
        primary key,
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '模板属性';

create table if not exists base_verification_code
(
    id                bigint auto_increment comment '主键'
        primary key,
    type              int                                null comment '类型 1：业主端注册验证码 2：业主端登录验证码',
    phone             varchar(20)                        null comment '电话号码',
    verification_code varchar(20)                        null comment '验证码',
    expiration_time   datetime                           null comment '过期时间',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    status            int                                null comment '有效状态 1：有效 2：失效'
)
    comment '验证码表';

create table if not exists bbs_post
(
    id                bigint auto_increment comment '主键'
        primary key,
    source            int                                  null comment '来源  1:业主用户 2:物业用户',
    community_id      bigint                               null comment '社区id',
    topic_id          bigint     default 0                 null comment '话题id',
    content           text                                 null comment '内容',
    allow_comment     tinyint(1) default 1                 null comment '是否允许评论 0：不能评论 1：可以评论',
    hide              tinyint(1) default 0                 null comment '是否隐藏 0:不隐藏 1:隐藏',
    top               tinyint(1) default 0                 null comment '是否置顶 0:不置顶 1:置顶',
    highlight         tinyint(1) default 0                 null comment '是否加精 0:未加精 1:已加精',
    view_num          int        default 0                 null comment '浏览数',
    comment_num       int        default 0                 null comment '评论数',
    praise_num        int        default 0                 null comment '点赞数',
    community_user_id bigint                               null comment 'APP用户创建id',
    company_user_id   bigint                               null comment '后台用户创建id',
    company_id        bigint                               null comment '公司id',
    hide_user_id      bigint                               null comment '隐藏操作用户id',
    delete_type       int        default 0                 null comment '删除类型 1：业主删除 2：管理员删除',
    deleted           tinyint    default 0                 null comment '用户删除标记',
    version           int        default 0                 null comment '数据版本号',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_date           datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '邻里论坛帖子';

create table if not exists bbs_post_comment
(
    id                bigint auto_increment comment '主键'
        primary key,
    post_id           bigint                               null comment '帖子id',
    type              int                                  null comment '类型 1:评论 2:回复',
    source            int                                  null comment '评论用户类型 1:业主用户 2:物业用户',
    base_comment_id   bigint                               null comment '基础评论id',
    reply_num         int        default 0                 null comment '基础评论被回复数量',
    praise_num        int        default 0                 null comment '点赞数',
    reply_source      int                                  null comment '回复用户类型 1:业主用户 2:物业用户',
    reply_comment_id  bigint                               null comment '回复评论id',
    reply_user_id     bigint                               null comment '回复人id',
    content           varchar(2000)                        null comment '评论内容',
    hide              tinyint(1) default 0                 null comment '是否隐藏 0：不隐藏 1：隐藏',
    delete_type       int        default 0                 null comment '删除类型 1：业主删除 2：管理员删除',
    deleted           tinyint    default 0                 null comment '删除标记',
    version           int        default 0                 null comment '数据版本号',
    community_id      bigint                               null comment '社区id',
    community_user_id bigint                               null comment '用户',
    company_user_id   bigint                               null comment '公司人员',
    company_id        bigint                               null comment '公司id',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    delete_user_id    bigint                               null comment '公司用户最后操作人',
    lm_date           datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '邻里论坛帖子评论';

create table if not exists bbs_post_comment_praise_record
(
    id         bigint auto_increment comment '主键'
        primary key,
    comment_id bigint                             null comment '评论id',
    deleted    tinyint  default 0                 null comment '逻辑删除',
    ct_user_id bigint                             null comment '点赞人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '点赞时间'
)
    comment '邻里论坛评论点赞记录';

create table if not exists bbs_post_image
(
    id         bigint auto_increment comment '主键'
        primary key,
    post_id    bigint        null comment '帖子id',
    path       varchar(255)  null comment '路径',
    thumbnail  varchar(255)  null comment '缩略图路径',
    show_order int default 0 null comment '排序号'
)
    comment '邻里论坛帖子图片';

create table if not exists bbs_post_view_praise_record
(
    id         bigint auto_increment comment '主键'
        primary key,
    type       int                                null comment '类型 1：查看 2：点赞',
    post_id    bigint                             null comment '帖子id',
    deleted    tinyint  default 0                 null comment '逻辑删除',
    ct_user_id bigint                             null comment '查看、点赞人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '查看、点赞时间'
)
    comment '邻里论坛查看评论记录';

create table if not exists bbs_topic
(
    id          bigint auto_increment comment '主键'
        primary key,
    code        varchar(20)                        null comment '话题编号',
    title       varchar(100)                       null comment '话题标题',
    description varchar(500)                       null comment '话题描述',
    icon        varchar(255)                       null comment '话题图标',
    status      int                                null comment '话题状态 0:未发布 1:已发布 2:已撤销',
    show_order  int      default 0                 null comment '排序号',
    deleted     tinyint  default 0                 null comment '删除标记',
    version     int      default 0                 null comment '数据版本号',
    company_id  bigint                             null comment '公司id',
    ct_user_id  bigint                             null comment '创建人',
    ct_date     datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                             null comment '最后修改人',
    lm_date     datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '邻里论坛话题';

create table if not exists bbs_topic_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    community_id bigint                             null comment '社区id',
    topic_id     bigint                             null comment '话题id',
    deleted      tinyint  default 0                 null comment '删除标记',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '邻里论坛话题社区对应关系';

create table if not exists callcenter_visitor
(
    id         bigint auto_increment comment '主键'
        primary key,
    phone      varchar(20)                        null comment '联系电话',
    cust_name  varchar(50)                        null comment '姓名',
    house_id   bigint                             null comment '房产id',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '400 呼叫中心非业主临时 客户关系表';

create table if not exists community
(
    id          bigint auto_increment comment '主键'
        primary key,
    area_id     bigint                               null comment '所在区域',
    code        varchar(20)                          null comment '社区编号',
    name        varchar(100)                         null comment '社区名称',
    phone       varchar(20)                          null comment '联系电话',
    begin_time  varchar(20)                          null comment '上班时间',
    end_time    varchar(20)                          null comment '下班时间',
    address     varchar(500)                         null comment '详细地址',
    area_size   int                                  null comment '面积',
    households  int                                  null comment '户数',
    img_path    varchar(255)                         null comment '缩略图地址',
    online      tinyint(1) default 0                 null comment '是否上线0 非上线 1 上线',
    lon         varchar(32)                          null comment '经度',
    lat         varchar(32)                          null comment '纬度',
    open_date   datetime                             null comment '开盘时间',
    description varchar(1000)                        null comment '社区描述',
    remark      varchar(1000)                        null comment '备注',
    show_order  int        default 0                 null comment '排序号',
    version     int        default 0                 null comment '数据版本号',
    deleted     tinyint    default 0                 null comment '删除标记',
    company_id  bigint                               null comment '所属公司',
    ct_user_id  bigint                               null comment '创建人',
    ct_date     datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                               null comment '最后修改人',
    lm_date     datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
);

create table if not exists community_app_advertisement
(
    id               bigint auto_increment comment '主键'
        primary key,
    community_id     bigint                             null comment '关联社区',
    advertisement_id bigint                             null comment '关联广告资源',
    show_order       int      default 0                 null comment '排序号',
    deleted          tinyint  default 0                 null comment '删除标记',
    version          int      default 0                 null comment '数据版本号',
    company_id       bigint                             null comment '公司id',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区广告位配置';

create table if not exists community_app_headline
(
    id           bigint auto_increment comment '主键'
        primary key,
    headline_id  bigint                             null comment '头条id',
    community_id bigint                             null comment '社区id',
    remark       varchar(1000)                      null comment '备注',
    show_order   int      default 0                 null comment '排序号',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    company_id   bigint                             null comment '公司id',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment 'app头条快讯';

create table if not exists community_app_module
(
    id              bigint auto_increment comment '主键'
        primary key,
    community_id    bigint                             null comment '社区id',
    module_id       bigint                             null comment '模块id',
    display_on_home tinyint(1)                         null comment '是否首页显示',
    show_order      int      default 0                 null comment '模块排序号',
    home_order      int      default 0                 null comment '首页排序号',
    version         int      default 0                 null comment '数据版本号',
    deleted         tinyint  default 0                 null comment '删除标记',
    company_id      bigint                             null comment '所属公司',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区app模块表';

create table if not exists community_area
(
    id           bigint auto_increment comment '主键'
        primary key,
    community_id bigint                             null comment '所属社区',
    parent_id    bigint                             null comment '父节点ID',
    code         varchar(200)                       null comment '区域编号',
    name         varchar(300)                       null comment '区域名称',
    full_name    varchar(500)                       null comment '全名称',
    area_num     int                                null comment '区域号',
    type         int                                null comment '区域类型 1:楼栋 2:单元 3:楼层 4:公共区域',
    leaf         tinyint(1)                         null comment '是否叶子节点 0:非叶子节点 1:叶子节点',
    child_num    int                                null comment '孩子节点数量',
    id_path      varchar(200)                       null comment 'id路径',
    remark       varchar(1000)                      null comment '备注',
    show_order   int      default 0                 null comment '排序号',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    company_id   bigint                             null comment '公司id',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
);

create table if not exists community_house
(
    id            bigint auto_increment comment '主键'
        primary key,
    community_id  bigint                             null comment '社区id',
    area_id       bigint                             null comment '区域id',
    code          varchar(20)                        null comment '房产编号',
    full_code     varchar(200)                       null comment '包含楼层code的全code',
    name          varchar(200)                       null comment '房产名称',
    type          int                                null comment '类型 1:多层 2:小高层 3:高层 4:独栋 5:联排',
    property      int                                null comment '性质 1:住宅 2:商用 3:办公 4:配套',
    status        int                                null comment '状态 1:已售 2:未售 3:物业用房 4:出租',
    hi_status     int      default 1                 null comment '验房状态 1 未验房 2 未完成 3 未验收 4 已完成',
    building_area decimal(18, 4)                     null comment '建筑面积',
    actual_area   decimal(18, 4)                     null comment '套内面积',
    delivery_date datetime                           null comment '交付时间',
    warranty_date datetime                           null comment '质保到期时间',
    bind_code     varchar(20)                        null comment '房产绑定码',
    building      int                                null comment '楼栋',
    unit          int                                null comment '单元',
    floor         int                                null comment '楼层',
    building_id   bigint                             null comment '楼栋id',
    unit_id       bigint                             null comment '单元id',
    floor_id      bigint                             null comment '楼层id',
    remark        varchar(1000)                      null comment '备注',
    show_order    int      default 0                 null comment '排序号',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    company_id    bigint                             null comment '公司id',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区住宅';

create table if not exists community_house_owner
(
    id                bigint auto_increment comment '主键'
        primary key,
    community_id      bigint                             null comment '社区id',
    community_user_id bigint                             null comment '社区用户id',
    house_id          bigint                             null comment '房产id',
    phone             varchar(11)                        null comment '联系电话',
    name              varchar(50)                        null comment '姓名',
    sex               int                                null comment '性别 1:女 2:男 3:未知',
    birthday          datetime                           null comment '生日',
    level             int                                null comment '等级 1:A级 2:B级 3:C级',
    native_place      varchar(200)                       null comment '籍贯',
    id_card           varchar(20)                        null comment '身份证号',
    work_unit         varchar(200)                       null comment '工作单位',
    profession        varchar(100)                       null comment '职业',
    home_address      varchar(200)                       null comment '家庭地址',
    hobbies           varchar(500)                       null comment '兴趣爱好',
    relationship      int                                null comment '与业主关系',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    company_id        bigint                             null comment '公司id',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '业主档案表';

create table if not exists community_user
(
    id                   bigint auto_increment comment '主键'
        primary key,
    dept_id              bigint                               null comment '所属部门',
    phone                varchar(20)                          null comment '登录手机号',
    password             varchar(100)                         null comment '密码',
    nickname             varchar(100)                         null comment '昵称',
    real_name            varchar(100)                         null comment '真实名称',
    birthday             datetime                             null comment '生日',
    sex                  int                                  null comment '性别 1:男 2:女 3:中性 4:未知',
    level                int                                  null comment '级别',
    avatar               varchar(200)                         null comment '头像',
    allow_speak          tinyint(1) default 1                 null comment '可否发言',
    allow_push           tinyint(1) default 1                 null comment '可否推送',
    default_community_id bigint                               null comment '默认绑定社区id',
    remark               varchar(1000)                        null comment '备注',
    show_order           int        default 0                 null comment '排序号',
    deleted              tinyint    default 0                 null comment '删除标记',
    version              int        default 0                 null comment '数据版本号',
    ct_dept_id           int                                  null comment '创建部门',
    ct_user_id           int                                  null comment '创建人',
    ct_date              datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           int                                  null comment '最后修改人',
    lm_date              datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '业主用户表';

create table if not exists community_user_house
(
    id                bigint auto_increment comment '主键'
        primary key,
    community_user_id bigint                             null comment '社区用户',
    house_id          bigint                             null comment '房产id',
    type              int                                null comment '绑定方式 1:房产验证码 2:电话验证 3:租户绑定 4:绑定业主',
    remark            varchar(1000)                      null comment '备注',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    company_id        bigint                             null comment '公司id',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '用户房产绑定关系表';

create table if not exists community_user_inviter
(
    id                bigint auto_increment comment '主键'
        primary key,
    community_user_id bigint                             null comment '社区用户id',
    inviter_id        bigint                             null comment '邀请人id',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '社区用户邀请人';

create table if not exists community_user_login_log
(
    id          bigint auto_increment comment '主键'
        primary key,
    type        int                                null comment '登录类型 1:首次登录 2:登录 3:自动登录 4:登出',
    user_id     bigint                             null comment '用户id',
    year        int                                null comment '登录年份',
    month       varchar(20)                        null comment '登录月份',
    day         varchar(20)                        null comment '登录日期',
    time        datetime default CURRENT_TIMESTAMP null comment '登录时间',
    platform    varchar(50)                        null comment '设备类型',
    os_model    varchar(50)                        null comment '设备型号',
    os_version  varchar(50)                        null comment '系统版本',
    app_version varchar(50)                        null comment 'app版本'
)
    comment '业主用户登录日志';

create table if not exists community_user_msg
(
    id                bigint auto_increment comment '主键'
        primary key,
    business_type     int                                  null comment '业务类型 1：即时通讯 2：通知公告 3：邻里论坛 4 社区活动',
    type              int                                  null comment '互动类型 1：内容点赞 2：评论 3：评论点赞 4：回复 5：回复点赞',
    jump_type         int                                  null comment '跳转类型 1：跳转到业务 2：跳转到评论',
    business_id       bigint                               null comment '业务id',
    base_comment_id   bigint                               null comment '基础评论id',
    comment_id        bigint                               null comment '评论id',
    object_id         bigint                               null comment '备用id',
    send_user_type    int                                  null comment '发送用户类型 1：业主 2：管理员',
    send_user_id      bigint                               null comment '发送用户ID',
    receive_user_type int                                  null comment '接收用户类型 1：业主 2：管理员',
    receive_user_id   bigint                               null comment '接收用户ID',
    org_content       varchar(500)                         null comment '原始消息',
    send_content      varchar(500)                         null comment '发送的消息',
    read_status       tinyint(1) default 0                 null comment '是否已读 false：未读  true：已读',
    read_date         datetime                             null comment '阅读时间',
    deleted           tinyint    default 0                 null comment '删除标记',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '社区用户消息(点赞、评论回复)';

create table if not exists community_user_sys_msg
(
    id              bigint auto_increment comment '主键'
        primary key,
    msg_type        int                                null comment '消息类型 1：通知公告 2：社区活动  3：投票活动 4：问卷调查 5.活动报名结果 6:物业缴费通知 7:维修进度反馈 8.投诉进度反馈 9.验房进度反馈 10.家政服务进度反馈',
    title           varchar(100)                       null comment '标题',
    content         varchar(500)                       null comment '消息内容',
    business_id     bigint                             null comment '业务id',
    extras          varchar(200)                       null comment '额外参数',
    receive_user_id bigint                             null comment '接收用户ID',
    deleted         tinyint  default 0                 null comment '删除标记',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '用户系统消息';

create table if not exists community_user_sys_msg_delete_log
(
    id                bigint auto_increment comment '主键'
        primary key,
    msg_id            bigint                             null comment '系统消息id',
    community_user_id bigint                             null comment '社区用户id',
    delete_date       datetime default CURRENT_TIMESTAMP null comment '删除时间'
)
    comment '系统消息删除记录';

create table if not exists community_user_sys_msg_read_log
(
    id                bigint auto_increment comment '主键'
        primary key,
    msg_id            bigint                             null comment '系统消息id',
    community_user_id bigint                             null comment '阅读用户id',
    read_date         datetime default CURRENT_TIMESTAMP null comment '阅读时间'
);

create table if not exists community_user_sys_msg_reader
(
    id        bigint auto_increment comment '主键'
        primary key,
    msg_id    bigint null comment '系统消息id',
    type      int    null comment '读者类型 1:社区 2:单元 3:房产',
    reader_id bigint null comment '读者id'
);

create table if not exists complaint
(
    id                   bigint auto_increment comment '主键'
        primary key,
    code                 varchar(32)                        null comment '订单编号',
    community_id         bigint                             null comment '社区id',
    area_id              bigint   default 0                 null comment '区域id',
    house_id             bigint                             null comment '房产id',
    community_user_id    bigint                             null comment '社区用户id',
    source               int                                null comment '来源 1:业主端APP 2:管理端APP 3:总控平台 4:公司运营平台',
    type                 int(1)                             null comment '流程种类 1投诉 2表扬',
    complaint_type_id    bigint                             null comment '投诉类型id',
    complaint_desc       varchar(2000)                      null comment '投诉问题描述',
    linkman              varchar(100)                       null comment '联系人',
    phone                varchar(20)                        null comment '联系电话',
    is_effective         bit                                null comment '是否有效投诉',
    important            bit                                null comment '重点提醒',
    commented            bit                                null comment '业主是否已评价',
    satisfied            bit                                null comment '业主是否满意',
    re_commented         bit                                null comment '是否客服回访',
    re_satisfied         bit                                null comment '客服回访是否满意',
    reply_on_time        bit                                null comment '是否准时对业主说',
    is_repair            bit                                null comment '是否生成报修',
    repair_id            bigint                             null comment '报修流程Id',
    proc_def_id          varchar(200)                       null comment '流程定义',
    proc_id              varchar(200)                       null comment '流程id',
    proc_task_id         varchar(200)                       null comment '流程任务id',
    proc_task_name       varchar(200)                       null comment '流程状态名称',
    proc_status_id       int                                null comment '流程状态 1.未发起 2.流程中 3.结束',
    proc_task_date       datetime                           null comment '流程当前任务进入时间',
    proc_task_limit_date datetime                           null comment '当前任务到期时间',
    proc_time_limit      int                                null comment '流程当前任务时限',
    proc_end_date        datetime                           null comment '流程结束时间',
    proc_overdue         bit      default b'0'              null comment '流程处理是否曾经逾期',
    remark               varchar(1000)                      null comment '备注',
    show_order           int      default 0                 null comment '排序号',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    company_id           bigint                             null comment '公司id',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '投诉表扬主表';

create table if not exists complaint_attachment
(
    id           bigint auto_increment comment '主键'
        primary key,
    complaint_id bigint        null comment '投诉表扬id',
    type         int           null comment '附件类型 1.图片 2.录音',
    path         varchar(500)  null comment '附件路径',
    show_order   int default 0 null comment '排序号'
)
    comment '投诉表扬附件';

create table if not exists complaint_evaluate
(
    id                bigint auto_increment comment '主键'
        primary key,
    company_id        bigint                             null comment '公司id',
    process_log_id    bigint                             null comment '流程环节id',
    complaint_id      bigint                             null comment '投诉主表id',
    community_id      bigint                             null comment '社区id',
    house_id          bigint                             null comment '房产id',
    community_user_id bigint                             null comment '社区用户id',
    comment_type      int(1)                             null comment '评价类型 1业主评价 2物业人员代评价',
    comment_score     int                                null comment '得分 (1：非常不满意 2、不满意 3、一般 4、很满意 5、非常满意）',
    satisfied         bit                                null comment '是否满意',
    comment_content   varchar(1000)                      null comment '评价内容',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '业主投诉表扬评论表';

create table if not exists complaint_type
(
    id         bigint auto_increment comment '主键'
        primary key,
    parent_id  bigint                             null comment '父节点ID',
    name       varchar(300)                       null comment '部位名称',
    is_leaf    tinyint(1)                         null comment '是否叶子节点 0:非叶子节点 1:叶子节点',
    id_path    varchar(200)                       null comment 'id路径',
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '投诉类型';

create table if not exists device
(
    id               bigint auto_increment comment '主键'
        primary key,
    company_id       bigint                             null comment '公司id',
    community_id     bigint                             null comment '社区Id',
    device_type_id   bigint                             null comment '设备类型Id',
    new_device_id    bigint                             null comment '关联新设备id',
    code             varchar(50)                        null comment '设备编码',
    name             varchar(50)                        null comment '设备名称',
    unit_type        varchar(200)                       null comment '设备型号',
    manufacturer     varchar(200)                       null comment '生产厂商',
    image            varchar(255)                       null comment '设备图片',
    qr_code          varchar(255)                       null comment '二维码标识',
    qr_code_path     varchar(255)                       null comment '二维码地址',
    location         varchar(255)                       null comment '位置',
    lon              decimal(20, 15)                    null comment '经度',
    lat              decimal(20, 15)                    null comment '纬度',
    install_date     datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '安装日期',
    device_params    longtext                           null comment '设备参数',
    maintain_guide   longtext                           null comment '保养指南',
    inspection_guide longtext                           null comment '巡检指南',
    inspection_num   int                                null comment '巡检次数',
    maintain_num     int                                null comment '保养次数',
    repair_num       int                                null comment '维修次数',
    sign             int(1)                             null comment '设备标记 0: 正常 1: 报修',
    status           int(1)                             null comment '设备状态 0: 正常 1: 报废',
    scrap_reason     varchar(255)                       null comment '报废原因',
    remark           varchar(255)                       null comment '备注',
    version          int      default 0                 null comment '数据版本号',
    deleted          tinyint  default 0                 null comment '删除标记',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备信息表';

create table if not exists device_check_item
(
    id             bigint auto_increment comment '主键'
        primary key,
    company_id     bigint                             null comment '公司id',
    type           int(1)                             null comment '类型: 1.巡检 2.保养',
    device_type_id bigint                             null comment '设备类型id',
    name           varchar(1000)                      null comment '检查项名称',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '巡检保养检查项';

create table if not exists device_off_line
(
    id           bigint auto_increment comment '主键'
        primary key,
    company_id   bigint                               null comment '公司id',
    community_id bigint                               null comment '社区id',
    file_name    varchar(100)                         null comment '离线文件名',
    file_url     varchar(255)                         null comment '离线文件url',
    is_current   tinyint(1) default 0                 null comment '是否最新数据包',
    version      int        default 0                 null comment '数据版本号',
    deleted      tinyint    default 0                 null comment '删除标记',
    ct_user_id   bigint                               null comment '创建人',
    ct_date      datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                               null comment '最后修改人',
    lm_date      datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备离线';

create table if not exists device_overdue_config
(
    id            bigint auto_increment comment '主键'
        primary key,
    company_id    bigint                             null comment '公司id',
    community_id  bigint                             null comment '社区Id',
    type          int(1)                             null comment '类型: 1.巡检 2.保养',
    plan_type     int(1)                             null comment '计划类型 1:天 2:月 3:年',
    warn_times    int(10)                            null comment '预警次数',
    warn_day      int(10)                            null comment '提前天数',
    warn_interval int(10)                            null comment '预警间隔(天)',
    warn_content  varchar(255)                       null comment '预警内容',
    is_send_sms   bit      default b'0'              null comment '是否发送短信',
    version       int      default 0                 null comment '数据版本号',
    deleted       tinyint  default 0                 null comment '删除标记',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养逾期配置';

create table if not exists device_plan
(
    id              bigint auto_increment comment '主键'
        primary key,
    company_id      bigint                               null comment '公司id',
    community_id    bigint                               null comment '社区Id',
    type            int(1)                               null comment '类型: 1.巡检 2.保养',
    user_id         bigint                               null comment '用户Id',
    device_id       bigint                               null comment '设备Id',
    plan_type       int(1)                               null comment '计划类型 1:天 2:月 3:年',
    plan_date       varchar(10)                          null comment '计划日期',
    plan_year       int(4)                               null comment '计划年',
    plan_month      int(2)                               null comment '计划月',
    plan_day        int(2)                               null comment '计划日',
    total_num       int                                  null comment '总次数',
    completed_num   int                                  null comment '已完成/逾期次数',
    is_suspend      tinyint(1) default 0                 null comment '是否暂停',
    is_complete     tinyint(1) default 0                 null comment '是否完成',
    complete_time   datetime                             null comment '完成时间（全部详情任务都结束或逾期）',
    is_route        tinyint(1)                           null comment '是否是路线计划',
    device_route_id bigint                               null comment '路线id',
    import_uuid     varchar(50)                          null comment '导入UUID',
    import_temp_id  int                                  null comment '导入临时ID',
    remark          varchar(255)                         null comment '备注',
    version         int        default 0                 null comment '数据版本号',
    deleted         tinyint    default 0                 null comment '删除标记',
    ct_user_id      bigint                               null comment '创建人',
    ct_date         datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                               null comment '最后修改人',
    lm_date         datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划';

create table if not exists device_plan_detail
(
    id                    bigint auto_increment comment '主键'
        primary key,
    company_id            bigint                               null comment '公司id',
    community_id          bigint                               null comment '社区Id',
    type                  int(1)                               null comment '类型: 1.巡检 2.保养',
    plan_type             int(1)                               null comment '计划类型 1:天 2:月 3:年',
    device_plan_id        bigint                               null comment '计划Id',
    device_id             bigint                               null comment '设备id',
    user_id               bigint                               null comment '用户Id',
    start_date            datetime                             null comment '计划开始时间',
    end_date              datetime                             null comment '计划结束时间',
    is_suspend            tinyint(1) default 0                 null comment '是否暂停',
    is_unplanned          tinyint(1)                           null comment '是否是计划外提交的记录',
    is_route              tinyint(1)                           null comment '是否是路线计划',
    device_route_id       bigint                               null comment '路线id',
    device_order          int                                  null comment '路线计划需要排序',
    electricity           varchar(100)                         null comment '电流',
    voltage               varchar(100)                         null comment '电压',
    water_pressure        varchar(100)                         null comment '水压',
    temperature           varchar(100)                         null comment '温度',
    exception_description varchar(1000)                        null comment '设备异常说明',
    deal_type             int(1)                               null comment '处理类型：1不报修 2自修 3派单维修',
    deal_time             datetime                             null comment '计划处理时间',
    submit_time           datetime                             null comment '数据提交时间',
    status                int(1)                               null comment '0未完成 1正常 2异常 3逾期 (所有的检查项正常则正常，否则异常)',
    import_uuid           varchar(50)                          null comment '导入UUID',
    import_temp_id        int                                  null comment '导入临时ID',
    version               int        default 0                 null comment '数据版本号',
    deleted               tinyint    default 0                 null comment '删除标记',
    ct_user_id            bigint                               null comment '创建人',
    ct_date               datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id            bigint                               null comment '最后修改人',
    lm_date               datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划明细';

create table if not exists device_plan_detail_attach
(
    id               bigint auto_increment comment '主键'
        primary key,
    plan_detail_id   bigint                             null comment '巡检保养计划明细Id',
    repair_record_id bigint                             null comment '设备维修id',
    url              varchar(500)                       null comment '附件地址',
    version          int      default 0                 null comment '数据版本号',
    deleted          tinyint  default 0                 null comment '删除标记',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '巡检保养计划明细附件表';

create table if not exists device_plan_detail_item
(
    id          bigint auto_increment comment '主键'
        primary key,
    type        int(1)                               null comment '类型: 1.巡检 2.保养',
    detail_id   bigint                               null comment '明细id',
    user_id     bigint                               null comment '用户Id',
    item_id     bigint                               null comment '检查项id',
    item_status tinyint(1) default 0                 null comment '检查项是否正常',
    version     int        default 0                 null comment '数据版本号',
    deleted     tinyint    default 0                 null comment '删除标记',
    ct_user_id  bigint                               null comment '创建人',
    ct_date     datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                               null comment '最后修改人',
    lm_date     datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '巡检保养明细检查项';

create table if not exists device_plan_transfer_log
(
    id             bigint auto_increment comment '主键'
        primary key,
    company_id     bigint                             null comment '公司id',
    device_plan_id bigint                             null comment '计划id',
    device_id      bigint                             null comment '设备id',
    before_user_id bigint                             null comment '被转移用户Id',
    now_user_id    bigint   default 0                 null comment '当前用户id',
    execution_date datetime                           null comment '执行日期',
    remark         varchar(255)                       null comment '备注',
    version        int      default 0                 null comment '数据版本号',
    deleted        tinyint  default 0                 null comment '删除标记',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养计划转移日志';

create table if not exists device_repair_record
(
    id                    bigint auto_increment comment '主键'
        primary key,
    company_id            bigint                             null comment '公司id',
    community_id          bigint                             null comment '社区Id',
    device_id             bigint                             null comment '设备id',
    discover_way          int(1)                             null comment '发现问题途径: 1.巡检 2.保养 3设备台账直接报修',
    plan_detail_id        bigint                             null comment '巡检保养计划明细Id',
    repair_type           int(1)                             null comment '维修方式：1自修，2派单维修',
    exception_description varchar(1000)                      null comment '异常说明',
    deal_result           varchar(1000)                      null comment '自修处理结果',
    area_id               bigint                             null comment '区域id(暂无用 只为映射)',
    proc_def_id           varchar(200)                       null comment '流程定义',
    proc_id               varchar(200)                       null comment '流程id',
    proc_task_id          varchar(200)                       null comment '流程任务id',
    proc_task_name        varchar(200)                       null comment '流程状态名称',
    proc_status_id        int                                null comment '流程状态 1.未发起 2.流程中 3.结束',
    proc_task_date        datetime                           null comment '流程当前任务进入时间',
    proc_task_limit_date  datetime                           null comment '当前任务到期时间',
    proc_time_limit       int                                null comment '流程当前任务时限',
    proc_end_date         datetime                           null comment '流程结束时间',
    proc_overdue          bit      default b'0'              null comment '流程处理是否曾经逾期',
    version               int      default 0                 null comment '数据版本号',
    deleted               tinyint  default 0                 null comment '删除标记',
    ct_user_id            bigint                             null comment '创建人',
    ct_date               datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id            bigint                             null comment '最后修改人',
    lm_date               datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备维修记录表';

create table if not exists device_route
(
    id           bigint auto_increment comment '主键'
        primary key,
    company_id   bigint                             null comment '公司id',
    community_id bigint                             null comment '社区id',
    type         int(1)                             null comment '类型: 1.巡检 2.保养',
    route_name   varchar(100)                       null comment '路线名称',
    remark       varchar(255)                       null comment '备注',
    version      int      default 0                 null comment '数据版本号',
    deleted      tinyint  default 0                 null comment '删除标记',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备巡检保养线路';

create table if not exists device_route_relation
(
    id              bigint auto_increment comment '主键'
        primary key,
    device_route_id bigint                             null comment '路线id',
    device_id       bigint                             null comment '设备Id',
    show_order      int                                null comment '排序',
    version         int      default 0                 null comment '数据版本号',
    deleted         tinyint  default 0                 null comment '删除标记',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '巡检保养线路与设备管理表';

create table if not exists device_type
(
    id               bigint auto_increment comment '主键'
        primary key,
    parent_id        bigint                               null comment '设备父级id',
    level            int                                  null comment '类型级别',
    code             varchar(50)                          null comment '类型编码',
    name             varchar(50)                          null comment '类型名称',
    image            varchar(255)                         null comment '缩略图',
    maintain_guide   longtext                             null comment '保养指南',
    inspection_guide longtext                             null comment '巡检指南',
    qr_code_size     int(1)                               null comment '二维码大小 1: 小码 2: 大码',
    remark           varchar(255)                         null comment '备注',
    is_leaf          tinyint(1) default 0                 null comment '是否叶子节点',
    company_id       bigint                               null comment '公司id',
    version          int        default 0                 null comment '数据版本号',
    deleted          tinyint    default 0                 null comment '删除标记',
    ct_user_id       bigint                               null comment '创建人',
    ct_date          datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                               null comment '最后修改人',
    lm_date          datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备类型表';

create table if not exists device_type_community
(
    id             bigint auto_increment comment '主键'
        primary key,
    device_type_id bigint                             null comment '设备类型id',
    community_id   bigint                             null comment '社区id',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    company_id     bigint                             null comment '公司id',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '设备类型关联小区';

create table if not exists hi_group
(
    id                   bigint auto_increment comment '主键'
        primary key,
    code                 varchar(32)                        null comment '订单编号',
    community_id         bigint                             null comment '社区id',
    area_id              bigint                             null comment '区域id',
    region_id            bigint                             null comment '部位id',
    house_id             bigint                             null comment '房产id',
    source               int                                null comment '来源 1:业主端APP 2:管理端APP 3:总控平台 4:公司运营平台',
    content              varchar(500)                       null comment '内容',
    linkman              varchar(100)                       null comment '联系人',
    phone                varchar(20)                        null comment '联系电话',
    proc_def_id          varchar(200)                       null comment '流程定义',
    proc_id              varchar(200)                       null comment '流程id',
    proc_task_id         varchar(200)                       null comment '流程任务id',
    proc_task_name       varchar(200)                       null comment '流程状态名称',
    proc_status_id       int                                null comment '流程状态 1.未发起 2.流程中 3.结束',
    proc_task_date       datetime                           null comment '流程当前任务进入时间',
    proc_task_limit_date datetime                           null comment '当前任务到期时间',
    proc_time_limit      int                                null comment '流程当前任务时限',
    proc_end_date        datetime                           null comment '流程结束时间',
    proc_overdue         bit      default b'0'              null comment '流程处理是否曾经逾期',
    remark               varchar(1000)                      null comment '备注',
    show_order           int      default 0                 null comment '排序号',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    company_id           bigint                             null comment '公司id',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '集团验房';

create table if not exists hi_group_attachment
(
    id          bigint auto_increment comment '主键'
        primary key,
    hi_group_id bigint        null comment '集团验房id',
    type        int default 1 null comment '附件类型 1.图片 2.录音',
    path        varchar(500)  null comment '附件路径'
)
    comment '集团验房附件';

create table if not exists hi_owner
(
    id                   bigint auto_increment comment '主键'
        primary key,
    code                 varchar(32)                        null comment '订单编号',
    community_id         bigint                             null comment '社区id',
    area_id              bigint                             null comment '区域id',
    region_id            bigint                             null comment '部位id',
    house_id             bigint                             null comment '房产id',
    community_user_id    bigint                             null comment '社区用户id',
    source               int                                null comment '来源 1:业主端APP 2:管理端APP 3:总控平台 4:公司运营平台',
    content              varchar(500)                       null comment '内容',
    linkman              varchar(100)                       null comment '联系人',
    phone                varchar(20)                        null comment '联系电话',
    comment_type_id      int                                null comment '评价类型1.业主评价 2.回访评价',
    commented            bit                                null comment '是否已评价',
    comment_score        int                                null comment '1-5 (1：非常不满意 2、不满意 3、一般 4、很满意 5、非常满意）',
    comment_content      varchar(1000)                      null comment '评价内容',
    comment_date         datetime                           null comment '评价时间',
    visited_user_id      bigint                             null comment '回访人员id',
    proc_def_id          varchar(200)                       null comment '流程定义',
    proc_id              varchar(200)                       null comment '流程id',
    proc_task_id         varchar(200)                       null comment '流程任务id',
    proc_task_name       varchar(200)                       null comment '流程状态名称',
    proc_status_id       int                                null comment '流程状态 1.未发起 2.流程中 3.结束',
    proc_task_date       datetime                           null comment '流程当前任务进入时间',
    proc_task_limit_date datetime                           null comment '当前任务到期时间',
    proc_time_limit      int                                null comment '流程当前任务时限',
    proc_end_date        datetime                           null comment '流程结束时间',
    proc_overdue         bit      default b'0'              null comment '流程处理是否曾经逾期',
    remark               varchar(1000)                      null comment '备注',
    show_order           int      default 0                 null comment '排序号',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    company_id           bigint                             null comment '公司id',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '业主验房';

create table if not exists hi_owner_attachment
(
    id          bigint auto_increment comment '主键'
        primary key,
    hi_owner_id bigint        null comment '业主验房id',
    type        int default 1 null comment '附件类型 1.图片 2.录音',
    path        varchar(500)  null comment '附件路径'
)
    comment '业主验房附件';

create table if not exists hi_region
(
    id         bigint auto_increment comment '主键'
        primary key,
    parent_id  bigint                             null comment '父节点ID',
    name       varchar(300)                       null comment '部位名称',
    type       int                                null comment '部位类型 1 个人 2 公区',
    is_leaf    tinyint(1)                         null comment '是否叶子节点 0:非叶子节点 1:叶子节点',
    id_path    varchar(200)                       null comment 'id路径',
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '验房部位';

create table if not exists hk_apply
(
    id                     bigint auto_increment comment '主键'
        primary key,
    code                   varchar(32)                        null comment '订单编号',
    community_user_id      bigint                             null comment '预约人id',
    house_id               bigint                             null comment '房产id',
    community_id           bigint                             null comment '社区id',
    area_id                bigint   default 0                 null comment '区域id',
    service_id             bigint                             null comment '服务id',
    company_user_id        bigint                             null comment '服务人员id',
    org_id                 bigint                             null comment '服务人员组织id',
    contact_phone          varchar(50)                        null comment '联系人电话',
    contact_name           varchar(50)                        null comment '联系人姓名',
    address                varchar(255)                       null comment '地址',
    apply_time             datetime                           null comment '预约服务时间',
    arrival_time           datetime                           null comment '预计到达时间',
    deposit_code           varchar(255)                       null comment '定金单号',
    deposit_refund_code    varchar(255)                       null comment '定金退款单号',
    deposit_pay_date       datetime                           null comment '定金支付时间',
    deposit_type           int                                null comment '定金类型 1 定金 2 全款',
    deposit_is_paid        tinyint(1)                         null comment '定金是否已支付 1 是  0 否',
    deposit_is_refund      tinyint(1)                         null comment '定金是否已退款 1 是  0 否',
    deposit_need_refund    tinyint(1)                         null comment '取消订单后是否退定金，0否，1是(在选择回单原因时确定)',
    deposit_pay_type       int                                null comment '定金支付方式  1 支付宝 2 微信支付',
    deposit_amount         decimal(16, 2)                     null comment '定金金额',
    pay_code               varchar(255)                       null comment '尾款单号',
    pay_date               datetime                           null comment '尾款支付时间',
    pay_type               int                                null comment '尾款支付方式  1 支付宝 2 微信支付',
    pay_amount             decimal(16, 2)                     null comment '尾款金额',
    total_amount           decimal(16, 2)                     null comment '总金额',
    cancel_reason          int                                null comment '回单原因  1 超出服务范围  2 价格因素 3 业主取消服务',
    is_pay_online          tinyint(1)                         null comment '是否线上付款，1 是  0 否',
    status                 int                                null comment '预约状态 10 定金待支付 20 待受理 30 待接单 40 待服务 50 待付款 60 已完成 70 已取消',
    comment_type_id        int                                null comment '评价类型1.业主评价 2.回访评价',
    community_user_deleted tinyint(1)                         null comment '是否业主已删除 1 是  0 否2',
    is_commend             tinyint(1)                         null comment '是否已评价 1 是  0 否',
    is_revisit             tinyint(1)                         null comment '是否已回访 1 是  0 否',
    revisit_id             bigint                             null comment '回访人员id',
    is_satisfaction        tinyint(1)                         null comment '是否满意：0：不满意；1：满意',
    score                  int                                null comment '评价打分',
    evaluate_content       varchar(1024)                      null comment '评价内容',
    evaluate_time          datetime                           null comment '评价时间',
    proc_def_id            varchar(200)                       null comment '流程定义',
    proc_id                varchar(200)                       null comment '流程id',
    proc_task_id           varchar(200)                       null comment '流程任务id',
    proc_task_name         varchar(200)                       null comment '流程状态名称',
    proc_status_id         int                                null comment '流程状态 1.未发起 2.流程中 3.结束',
    proc_task_date         datetime                           null comment '流程当前任务进入时间',
    proc_task_limit_date   datetime                           null comment '当前任务到期时间',
    proc_time_limit        int                                null comment '流程当前任务时限',
    proc_end_date          datetime                           null comment '流程结束时间',
    proc_overdue           bit      default b'0'              null comment '流程处理是否曾经逾期',
    remark                 varchar(255)                       null comment '备注',
    deleted                tinyint  default 0                 null comment '删除标记',
    version                int      default 0                 null comment '数据版本号',
    company_id             bigint                             null comment '公司id',
    ct_user_id             bigint                             null comment '创建人',
    ct_date                datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id             bigint                             null comment '最后修改人',
    lm_date                datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '家政服务预约';

create table if not exists hk_apply_cleaner
(
    id              bigint auto_increment comment '主键'
        primary key,
    apply_id        bigint                             null comment '预约id',
    company_user_id bigint                             null comment '清洁工id',
    deleted         tinyint  default 0                 null comment '删除标记',
    version         int      default 0                 null comment '数据版本号',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '家政服务预约与清洁工关联表';

create table if not exists hk_service
(
    id             bigint auto_increment comment '主键'
        primary key,
    type_id        bigint                             null comment '服务类型id',
    name           varchar(50)                        null comment '服务名称',
    description    varchar(255)                       null comment '服务简介',
    thumbnail      varchar(255)                       null comment '缩略图',
    content_img    varchar(255)                       null comment '内容主图',
    tips           text                               null comment '温馨提示',
    agreement      text                               null comment '协议内容',
    content        text                               null comment '服务详情',
    deposit_type   int                                null comment '定金类型 1 定金 2 全款',
    min_price      decimal(16, 2)                     null comment '最低价',
    max_price      decimal(16, 2)                     null comment '最高价',
    price_type     int                                null comment '计费单位 1 小时 2 平方米',
    deposit_amount decimal(16, 2)                     null comment '定金金额',
    is_show        tinyint(1)                         null comment '是否显示 1 是  0 否',
    show_order     int      default 0                 null comment '排序号',
    remark         varchar(255)                       null comment '备注',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    company_id     bigint                             null comment '公司id',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '家政服务内容';

create table if not exists hk_service_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    service_id   bigint                             null comment '服务id',
    community_id bigint                             null comment '社区id',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '家政服务内容与社区关联表';

create table if not exists hk_service_organization
(
    id            bigint auto_increment comment '主键'
        primary key,
    service_id    bigint                             null comment '服务id',
    org_id        bigint                             null comment '家政公司组织id',
    is_pay_online tinyint(1)                         null comment '是否在线收尾款 1 是  0 否',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '家政服务内容与组织关联表';

create table if not exists hk_service_type
(
    id         bigint auto_increment comment '主键'
        primary key,
    name       varchar(20)                        null comment '分类名称',
    thumbnail  varchar(255)                       null comment '缩略图',
    remark     varchar(1000)                      null comment '备注',
    is_show    tinyint(1)                         null comment '是否显示 1 是  0 否',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '家政服务类型';

create table if not exists hk_service_user
(
    id                   bigint auto_increment comment '主键'
        primary key,
    service_id           bigint                             null comment '服务id',
    saas_company_user_id bigint                             null comment '公司用户id',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '家政服务内容与公司人员关联表';

create table if not exists house_intent
(
    id                bigint auto_increment comment '主键'
        primary key,
    code              varchar(100)                       null comment '编号',
    community_user_id bigint                             null comment '社区用户id',
    realtor_id        bigint                             null comment '置业顾问id',
    custom_name       varchar(50)                        null comment '姓名',
    custom_phone      varchar(30)                        null comment '联系电话',
    source            int                                null comment '来源 1 业主app 2 物业app 3管理平台',
    type              int                                null comment '意向类型 1 出售 2 出租',
    deposit_amount    decimal(16, 2)                     null comment '定金',
    agent_amount      decimal(16, 2)                     null comment '中介费',
    assgin_time       datetime                           null comment '指派时间',
    finish_time       datetime                           null comment '完成时间',
    publish_time      datetime                           null comment '发布时间',
    status            int                                null comment '状态：10未指派、20已指派、30已发布、40已成交、50意向取消、60竞对失败、70终止开发、80客户删除',
    fail_reason       varchar(255)                       null comment '失败原因',
    remark            varchar(255)                       null comment '备注',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    company_id        bigint                             null comment '公司id',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '租售意向信息';

create table if not exists house_lease
(
    id             bigint auto_increment comment '主键'
        primary key,
    intent_id      bigint                               null comment '意向id',
    title          varchar(100)                         null comment '标题',
    thumbnail      varchar(255)                         null comment '缩略图',
    contact_phone  varchar(100)                         null comment '联系电话',
    area_id        bigint                               null comment '所在区域id',
    area_name      varchar(100)                         null comment '区域名称',
    address        varchar(100)                         null comment '地址',
    community_name varchar(100)                         null comment '楼盘名称',
    lon            varchar(100)                         null comment '经度',
    lat            varchar(100)                         null comment '纬度',
    building       varchar(100)                         null comment '楼栋',
    unit           varchar(100)                         null comment '单元',
    floor          varchar(100)                         null comment '楼层',
    floor_total    varchar(100)                         null comment '总楼层',
    floor_type     int                                  null comment '楼层分类  1高2中3低',
    orientation    int                                  null comment '朝向 1朝东2朝西3朝南4朝北',
    room           varchar(100)                         null comment '房间',
    chamber_num    int                                  null comment '户型室',
    hall_num       int                                  null comment '户型厅',
    bathroom_num   int                                  null comment '户型卫',
    decorate_type  int                                  null comment '装修情况 1精致装修 2中等装修 3简单装修',
    pay_type       int                                  null comment '付款方式 1押一付一 2押一付三 3季付 4年付',
    parking_type   int                                  null comment '车位情况  1无2地上3地下',
    sex_limit      int                                  null comment '性别限制 1限男性 2限女性 3男女不限',
    lease_type     int                                  null comment '出租方式 1合租 2整租',
    support        varchar(255)                         null comment '配套设施(逗号隔开)1床2空调3洗衣机4阳台5冰箱6厨房7衣柜8卫生间9电视10宽带',
    tags           varchar(255)                         null comment '标签(逗号隔开)1 近地铁2精装3免中介费4新上5随时看房',
    house_area     decimal(16, 2)                       null comment '房屋面积',
    rent_amount    decimal(16, 2)                       null comment '租金',
    has_elevator   tinyint(1) default 0                 null comment '是否有电梯 0：否 1：是',
    is_top         tinyint(1) default 0                 null comment '是否加精 0：否 1：是',
    description    varchar(512)                         null comment '房源描述',
    remark         varchar(255)                         null comment '备注',
    deleted        tinyint    default 0                 null comment '删除标记',
    version        int        default 0                 null comment '数据版本号',
    company_id     bigint                               null comment '公司id',
    ct_user_id     bigint                               null comment '创建人',
    ct_date        datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                               null comment '最后修改人',
    lm_date        datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '房产出租';

create table if not exists house_lease_image
(
    id             bigint auto_increment comment '主键'
        primary key,
    house_lease_id bigint        null comment '出租房源id',
    path           varchar(255)  null comment '路径',
    thumbnail      varchar(255)  null comment '缩略图路径',
    show_order     int default 0 null comment '排序号'
)
    comment '房产出租图片';

create table if not exists house_realtor_record
(
    id              bigint auto_increment comment '主键'
        primary key,
    intent_id       bigint                             null comment '意向id',
    company_user_id bigint                             null comment '置业顾问id',
    reason          varchar(255)                       null comment '变更原因',
    change_date     datetime                           null comment '变更时间',
    remark          varchar(255)                       null comment '备注',
    deleted         tinyint  default 0                 null comment '删除标记',
    version         int      default 0                 null comment '数据版本号',
    company_id      bigint                             null comment '公司id',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '置业顾问变更记录';

create table if not exists house_sale
(
    id             bigint auto_increment comment '主键'
        primary key,
    intent_id      bigint                               null comment '意向id',
    title          varchar(100)                         null comment '标题',
    thumbnail      varchar(255)                         null comment '缩略图',
    contact_phone  varchar(100)                         null comment '联系电话',
    area_id        bigint                               null comment '所在区域id',
    area_name      varchar(100)                         null comment '区域名称',
    address        varchar(100)                         null comment '地址',
    community_name varchar(100)                         null comment '楼盘名称',
    lon            varchar(100)                         null comment '经度',
    lat            varchar(100)                         null comment '纬度',
    building       varchar(100)                         null comment '楼栋',
    unit           varchar(100)                         null comment '单元',
    floor          varchar(100)                         null comment '楼层',
    floor_total    varchar(100)                         null comment '总楼层',
    floor_type     int                                  null comment '楼层分类  1高2中3低',
    orientation    int                                  null comment '朝向 1 南北通透 ,2 朝东 ,3 朝西 ,4 朝南 ,5 朝北 ',
    room           varchar(100)                         null comment '房间',
    chamber_num    int                                  null comment '户型室',
    hall_num       int                                  null comment '户型厅',
    bathroom_num   int                                  null comment '户型卫',
    decorate_type  int                                  null comment '装修情况 1 精致装修 ,2 中等装修 ,3 简单装修 ,4 毛坯房 ',
    year_type      int                                  null comment '产权年限 1、20年2、30年3、40年4、50年5、70年',
    building_type  int                                  null comment '楼型 1板楼2塔楼3板塔结合4其他',
    house_type     int                                  null comment '房屋类型 1商品房2经济适用房',
    parking_type   int                                  null comment '车位情况  1无2地上3地下',
    tags           varchar(255)                         null comment '标签(逗号隔开)1 近地铁2精装3免中介费4新上5随时看房',
    building_year  int                                  null comment '建筑年代(四位年份)',
    house_area     decimal(16, 2)                       null comment '房屋面积',
    total_price    decimal(16, 2)                       null comment '房屋总价',
    unit_price     decimal(16, 2)                       null comment '房屋单价',
    out_tax        tinyint(1) default 0                 null comment '是否出税期 0：未出税 1：出税',
    has_storage    tinyint(1) default 0                 null comment '是否有储藏室 0：否 1：是',
    has_elevator   tinyint(1) default 0                 null comment '是否有电梯 0：否 1：是',
    is_top         tinyint(1) default 0                 null comment '是否加精 0：否 1：是',
    description    varchar(512)                         null comment '房源描述',
    remark         varchar(255)                         null comment '备注',
    deleted        tinyint    default 0                 null comment '删除标记',
    version        int        default 0                 null comment '数据版本号',
    company_id     bigint                               null comment '公司id',
    ct_user_id     bigint                               null comment '创建人',
    ct_date        datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                               null comment '最后修改人',
    lm_date        datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '房产出售';

create table if not exists house_sale_image
(
    id            bigint auto_increment comment '主键'
        primary key,
    house_sale_id bigint        null comment '出售房源id',
    path          varchar(255)  null comment '路径',
    thumbnail     varchar(255)  null comment '缩略图路径',
    show_order    int default 0 null comment '排序号'
)
    comment '房产出售图片';

create table if not exists merchant
(
    id             bigint auto_increment comment '主键'
        primary key,
    type_id        bigint                             null comment '商户类型id',
    area_id        bigint                             null comment '所在区域',
    name           varchar(50)                        null comment '商户名称',
    phone          varchar(20)                        null comment '联系电话',
    address        varchar(100)                       null comment '地址',
    thumbnail      varchar(255)                       null comment '列表缩略图',
    content        text                               null comment '商户详细描述',
    content_img    varchar(255)                       null comment '内容主图',
    business_hours varchar(100)                       null comment '营业时间',
    lon            varchar(32)                        null comment '经度',
    lat            varchar(32)                        null comment '纬度',
    view_num       int      default 0                 null comment '浏览数',
    status         int      default 0                 null comment '状态 0：新建 1：已发布 2：已撤销',
    remark         varchar(1000)                      null comment '备注',
    show_order     int      default 0                 null comment '排序号',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    company_id     bigint                             null comment '公司id',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区周边商户';

create table if not exists merchant_community_distance
(
    id            bigint auto_increment comment '主键'
        primary key,
    merchant_id   bigint        null comment '商户id',
    community_id  bigint        null comment '社区id',
    merchant_lat  varchar(32)   null comment '商户纬度',
    community_lon varchar(32)   null comment '社区经度',
    community_lat varchar(32)   null comment '社区纬度',
    merchant_lon  varchar(32)   null comment '商户经度',
    distance      int default 0 null comment '距离(米)'
)
    comment '社区周边商户与社区距离';

create table if not exists merchant_type
(
    id         bigint auto_increment comment '主键'
        primary key,
    name       varchar(20)                        null comment '分类名称',
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商户类型';

create table if not exists merchant_view_record
(
    id                bigint auto_increment comment '主键'
        primary key,
    merchant_id       bigint   null comment '商户id',
    community_user_id bigint   null comment '社区用户id',
    ct_date           datetime null comment '浏览时间'
)
    comment '社区周边浏览记录';

create table if not exists notice
(
    id            bigint auto_increment comment '主键'
        primary key,
    title         varchar(100)                         null comment '标题',
    summary       varchar(500)                         null comment '摘要',
    content       longtext                             null comment '内容',
    allow_comment tinyint(1) default 1                 null comment '是否允许评论 0：不能评论 1：可以评论',
    allow_push    tinyint(1) default 0                 null comment '是否允许推送 0：不推送 1：推送',
    view_num      int        default 0                 null comment '浏览数',
    comment_num   int        default 0                 null comment '评论数',
    praise_num    int        default 0                 null comment '点赞数',
    sms_num       int        default 0                 null comment '短信发送次数',
    thumbnail     varchar(255)                         null comment '缩略图',
    type          int                                  null comment '查看类型 1：社区 2：单元 3：房产',
    publish_time  datetime                             null comment '发布时间',
    invalid_time  datetime                             null comment '失效时间',
    status        int        default 0                 null comment '状态 0：新建 1：已发布 2：已撤销',
    show_order    int        default 0                 null comment '排序号',
    deleted       tinyint    default 0                 null comment '删除标记',
    version       int        default 0                 null comment '数据版本号',
    company_id    bigint                               null comment '公司id',
    ct_user_id    bigint                               null comment '创建人',
    ct_date       datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                               null comment '最后修改人',
    lm_date       datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '通知公告主表';

create table if not exists notice_comment
(
    id                bigint auto_increment comment '主键'
        primary key,
    notice_id         bigint                               null comment '通知ID',
    type              int                                  null comment '类型 1:评论 2:回复',
    source            int                                  null comment '评论用户类型 1:业主用户 2:物业用户',
    base_comment_id   bigint                               null comment '基础评论id',
    reply_num         int        default 0                 null comment '基础评论被回复数量',
    praise_num        int        default 0                 null comment '点赞数',
    reply_source      int                                  null comment '回复用户类型 1:业主用户 2:物业用户',
    reply_comment_id  bigint                               null comment '回复评论id',
    reply_user_id     bigint                               null comment '回复人id',
    content           varchar(500)                         null comment '评论内容',
    hide              tinyint(1) default 0                 null comment '是否隐藏 0：不隐藏 1：隐藏',
    delete_type       int                                  null comment '删除类型 1：业主删除 2：管理员删除',
    deleted           tinyint    default 0                 null comment '删除标记',
    version           int        default 0                 null comment '数据版本号',
    community_id      bigint                               null comment '社区id',
    community_user_id bigint                               null comment '用户',
    company_user_id   bigint                               null comment '公司人员',
    company_id        bigint                               null comment '公司id',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    delete_user_id    bigint                               null comment '公司用户最后操作人',
    lm_date           datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '通知评论';

create table if not exists notice_comment_praise_record
(
    id         bigint auto_increment comment '主键'
        primary key,
    comment_id bigint                             null comment '评论id',
    deleted    tinyint  default 0                 null comment '逻辑删除',
    ct_user_id bigint                             null comment '点赞人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '点赞时间'
)
    comment '评论点赞记录';

create table if not exists notice_object
(
    id           bigint auto_increment comment '主键'
        primary key,
    type         int                                null comment '推送类型 1：社区 2：单元 3：房产',
    community_id bigint                             null comment '社区id',
    notice_id    bigint                             null comment '通知id',
    object_id    bigint                             null comment '目标ID',
    company_id   bigint                             null comment '公司id',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '通知公告查看对象';

create table if not exists notice_view_praise_record
(
    id         bigint auto_increment comment '主键'
        primary key,
    type       int                                null comment '类型 1：查看 2：点赞',
    notice_id  bigint                             null comment '通知公告',
    deleted    tinyint  default 0                 null comment '逻辑删除',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '通知公告查看、点赞记录';

create table if not exists pay_channel_alipay
(
    id            bigint auto_increment comment '主键'
        primary key,
    business_type int                                null comment '业务类型 1 物业交费 2 购买商品 3 报修交费 4 家政服务',
    merchant_name varchar(100)                       null comment '商户名称',
    company_id    bigint                             null comment '公司id',
    appid         varchar(100)                       null comment 'app id',
    partner       varchar(100)                       null comment '合作身份者ID',
    public_key    text                               null comment '商户公钥',
    private_key   text                               null comment '私钥',
    seller_email  varchar(255)                       null comment '商户邮箱',
    remark        varchar(100)                       null comment '备注',
    version       int      default 0                 null comment '数据版本号',
    deleted       tinyint  default 0                 null comment '删除标记',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '支付宝商户表';

create table if not exists pay_channel_wechat
(
    id            bigint auto_increment comment '主键'
        primary key,
    business_type int                                null comment '业务类型 1 物业交费 2 购买商品 3 报修交费 4 家政服务',
    merchant_name varchar(100)                       null comment '商户名称',
    company_id    bigint                             null comment '公司id',
    appid         varchar(100)                       null comment 'app id',
    mchid         varchar(100)                       null comment '商户id',
    wechat_key    text                               null comment '秘钥',
    refund_cert   longblob                           null comment '退款证书',
    remark        varchar(100)                       null comment '备注',
    version       int      default 0                 null comment '数据版本号',
    deleted       tinyint  default 0                 null comment '删除标记',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '微信支付商户表';

create table if not exists pay_community_channel
(
    id            bigint auto_increment comment '主键'
        primary key,
    community_id  bigint                             null comment '社区id',
    channel_type  int                                null comment '渠道类型 1 支付宝 2  微信支付',
    channel_id    bigint                             null comment '交费渠道id',
    business_type int                                null comment '业务类型 1 物业交费 2 购买商品 3 报修交费 4 家政服务',
    remark        varchar(100)                       null comment '备注',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区交费渠道关联表';

create table if not exists pay_community_property_point_rule
(
    id           bigint auto_increment comment '主键'
        primary key,
    community_id bigint                             null comment '社区id',
    rule_id      bigint                             null comment '规则id',
    remark       varchar(100)                       null comment '备注',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区与积分赠送规则关联表';

create table if not exists pay_property_fee_record
(
    id                   bigint auto_increment comment '主键'
        primary key,
    company_id           bigint                             null comment '公司id',
    community_id         bigint                             null comment '社区id',
    community_user_id    bigint                             null comment '社区用户id',
    voucher_code         varchar(100)                       null comment '订单号',
    total_money          decimal(10, 2)                     null comment '总金额',
    total_reward_point   int      default 0                 null comment '总赠送积分',
    present_money        decimal(10, 2)                     null comment '应交金额',
    present_reward_point int      default 0                 null comment '应交赠送积分',
    prepay_money         decimal(10, 2)                     null comment '预交金额',
    prepay_reward_point  int      default 0                 null comment '预交赠送积分',
    remark               varchar(100)                       null comment '备注',
    version              int      default 0                 null comment '数据版本号',
    deleted              tinyint  default 0                 null comment '删除标记',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '物业费记录表';

create table if not exists pay_property_point_rule
(
    id                        bigint auto_increment comment '主键'
        primary key,
    company_id                bigint                               null comment '公司id',
    rule_type                 int        default 0                 null comment '规则类型 1 每月活动日 2 固定日期',
    name                      varchar(100)                         null comment '规则名称',
    month_day                 varchar(100)                         null comment '每月活动日，多个用逗号隔开',
    begin_date                datetime                             null comment '固定日期开始时间',
    end_date                  datetime                             null comment '固定日期结束时间',
    is_active                 tinyint(1) default 0                 null comment '是否生效 0：否 1：是',
    present_reward_proportion decimal(10, 2)                       null comment '应交赠送比例',
    prepay_reward_proportion  decimal(10, 2)                       null comment '预交赠送比例',
    show_order                int        default 0                 null comment '排序号（数字越大优先级越高）',
    remark                    varchar(100)                         null comment '备注',
    version                   int        default 0                 null comment '数据版本号',
    deleted                   tinyint    default 0                 null comment '删除标记',
    ct_user_id                bigint                               null comment '创建人',
    ct_date                   datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id                bigint                               null comment '最后修改人',
    lm_date                   datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '物业交费积分奖励规则';

create table if not exists pay_record
(
    id                bigint auto_increment comment '主键'
        primary key,
    company_id        bigint                             null comment '公司id',
    community_id      bigint                             null comment '社区id',
    house_id          bigint                             null comment '房产id',
    business_id       bigint                             null comment '业务id',
    business_code     varchar(100)                       null comment '业务单号',
    community_user_id bigint                             null comment '社区用户id',
    voucher_code      varchar(100)                       null comment '订单号',
    bill_source       int      default 0                 null comment '账单来源  1 思源',
    trans_type        int      default 0                 null comment '交易类型 1 物业交费 2 购买商品 3 报修交费 ,4 家政服务 ,5 物业交费(一体机)',
    channel_type      int      default 0                 null comment '渠道类型 1 支付宝 2  微信支付 3 银联支付',
    channel_id        bigint                             null comment '渠道id',
    pay_date          datetime                           null comment '支付时间',
    trans_digest      varchar(100)                       null comment '交易摘要',
    trans_amount      decimal(10, 2)                     null comment '交易金额',
    upload_status     int      default 0                 null comment '上传状态：10 新建  20 已上传  30 上传失败',
    status            int      default 0                 null comment '支付状态：10 待支付 20 支付中  30已支付  40已完成 50已取消',
    remark            varchar(100)                       null comment '备注',
    version           int      default 0                 null comment '数据版本号',
    deleted           tinyint  default 0                 null comment '删除标记',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '交费记录表';

create table if not exists pay_sync_log
(
    id           bigint auto_increment comment '主键'
        primary key,
    type         int                                null comment '请求类型',
    voucher_code varchar(100)                       null comment '订单号',
    req_data     text                               null comment '请求数据',
    resp_data    text                               null comment '响应数据',
    remark       varchar(100)                       null comment '备注',
    company_id   bigint                             null comment '公司id',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '请求日志表';

create table if not exists pay_syswin_fee_record
(
    id                bigint auto_increment comment '主键'
        primary key,
    community_user_id bigint                                   null comment '社区用户id',
    company_id        bigint                                   null comment '公司id',
    community_id      bigint                                   null comment '社区id',
    house_id          bigint                                   null comment '房产id',
    source            int                                      null comment '数据来源 1 APP，2 一体机',
    voucher_code      varchar(100)                             null comment '订单号',
    e_rev_id          varchar(100)                             null comment '未知',
    p_rev_id          varchar(100)                             null comment '应收款ID',
    p_rev_name        varchar(100)                             null comment '应收款名称',
    e_cst_id          varchar(100)                             null comment '客户ID',
    e_cst_name        varchar(100)                             null comment '客户姓名',
    trading_id        int                                      null comment '支付方式 1支付宝 2微信 3 银联支付',
    trading_val       varchar(100)                             null comment '支付方式名称',
    rev_money         decimal(10, 2) default 0.00              null comment '实收金额',
    rev_date          varchar(100)                             null comment '费用账期',
    status            int                                      null comment '状态，10新建 20付款中 30已付款 40已上传 50上传失败 60已收清 70上传异常 80 解析异常',
    remark            varchar(100)                             null comment '备注',
    deleted           tinyint        default 0                 null comment '删除标记',
    version           int            default 0                 null comment '数据版本号',
    ct_user_id        bigint                                   null comment '创建人',
    ct_date           datetime       default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                                   null comment '最后修改人',
    lm_date           datetime       default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '思源交费记录表';

create index idx_pay_syswin_fee_record_p_rev_name
    on pay_syswin_fee_record (p_rev_name);

create table if not exists pay_syswin_house
(
    id                bigint auto_increment comment '主键'
        primary key,
    house_id          bigint                             null comment '房产id',
    syswin_project_id varchar(100)                       null comment '思源项目id',
    syswin_res_id     varchar(100)                       null comment '思源资源id',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '思源房产关联表';

create table if not exists pay_syswin_house_change_record
(
    id                       bigint auto_increment comment '主键'
        primary key,
    house_id                 bigint                             null comment '房产id',
    change_type              int                                null comment '变更类型 1 表格导入 2 手动编辑',
    syswin_project_id_before varchar(100)                       null comment '思源项目id(变更前)',
    syswin_res_id_before     varchar(100)                       null comment '思源资源id(变更前)',
    building_area_before     decimal(18, 4)                     null comment '建筑面积(变更前)',
    syswin_project_id_after  varchar(100)                       null comment '思源项目id(变更后)',
    syswin_res_id_after      varchar(100)                       null comment '思源资源id(变更后)',
    building_area_after      decimal(18, 4)                     null comment '建筑面积(变更后)',
    remark                   varchar(255)                       null comment '备注',
    deleted                  tinyint  default 0                 null comment '删除标记',
    version                  int      default 0                 null comment '数据版本号',
    company_id               bigint                             null comment '公司id',
    ct_user_id               bigint                             null comment '创建人',
    ct_date                  datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '思源房产变更记录';

create table if not exists pay_third_log
(
    id              bigint auto_increment comment '主键'
        primary key,
    type            int                                null comment '请求类型 1 支付回调  2 退款回调 3 查询支付 4 退款',
    pay_type        int                                null comment '付款方式',
    out_trade_no    varchar(100)                       null comment '外部单号',
    callback_status varchar(100)                       null comment '回调状态',
    is_success      tinyint(1)                         null comment '是否成功 1 成功 0 失败',
    trade_no        varchar(100)                       null comment '交易单号',
    content         text                               null comment '详细信息',
    remark          text                               null comment '备注',
    company_id      bigint                             null comment '公司id',
    deleted         tinyint  default 0                 null comment '删除标记',
    version         int      default 0                 null comment '数据版本号',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '第三方支付日志';

create table if not exists points
(
    id                bigint auto_increment comment '主键'
        primary key,
    company_id        bigint                             null comment '公司id',
    community_id      bigint                             null comment '社区id',
    community_user_id bigint                             null comment '用户id',
    points            int                                null comment '当前积分',
    total_points      int                                null comment '累计获得积分',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '用户积分表';

create table if not exists points_award_rule
(
    id              bigint auto_increment comment '主键'
        primary key,
    template_id     bigint                               null comment '模板id',
    available       tinyint(1) default 0                 null comment '是否启用',
    daily_limit     tinyint(1) default 0                 null comment '是否日限制',
    daily_limit_num int        default 0                 null comment '每日限制次数',
    value           int        default 0                 null comment '每个公司设置的值',
    deleted         tinyint    default 0                 null comment '删除标记',
    version         int        default 0                 null comment '数据版本号',
    company_id      bigint                               null comment '公司id',
    ct_user_id      bigint                               null comment '创建人',
    ct_date         datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                               null comment '最后修改人',
    lm_date         datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分奖励规则';

create table if not exists points_award_rule_template
(
    id          bigint auto_increment comment '主键'
        primary key,
    code        varchar(20)                        null comment '编号',
    description varchar(200)                       null comment '描述',
    type        int                                null comment '变动类型 1:增加 2:减少',
    platform    int                                null comment '平台1:业主端 2:管理端',
    remark      varchar(1000)                      null comment '备注',
    show_order  int      default 0                 null comment '排序号',
    deleted     tinyint  default 0                 null comment '删除标记',
    version     int      default 0                 null comment '数据版本号',
    ct_user_id  bigint                             null comment '创建人',
    ct_date     datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                             null comment '最后修改人',
    lm_date     datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分奖励规则模板';

create table if not exists points_lucky_wheel
(
    id              bigint auto_increment comment '主键'
        primary key,
    name            varchar(20)                          null comment '抽奖活动名称',
    points          int                                  null comment '消耗积分数量',
    daily_limit     tinyint(1) default 0                 null comment '是否日限制',
    daily_limit_num int        default 0                 null comment '每日限制次数',
    status          int                                  null comment '状态 0：新建 1：已发布 2：已撤销',
    remark          varchar(1000)                        null comment '备注',
    show_order      int        default 0                 null comment '排序号',
    deleted         tinyint    default 0                 null comment '删除标记',
    version         int        default 0                 null comment '数据版本号',
    company_id      bigint                               null comment '公司id',
    ct_user_id      bigint                               null comment '创建人',
    ct_date         datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                               null comment '最后修改人',
    lm_date         datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分大转盘';

create table if not exists points_lucky_wheel_item
(
    id             bigint auto_increment comment '主键'
        primary key,
    lucky_wheel_id bigint                             null comment '转盘id',
    name           varchar(100)                       null comment '奖品名称',
    thumbnail      varchar(255)                       null comment '缩略图',
    description    varchar(200)                       null comment '中奖提示',
    points         int      default 0                 null comment '中奖积分',
    rate           decimal(18, 4)                     null comment '中奖概率',
    max_num        int                                null comment '最大中奖次数',
    lucky_num      int                                null comment '抽中次数',
    type           int                                null comment '类型1:积分 2:实物',
    remark         varchar(1000)                      null comment '备注',
    show_order     int      default 0                 null comment '排序号',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    company_id     bigint                             null comment '公司id',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分大转盘';

create table if not exists points_lucky_wheel_record
(
    id                  bigint auto_increment comment '主键'
        primary key,
    community_id        bigint                             null comment '社区id',
    community_user_id   bigint                             null comment '用户id',
    lucky_wheel_id      bigint                             null comment '转盘id',
    lucky_wheel_item_id bigint                             null comment '转盘明细id',
    code                varchar(20)                        null comment '抽奖单号',
    name                varchar(100)                       null comment '奖品名称',
    type                int                                null comment '类型1:积分 2:实物',
    points              int      default 0                 null comment '中奖积分',
    status              int                                null comment '状态 1:待领取 2:已领取',
    remark              varchar(1000)                      null comment '备注',
    deleted             tinyint  default 0                 null comment '删除标记',
    version             int      default 0                 null comment '数据版本号',
    company_id          bigint                             null comment '公司id',
    ct_user_id          bigint                             null comment '创建人',
    ct_date             datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id          bigint                             null comment '最后修改人',
    lm_date             datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分大转盘';

create table if not exists points_product
(
    id                    bigint auto_increment comment '主键'
        primary key,
    name                  varchar(100)                         null comment '商品名称',
    content               text                                 null comment '产品介绍',
    specifications        text                                 null comment '商品规格参数',
    thumbnail             varchar(255)                         null comment '缩略图',
    points                int        default 0                 null comment '所需积分',
    total_stock_num       int        default 0                 null comment '累计库存',
    sale_num              int        default 0                 null comment '销售数量',
    allow_arrearage       tinyint(1) default 1                 null comment '是否允许拖欠物业费报名',
    allow_arrearage_mouth int        default 0                 null comment '允许欠费报名月数,默认0,不允许欠费',
    status                int        default 1                 null comment '状态 0：新建 1：已发布 2：已撤销',
    remark                varchar(1000)                        null comment '备注',
    show_order            int        default 0                 null comment '排序号',
    deleted               tinyint    default 0                 null comment '删除标记',
    version               int        default 0                 null comment '数据版本号',
    company_id            bigint                               null comment '公司id',
    ct_user_id            bigint                               null comment '创建人',
    ct_date               datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id            bigint                               null comment '最后修改人',
    lm_date               datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分商品';

create table if not exists points_product_community
(
    id                bigint auto_increment comment '主键id'
        primary key,
    product_id        bigint                             null comment '产品id',
    community_id      bigint                             null comment '发布社区',
    total_stock_num   int      default 0                 null comment '库存数量',
    current_stock_num int                                null comment '当前库存',
    sale_num          int      default 0                 null comment '销售数量',
    limit_num         int                                null comment '限购数量',
    top               tinyint(1)                         null comment '是否在社区内置顶',
    online            tinyint(1)                         null comment '是否商家',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    company_id        bigint                             null comment '公司id',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '发布社区';

create table if not exists points_product_community_stock_record
(
    id                   bigint auto_increment comment '主键id'
        primary key,
    product_id           bigint                             null comment '产品id',
    community_id         bigint                             null comment '发布社区',
    product_community_id bigint                             null comment '商品发布社区id',
    type                 int                                null comment '变动类型 1:增加库存 2::减少库存 3:兑换 4:取消兑换',
    value                int      default 0                 null comment '变动值',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    company_id           bigint                             null comment '公司id',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分商品库存变动记录';

create table if not exists points_product_image
(
    id         bigint auto_increment comment '主键'
        primary key,
    product_id bigint                             null comment '产品id',
    image      varchar(255)                       null comment '图片地址',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分商品图片';

create table if not exists points_product_order
(
    id                bigint auto_increment comment '主键'
        primary key,
    code              varchar(30)                        null comment '订单编号',
    product_id        bigint                             null comment '产品id',
    community_id      bigint                             null comment '社区id',
    product_num       int                                null comment '购买数量',
    product_point     int                                null comment '产品单价',
    points            int                                null comment '使用积分',
    community_user_id bigint                             null comment '社区用户id',
    cancel_date       datetime default CURRENT_TIMESTAMP null comment '取消时间',
    cancel_reason     varchar(1000)                      null comment '取消原因',
    pick_date         datetime default CURRENT_TIMESTAMP null comment '领取时间',
    status            int                                null comment '状态 10:已支付 20:已领取 30:已取消',
    remark            varchar(1000)                      null comment '备注',
    show_order        int      default 0                 null comment '排序号',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    company_id        bigint                             null comment '公司id',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '积分商品订单';

create table if not exists points_record
(
    id                bigint auto_increment comment '主键'
        primary key,
    community_user_id bigint                             null comment '社区用户id',
    community_id      bigint                             null comment '社区id',
    rule_id           bigint   default 0                 null comment '规则id',
    description       varchar(500)                       null comment '描述',
    change_event      int                                null comment '变动事件 1:每日登陆 2:积分兑换 3:积分取消兑换 4:积分抽奖消耗 5:积分抽奖奖励 6:投票 7:问卷',
    change_type       int                                null comment '积分变动类型 1:增加 2:扣减',
    source_id         bigint                             null comment '源头业务id',
    points            int                                null comment '变动积分',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    company_id        bigint                             null comment '公司id',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '用户积分表

';

create table if not exists push_app_job
(
    id               bigint auto_increment comment '主键'
        primary key,
    type             int                                null comment '推送业务类型SYS0013',
    source           int                                null comment '来源 SYS0004',
    target           int                                null comment '推送目标 SYS0006',
    push_type        int                                null comment '类型SYS0014',
    source_id        bigint                             null comment '来源业务id',
    title            varchar(100)                       null comment '通知标题',
    extras           varchar(200)                       null comment '额外参数',
    content          varchar(500)                       null comment '推送内容',
    deleted          tinyint  default 0                 null comment '删除标记',
    company_id       bigint                             null comment '公司id',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    exec_date        datetime                           null comment '作业执行开始时间',
    exec_end_date    datetime                           null comment '作业执行结束时间',
    exec_status      int      default 0                 null comment '发送状态 0：待发送 1：发送中 2：发送完成',
    exec_num         int                                null comment '发送数量',
    exec_success_num int                                null comment '发送成功数量'
)
    comment 'app推送信息';

create table if not exists push_app_log
(
    id             bigint auto_increment comment '主键'
        primary key,
    job_id         bigint        null comment '作业id',
    object         varchar(100)  null comment 'tag或alias',
    send_date      datetime      null comment '发送时间',
    send_status    int default 0 null comment '发送状态 0：待发送 1：发送成功 2：发送失败',
    send_error_msg varchar(500)  null comment '发送失败原因'
)
    comment '推送短信';

create table if not exists push_sms_job
(
    id               bigint auto_increment comment '主键'
        primary key,
    type             int                                null comment '短信类型 1:业主登录验证码  2:密码找回验证码 3:通知公告',
    source           int                                null comment '发送源 1:业主端APP 2:管理端APP 3:总控平台 4:公司运营平台',
    source_id        bigint                             null comment '来源业务id',
    sign_name        varchar(20)                        null comment '阿里短信签名',
    template_code    varchar(20)                        null comment '短信模板',
    template_param   varchar(500)                       null comment '短信模板变量值',
    deleted          tinyint  default 0                 null comment '删除标记',
    company_id       bigint                             null comment '公司id',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    exec_date        datetime                           null comment '作业开始时间',
    exec_end_date    datetime                           null comment '作业结束时间',
    exec_status      int      default 0                 null comment '发送状态 0：待发送 1：发送中 2：发送完成',
    exec_num         int                                null comment '发送数量',
    exec_success_num int      default 0                 null comment '发送成功数量'
)
    comment '推送短信';

create table if not exists push_sms_log
(
    id             bigint auto_increment comment '主键'
        primary key,
    job_id         bigint                             null comment '作业id',
    phone          varchar(20)                        null comment '电话号码',
    send_date      datetime default CURRENT_TIMESTAMP null comment '发送时间',
    send_status    int      default 0                 null comment '发送状态 0：待发送 1：发送成功 2：发送失败',
    send_error_msg varchar(500)                       null comment '发送失败原因'
)
    comment '推送短信';

create table if not exists questionnaire
(
    id         bigint auto_increment comment '主键'
        primary key,
    title      varchar(100)                         null comment '标题',
    type       int                                  null comment '调查类型 1:集团 2:物业 3:运营',
    start_time datetime                             null comment '调查开始时间',
    end_time   datetime                             null comment '调查结束时间',
    points     int        default 0                 null comment '奖励积分',
    summary    text                                 null comment '摘要',
    thumbnail  varchar(255)                         null comment '列表缩略图',
    content    text                                 null comment '内容',
    allow_push tinyint(1) default 0                 null comment '是否允许推送 0：不推送 1：推送',
    view_num   int        default 0                 null comment '浏览数',
    finish_num int                                  null comment '完成数量',
    status     int        default 0                 null comment '状态 0：新建 1：已发布 2：已撤销',
    remark     varchar(1000)                        null comment '备注',
    show_order int        default 0                 null comment '排序号',
    deleted    tinyint    default 0                 null comment '删除标记',
    version    int        default 0                 null comment '数据版本号',
    company_id bigint                               null comment '公司id',
    ct_user_id bigint                               null comment '创建人',
    ct_date    datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                               null comment '最后修改人',
    lm_date    datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '问卷调查
';

create table if not exists questionnaire_community
(
    id               bigint auto_increment comment '主键'
        primary key,
    questionnaire_id bigint                             null comment '调查问卷id',
    community_id     bigint                             null comment '社区id',
    deleted          tinyint  default 0                 null comment '删除标记',
    version          int      default 0                 null comment '数据版本号',
    company_id       bigint                             null comment '公司id',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '问卷调查发布社区

';

create table if not exists questionnaire_item
(
    id               bigint auto_increment comment '主键'
        primary key,
    questionnaire_id bigint                               null comment '调查问卷id',
    required         tinyint(1) default 1                 null comment '是否必填',
    description      varchar(500)                         null comment '问题描述',
    type             int                                  null comment '问题类型1:单选 2:多选 3:文字',
    remark           varchar(1000)                        null comment '备注',
    show_order       int        default 0                 null comment '排序号',
    deleted          tinyint    default 0                 null comment '删除标记',
    version          int        default 0                 null comment '数据版本号',
    company_id       bigint                               null comment '公司id',
    ct_user_id       bigint                               null comment '创建人',
    ct_date          datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                               null comment '最后修改人',
    lm_date          datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '问卷调查题目';

create table if not exists questionnaire_option
(
    id               bigint auto_increment comment '主键'
        primary key,
    questionnaire_id bigint                             null comment '问卷id',
    item_id          bigint                             null comment '问题id',
    title            varchar(100)                       null comment '标题',
    description      varchar(500)                       null comment '选项描述',
    image            varchar(255)                       null comment '图片',
    score            int      default 0                 null comment '得分',
    remark           varchar(1000)                      null comment '备注',
    show_order       int      default 0                 null comment '排序号',
    deleted          tinyint  default 0                 null comment '删除标记',
    version          int      default 0                 null comment '数据版本号',
    company_id       bigint                             null comment '公司id',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '问卷调查题目选项';

create table if not exists questionnaire_result
(
    id                bigint auto_increment comment '主键'
        primary key,
    questionnaire_id  bigint                               null comment '调查问卷id',
    community_id      bigint                               null comment '社区id',
    community_user_id bigint                               null comment '社区用户id',
    score             int                                  null comment '调查文件得分',
    start_time        datetime                             null comment '答卷开始时间',
    end_time          datetime                             null comment '答卷结束时间',
    show_order        int        default 0                 null comment '排序号',
    finish            tinyint(1) default 0                 null comment '是否完成',
    deleted           tinyint    default 0                 null comment '删除标记',
    version           int        default 0                 null comment '数据版本号',
    company_id        bigint                               null comment '公司id',
    ct_user_id        bigint                               null comment '创建人',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                               null comment '最后修改人',
    lm_date           datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '问卷调查结果主表

';

create table if not exists questionnaire_result_detail
(
    id               bigint auto_increment comment '主键'
        primary key,
    questionnaire_id bigint                             null comment '问卷id',
    result_id        bigint                             null comment '结果id',
    text_result      text                               null comment '文本结果',
    item_id          bigint                             null comment '问题id',
    option_id        bigint                             null comment '选项id',
    deleted          tinyint  default 0                 null comment '删除标记',
    version          int      default 0                 null comment '数据版本号',
    company_id       bigint                             null comment '公司id',
    ct_user_id       bigint                             null comment '创建人',
    ct_date          datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                             null comment '最后修改人',
    lm_date          datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '问卷调查结果详情

';

create table if not exists repair
(
    id                   bigint auto_increment comment '主键'
        primary key,
    code                 varchar(32)                        null comment '订单编号',
    community_id         bigint                             null comment '社区id',
    area_id              bigint   default 0                 null comment '区域id',
    house_id             bigint                             null comment '房产id',
    community_user_id    bigint                             null comment '社区用户id',
    source               int                                null comment '来源 1:业主端APP 2:管理端APP 3:总控平台 4:公司运营平台',
    repair_kind_id       int                                null comment '报修性质 1.个人 2.公共 3.报事',
    repair_type_id       bigint                             null comment '报修部位',
    priority             int                                null comment '问题优先级',
    repair_desc          varchar(2000)                      null comment '问题描述',
    linkman              varchar(100)                       null comment '联系人',
    phone                varchar(20)                        null comment '联系电话',
    pay_type             int                                null comment '支付类型 1:线上 2:线下 3:免单',
    pay_code             varchar(32)                        null comment '支付单号',
    pay_amount           decimal(18, 2)                     null comment '支付金额',
    pay_succeed          bit      default b'0'              null comment '是否支付成功',
    important            bit                                null comment '重点提醒',
    problem_type         int                                null comment '疑难问题类型',
    problem_duty_role    varchar(200)                       null comment '疑难问题处角色',
    problem_duty_user    varchar(200)                       null comment '疑难问题处理人',
    problem_plan_date    datetime                           null comment '疑难问题计划完成时间',
    comment_type_id      int                                null comment '评价类型1.业主评价 2.回访评价',
    commented            bit                                null comment '是否已评价',
    comment_score        int                                null comment '1-5 (1：非常不满意 2、不满意 3、一般 4、很满意 5、非常满意）',
    comment_content      varchar(1000)                      null comment '评价内容',
    comment_date         datetime                           null comment '评价时间',
    visited_user_id      bigint                             null comment '回访人员id',
    is_read              bit                                null comment '是否已读',
    reply_on_time        bit                                null comment '是否准时对业主说',
    available            bit                                null comment '是否有效',
    proc_def_id          varchar(200)                       null comment '流程定义',
    proc_id              varchar(200)                       null comment '流程id',
    proc_task_id         varchar(200)                       null comment '流程任务id',
    proc_task_name       varchar(200)                       null comment '流程状态名称',
    proc_status_id       int                                null comment '流程状态 1.未发起 2.流程中 3.结束',
    proc_task_date       datetime                           null comment '流程当前任务进入时间',
    proc_task_limit_date datetime                           null comment '当前任务到期时间',
    proc_time_limit      int                                null comment '流程当前任务时限',
    proc_end_date        datetime                           null comment '流程结束时间',
    proc_overdue         bit      default b'0'              null comment '流程处理是否曾经逾期',
    remark               varchar(1000)                      null comment '备注',
    show_order           int      default 0                 null comment '排序号',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    company_id           bigint                             null comment '公司id',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '报修主表';

create table if not exists repair_attachment
(
    id         bigint auto_increment comment '主键'
        primary key,
    repair_id  bigint        null comment '报修id',
    type       int           null comment '附件类型 1.图片 2.录音',
    path       varchar(500)  null comment '附件路径',
    show_order int default 0 null comment '排序号'
)
    comment '报修附件';

create table if not exists repair_difficult
(
    id                   bigint auto_increment comment '主键'
        primary key,
    community_id         bigint                             null comment '社区id',
    house_id             bigint                             null comment '房产id',
    linkman              varchar(100)                       null comment '联系人',
    phone                varchar(50)                        null comment '联系电话',
    type                 int                                null comment '报修问题分类 1:漏水类 2:门窗类 3:裂缝类 4:墙体类 5:给排水类 6:其他类',
    description          varchar(2000)                      null comment '报修问题描述',
    current_track_record varchar(2000)                      null comment '最新跟踪记录',
    start_date           datetime                           null comment '问题发生时间',
    plan_date            datetime                           null comment '计划完成时间',
    finish_date          datetime                           null comment '完成时间',
    arrearage            bit      default b'0'              null comment '是否欠费0：未欠费 1：欠费',
    arrearage_amount     decimal(16, 2)                     null comment '欠费金额',
    important            bit      default b'0'              null comment '是否重点提醒',
    unable_radicate      bit      default b'0'              null comment '是否无法根除0:否 1:是',
    finish               bit      default b'0'              null comment '是否完成 0：未完成 1：已完成',
    source               int                                null comment '来源 1：流程转入 2：创建',
    repair_id            bigint                             null comment '报修id',
    remark               varchar(1000)                      null comment '备注',
    show_order           int      default 0                 null comment '排序号',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    company_id           bigint                             null comment '公司id',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '报修疑难问题';

create table if not exists repair_difficult_change_record
(
    id                  bigint auto_increment comment '主键'
        primary key,
    repair_difficult_id bigint                             null comment '疑难问题id',
    type                varchar(100)                       null comment '报修问题分类',
    plan_date           datetime                           null comment '计划完成时间',
    duty_role           varchar(1000)                      null comment '责任角色',
    duty_user           varchar(1000)                      null comment '责任用户',
    ct_user_id          bigint                             null comment '创建人',
    ct_date             datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '疑难问题变更记录';

create table if not exists repair_difficult_duty_object
(
    id                  bigint auto_increment comment '主键'
        primary key,
    repair_difficult_id bigint null comment '疑难问题id',
    type                int    null comment '类型 1：用户 2：角色',
    object_id           bigint null comment '对象id'
)
    comment '疑难问负责人';

create table if not exists repair_difficult_track_record
(
    id                  bigint auto_increment comment '主键'
        primary key,
    repair_difficult_id bigint                             null comment '疑难问题id',
    content             varchar(2000)                      null comment '跟踪信息内容',
    ct_user_id          bigint                             null comment '创建用户',
    ct_date             datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '疑难问题跟踪信息';

create table if not exists repair_type
(
    id         bigint auto_increment comment '主键'
        primary key,
    parent_id  bigint                               null comment '父节点ID',
    name       varchar(300)                         null comment '部位名称',
    is_count   tinyint(1) default 1                 null comment '是否参与数据统计',
    is_leaf    tinyint(1)                           null comment '是否叶子节点 0:非叶子节点 1:叶子节点',
    id_path    varchar(200)                         null comment 'id路径',
    remark     varchar(1000)                        null comment '备注',
    show_order int        default 0                 null comment '排序号',
    deleted    tinyint    default 0                 null comment '删除标记',
    version    int        default 0                 null comment '数据版本号',
    company_id bigint                               null comment '公司id',
    ct_user_id bigint                               null comment '创建人',
    ct_date    datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                               null comment '最后修改人',
    lm_date    datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '维修类型';

create table if not exists saas_app_advertisement
(
    id                bigint auto_increment comment '主键'
        primary key,
    type              int                                    null comment '广告类型 1:首页顶部轮播 2:首页中部轮播 3:首页中部左侧广告位 4:首页中部右侧广告位 5.社区周边轮播',
    title             varchar(100)                           null comment '广告名称',
    description       varchar(500)                           null comment '描述',
    file_path         varchar(200) default '0'               null comment '文件路径',
    drill_type        int          default 1                 null comment '钻取类型  1:不钻取 2:网页 3:模块',
    drill_module_id   bigint                                 null comment '模块id',
    drill_module_type int                                    null comment '模块类型 1:列表 2:明细',
    drill_value       bigint                                 null comment '钻取值',
    url               varchar(200)                           null comment '跳转url',
    available         tinyint(1)   default 0                 null comment '启用状态',
    remark            varchar(1000)                          null comment '备注',
    show_order        int          default 0                 null comment '排序号',
    deleted           tinyint      default 0                 null comment '删除标记',
    version           int          default 0                 null comment '数据版本号',
    company_id        bigint                                 null comment '公司id',
    ct_user_id        bigint                                 null comment '创建人',
    ct_date           datetime     default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                                 null comment '最后修改人',
    lm_date           datetime     default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment 'app广告素材';

create table if not exists saas_app_boot_advertisement
(
    id         bigint auto_increment comment '主键'
        primary key,
    title      varchar(200)                           null comment '描述',
    file_path  varchar(200) default '0'               null comment '文件路径',
    start_date datetime                               null comment '开始时间',
    end_date   datetime                               null comment '结束时间',
    duration   int                                    null comment '持续时间(秒)',
    available  tinyint(1)   default 0                 null comment '启用状态',
    remark     varchar(1000)                          null comment '备注',
    show_order int          default 0                 null comment '排序号',
    deleted    tinyint      default 0                 null comment '删除标记',
    version    int          default 0                 null comment '数据版本号',
    company_id bigint                                 null comment '公司id',
    ct_user_id bigint                                 null comment '创建人',
    ct_date    datetime     default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                                 null comment '最后修改人',
    lm_date    datetime     default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '开机广告';

create table if not exists saas_app_headline
(
    id         bigint auto_increment comment '主键'
        primary key,
    content    varchar(200)                       null comment '内容',
    status     int      default 0                 null comment '状态 0：新建 1：已发布 2：已撤销',
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment 'app头条快讯';

create table if not exists saas_app_module
(
    id                        bigint auto_increment comment '主键'
        primary key,
    code                      varchar(20)                          null comment '模块编号',
    name                      varchar(100)                         null comment '模块名称',
    icon                      varchar(200)                         null comment '模块图标',
    module_group              int                                  null comment '模块分组(使用通用代码)',
    fixed                     tinyint(1)                           null comment '是否为固定模块',
    allow_advertisement_drill tinyint(1) default 1                 null comment '是否允许发布到广告位',
    android_view              varchar(200)                         null comment 'android首页视图路径',
    ios_view                  varchar(200)                         null comment 'ios首页视图路径',
    android_detail_view       varchar(200)                         null comment 'android详情页视图路径',
    ios_detail_view           varchar(200)                         null comment 'ios详情页视图路径',
    is_h5                     tinyint(1)                           null comment '是否为h5模块',
    h5_list_url               varchar(200)                         null comment 'h5列表URL',
    h5_detail_url             varchar(200)                         null comment 'h5详情URL',
    need_login                tinyint(1) default 0                 null comment '是否需要登录',
    need_bind_house           tinyint(1) default 0                 null comment '是否需要绑定房产',
    type                      int                                  null comment '类型：业主端 2：管理端',
    remark                    varchar(1000)                        null comment '备注',
    show_order                int        default 0                 null comment '排序号',
    deleted                   tinyint    default 0                 null comment '删除标记',
    version                   int        default 0                 null comment '数据版本号',
    ct_user_id                bigint                               null comment '创建人',
    ct_date                   datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id                bigint                               null comment '最后修改人',
    lm_date                   datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment 'app所有功能模块';

create table if not exists saas_business
(
    id                bigint auto_increment comment '主键'
        primary key,
    code              varchar(50)                        null comment '业务编号',
    name              varchar(100)                       null comment '业务名称',
    parent_id         bigint                             null comment '父亲节点id',
    icon              varchar(200)                       null comment '图标',
    vr_path           varchar(200)                       null comment '前端路径',
    vr_name           varchar(200)                       null comment '前端router name',
    vr_redirect       varchar(200)                       null comment '前端重定向地址',
    vr_component_path varchar(200)                       null comment '前端组件包路径',
    vr_keep_alive     bit                                null comment '是否缓存',
    show_in_menu      bit                                null comment '是否在菜单中显示',
    show_order        int      default 0                 null comment '显示顺序',
    version           int      default 0                 null comment '版本号',
    deleted           tinyint  default 0                 null comment '删除标记',
    ct_user_id        bigint                             null comment '创建人id',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '修改人id',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '修改时间'
)
    comment '总业务';

create table if not exists saas_business_operate
(
    id          bigint auto_increment comment '主键'
        primary key,
    business_id bigint                             null comment '业务id',
    name        varchar(100)                       null comment '操作名称',
    operate_key varchar(100)                       null comment '操作key配合业务编号,用于前端控制按钮显示隐藏',
    version     int      default 0                 null comment '版本号',
    deleted     tinyint  default 0                 null comment '删除标记',
    ct_user_id  bigint                             null comment '创建人id',
    ct_date     datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                             null comment '修改人id',
    lm_date     datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '修改时间'
)
    comment 'saas业务包含的所有操作';

create table if not exists saas_business_operate_resource
(
    id                  bigint auto_increment comment '主键'
        primary key,
    business_operate_id bigint                             null comment '业务id',
    operate_url         varchar(1000)                      null comment '操作uri',
    operate_method      varchar(100)                       null comment 'url的请求类型如GET POST PUT DELETE ALL',
    version             int      default 0                 null comment '版本号',
    deleted             tinyint  default 0                 null comment '删除标记',
    ct_user_id          bigint                             null comment '创建人id',
    ct_date             datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id          bigint                             null comment '修改人id',
    lm_date             datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '修改时间'
)
    comment 'saas业务操作对应的所有资源';

create table if not exists saas_company
(
    id           bigint auto_increment comment '主键'
        primary key,
    code         varchar(20)                        null comment '公司编码',
    name         varchar(100)                       null comment '公司名称',
    icon         varchar(200)                       null comment '公司图标',
    legal_person varchar(20)                        null comment '公司法人',
    address      varchar(200)                       null comment '公司地址',
    found_date   datetime                           null comment '成立时间',
    landline     varchar(20)                        null comment '座机',
    description  varchar(1000)                      null comment '公司简介',
    remark       varchar(1000)                      null comment '备注',
    show_order   int      default 0                 null comment '排序号',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司表';

create table if not exists saas_company_app_module
(
    id            bigint auto_increment comment '主键'
        primary key,
    company_id    bigint                             null comment '公司id',
    app_module_id bigint                             null comment '模块id',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '物业公司可用模块';

create table if not exists saas_company_business
(
    id          bigint auto_increment comment '主键'
        primary key,
    company_id  bigint                             null comment '公司id',
    business_id bigint                             null comment 'saas业务id',
    deleted     tinyint  default 0                 null comment '删除标记',
    version     int      default 0                 null comment '数据版本号',
    ct_user_id  bigint                             null comment '创建人',
    ct_date     datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                             null comment '最后修改人',
    lm_date     datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司可用模块';

create table if not exists saas_company_community_user_property
(
    id                bigint auto_increment comment '主键'
        primary key,
    company_id        bigint                               null comment '公司id',
    community_user_id bigint                               null comment '社区用户',
    allow_speak       tinyint(1) default 1                 null comment '可否发言',
    remark            varchar(1000)                        null comment '备注',
    deleted           tinyint    default 0                 null comment '删除标记',
    version           int        default 0                 null comment '数据版本号',
    ct_user_id        bigint                               null comment '创建人',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                               null comment '最后修改人',
    lm_date           datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司与社区用户配置表';

create table if not exists saas_company_evaluate_record
(
    id            bigint auto_increment comment '主键'
        primary key,
    business_type int                                null comment '业务类型 1：报修 2：家政',
    business_id   bigint                             null comment '业务id',
    tag_id        bigint                             null comment '标签id',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    company_id    bigint                             null comment '公司id',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司评价标签记录';

create table if not exists saas_company_evaluate_tag
(
    id            bigint auto_increment comment '主键'
        primary key,
    business_type int                                null comment '业务类型 1：报修 2：家政',
    score         int                                null comment '星级打分 1-5',
    content       varchar(20)                        null comment '标签内容',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    company_id    bigint                             null comment '公司id',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司评价标签';

create table if not exists saas_company_organization
(
    id                       bigint auto_increment comment '主键'
        primary key,
    name                     varchar(100)                       null comment '组织名称',
    code                     varchar(50)                        null comment '组织编号',
    parent_id                bigint                             null comment '父组织ID',
    org_type                 int                                null comment '组织类型，1总公司 2子公司 3第三方公司 4部门',
    outsourcing_company_type int                                null comment '外包单位类型1.维保公司 2.家政公司',
    introduction             varchar(1000)                      null comment '简介',
    id_path                  varchar(100)                       null comment 'id路径',
    is_third                 tinyint(1)                         null comment '是否是第三方公司0否 1是',
    child_num                int                                null comment '孩子节点数量',
    is_leaf                  tinyint(1)                         null comment '是否叶子节点',
    show_order               int                                null comment '排序号',
    deleted                  tinyint  default 0                 null comment '删除标记',
    version                  int      default 0                 null comment '数据版本号',
    company_id               bigint                             null comment '公司id',
    ct_user_id               bigint                             null comment '创建人',
    ct_date                  datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id               bigint                             null comment '最后修改人',
    lm_date                  datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司组织架构表';

create table if not exists saas_company_param
(
    id             bigint auto_increment comment '主键'
        primary key,
    company_id     bigint                             null comment '公司id',
    code           varchar(50)                        null comment '编码',
    type           int                                null comment '参数类型 1 整型  2 字符串  3 富文本 4 图片 5文件',
    name           varchar(100)                       null comment '参数名称',
    value_int      int                                null comment '整数值',
    value_str      varchar(500)                       null comment '字符串类型',
    value_longtext longtext                           null comment '富文本',
    remark         varchar(1000)                      null comment '备注',
    show_order     int      default 0                 null comment '排序号',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区公共参数配置';

create table if not exists saas_company_reply_template
(
    id            bigint auto_increment comment '主键'
        primary key,
    business_type int                                null comment '流程业务类型 1.报修 2.投诉 3.集团验房 4.业主验房 5.家政服务 6.设备维修',
    content       varchar(200)                       null comment '模板内容',
    show_order    int      default 0                 null comment '排序号',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    company_id    bigint                             null comment '公司id',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司对业主说模板';

create table if not exists saas_company_role
(
    id         bigint auto_increment comment '主键'
        primary key,
    code       varchar(20)                          null comment '角色编号',
    type       int                                  null comment '角色分类(码表)',
    name       varchar(100)                         null comment '角色名称',
    available  tinyint(1) default 1                 null comment '启用状态',
    remark     varchar(1000)                        null comment '备注',
    version    int        default 0                 null comment '数据版本号',
    company_id bigint                               null comment '公司id',
    ct_user_id bigint                               null comment '创建人',
    ct_date    datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                               null comment '最后修改人',
    lm_date    datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司自定义角色';

create table if not exists saas_company_role_app_module
(
    id         bigint auto_increment comment '主键'
        primary key,
    role_id    bigint                             null comment '角色id',
    module_id  bigint                             null comment '模块id',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '角色对应管理端app模块';

create table if not exists saas_company_role_dashboard
(
    id         bigint auto_increment comment '主键'
        primary key,
    role_id    bigint                             null comment '角色id',
    module_id  int                                null comment '模块id',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '角色数据看板模块分配表';

create table if not exists saas_company_role_operate
(
    id         bigint auto_increment comment '主键'
        primary key,
    role_id    bigint null comment '角色id',
    operate_id bigint null comment '操作id',
    company_id bigint null comment '公司id'
)
    comment '角色关联操作';

create table if not exists saas_company_steward
(
    id              bigint auto_increment comment '主键'
        primary key,
    code            varchar(20)                          null comment '工号',
    company_user_id bigint                               null comment '公司用户id',
    hiredate        datetime                             null comment '入职时间',
    title           int                                  null comment '头衔 1:客户管家 2:项目经理 3:秩序维护 4:综合维护 5:租售服务',
    avatar          varchar(255)                         null comment '头像',
    slogan          varchar(200)                         null comment '服务口号',
    phone           varchar(20)                          null comment '联系电话',
    description     varchar(500)                         null comment '员工简介',
    online          tinyint(1) default 0                 null comment '是否上线',
    remark          varchar(1000)                        null comment '备注',
    show_order      int        default 0                 null comment '排序号',
    deleted         tinyint    default 0                 null comment '删除标记',
    version         int        default 0                 null comment '数据版本号',
    company_id      bigint                               null comment '所属公司',
    ct_user_id      int                                  null comment '创建人',
    ct_date         datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      int                                  null comment '最后修改人',
    lm_date         datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '社区管家';

create table if not exists saas_company_user
(
    id               bigint auto_increment comment '主键'
        primary key,
    username         varchar(100)                         null comment '用户名',
    password         varchar(200)                         null comment '用户密码',
    real_name        varchar(100)                         null comment '用户真实姓名',
    sex              int                                  null comment '性别 1:男 2:女 3:中性 4:未知',
    data_access_type int                                  null comment '数据访问类型 1:仅查看自己 2:查看本组织',
    email            varchar(100)                         null comment '邮箱地址',
    mobile           varchar(100)                         null comment '联系电话',
    org_id           int                                  null comment '所属组织',
    type             int                                  null comment '用户类型',
    god              tinyint(1) default 0                 null comment '是否上帝用户',
    available        tinyint(1) default 1                 null comment '启用状态',
    avatar           varchar(200)                         null comment '头像',
    status           int                                  null comment '扩展状态字段 备用',
    remark           varchar(1000)                        null comment '备注',
    version          int        default 0                 null comment '数据版本号',
    company_id       bigint                               null comment '公司id',
    ct_user_id       bigint                               null comment '创建人',
    ct_date          datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id       bigint                               null comment '最后修改人',
    lm_date          datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司用户';

create table if not exists saas_company_user_community
(
    id              bigint auto_increment comment '主键'
        primary key,
    company_user_id bigint                             null comment '关联用户',
    community_id    bigint                             null comment '管理社区',
    drill           bit      default b'0'              null comment '钻取权限',
    deleted         tinyint  default 0                 null comment '删除标记',
    version         int      default 0                 null comment '数据版本号',
    company_id      bigint                             null comment '公司id',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司用户管理社区';

create table if not exists saas_company_user_community_area
(
    id              bigint auto_increment comment '主键'
        primary key,
    company_user_id bigint                             null comment '关联用户',
    community_id    bigint                             null comment '管理社区',
    area_id         bigint                             null comment '社区id',
    company_id      bigint                             null comment '公司id',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '公司用户管理社区区域';

create table if not exists saas_company_user_login_log
(
    id          bigint auto_increment comment '主键'
        primary key,
    source      int                                null comment '来源 SYS0004 2：管理端APP 4：公司运营平台 5:400客户端',
    type        int                                null comment '登录类型  2:登录 3:自动登录 4:登出',
    user_id     bigint                             null comment '用户id',
    year        int                                null comment '登录年份',
    month       varchar(20)                        null comment '登录月份',
    day         varchar(20)                        null comment '登录日期',
    time        datetime default CURRENT_TIMESTAMP null comment '登录时间',
    platform    varchar(50)                        null comment '设备类型',
    os_model    varchar(50)                        null comment '设备型号',
    os_version  varchar(50)                        null comment '系统版本',
    app_version varchar(50)                        null comment 'app版本'
)
    comment '业主用户登录日志';

create table if not exists saas_company_user_off_duty
(
    id              bigint auto_increment comment '主键'
        primary key,
    company_user_id bigint                             null comment '公司用户',
    type            int                                null comment '休班类型(1:请假,2:休班)',
    begin_time      datetime                           null comment '开始时间',
    end_time        datetime                           null comment '结束时间',
    deleted         tinyint  default 0                 null comment '删除标记',
    version         int      default 0                 null comment '数据版本号',
    company_id      bigint                             null comment '公司id',
    ct_user_id      bigint                             null comment '创建人',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id      bigint                             null comment '最后修改人',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '公司用户休班记录';

create table if not exists saas_company_user_org
(
    id         bigint auto_increment comment '主键'
        primary key,
    user_id    bigint                             null comment '用户id',
    org_id     bigint                             null comment '组织id',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '公司用户所属组织';

create table if not exists saas_company_user_role
(
    id         bigint auto_increment comment '主键'
        primary key,
    user_id    bigint null comment '用户id',
    role_id    bigint null comment '角色id',
    company_id bigint null comment '公司id'
)
    comment '公司用户角色';

create table if not exists saas_company_user_sys_msg
(
    id              bigint auto_increment comment '主键'
        primary key,
    msg_type        int                                null comment '消息类型 1：报事报修 2：投诉表扬 3：集团验房 4：业主验房 5：家政服务 6：设备维修',
    title           varchar(100)                       null comment '标题',
    content         varchar(500)                       null comment '消息内容',
    business_id     bigint                             null comment '业务id',
    extras          varchar(200)                       null comment '额外参数',
    receive_user_id bigint                             null comment '接收用户ID',
    deleted         tinyint  default 0                 null comment '删除标记',
    ct_date         datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_date         datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间',
    is_read         bit                                null comment '是否已读',
    read_date       datetime default CURRENT_TIMESTAMP null comment '阅读时间'
)
    comment '用户系统消息';

create table if not exists saas_param_template
(
    id             bigint auto_increment comment '主键'
        primary key,
    type           int                                  null comment '参数类型 1 整型  2 字符串  3 富文本 4 图片 5文件',
    code           varchar(50)                          null comment '编码',
    name           varchar(100)                         null comment '参数名称',
    value_int      int                                  null comment '整数值',
    value_str      varchar(500)                         null comment '字符串类型',
    value_longtext longtext                             null comment '富文本',
    system_param   tinyint(1) default 0                 null comment '是否系统变量',
    remark         varchar(1000)                        null comment '备注',
    show_order     int        default 0                 null comment '排序号',
    deleted        tinyint    default 0                 null comment '删除标记',
    version        int        default 0                 null comment '数据版本号',
    ct_user_id     bigint                               null comment '创建人',
    ct_date        datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                               null comment '最后修改人',
    lm_date        datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment 'saas系统参数模板
';

create table if not exists shedlock
(
    name        varchar(64)                         not null comment '定时任务唯一名称 主键'
        primary key,
    lock_until  timestamp(3)                        null comment '锁直到...时间结束',
    locked_at   timestamp(3)                        null comment '锁在时间',
    locked_by   varchar(255)                        not null comment '锁定冻结人',
    create_time timestamp default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '分布式定时任务锁表';

create table if not exists shop_cart
(
    id                bigint auto_increment comment '主键'
        primary key,
    community_id      bigint                               null comment '社区id',
    community_user_id bigint                               null comment '用户id',
    product_id        bigint                               null comment '商品id',
    product_spec_id   bigint                               null comment '商品规格id',
    is_check          tinyint(1) default 0                 null comment '是否选中 1 是  0 否',
    num               int                                  null comment '数量',
    deleted           tinyint    default 0                 null comment '删除标记',
    version           int        default 0                 null comment '数据版本号',
    ct_user_id        bigint                               null comment '创建人',
    ct_date           datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                               null comment '最后修改人',
    lm_date           datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商品购物车';

create table if not exists shop_order
(
    id                   bigint auto_increment comment '主键'
        primary key,
    company_id           bigint                             null comment '公司id',
    community_id         bigint                             null comment '社区id',
    community_user_id    bigint                             null comment '用户id',
    house_id             bigint                             null comment '房产id',
    code                 varchar(32)                        null comment '订单编号',
    pay_type             int                                null comment '支付类型 1 支付宝 2 微信支付',
    expire_date          datetime                           null comment '订单过期时间',
    pay_date             datetime                           null comment '支付时间',
    pay_code             varchar(32)                        null comment '支付单号',
    pay_amount           decimal(16, 2)                     null comment '支付金额',
    linkman              varchar(100)                       null comment '联系人',
    phone                varchar(20)                        null comment '联系电话',
    address              varchar(100)                       null comment '地址',
    buyer_msg            varchar(1000)                      null comment '买家留言',
    send_time            datetime                           null comment '发货时间',
    stock_out_order_id   bigint                             null comment '出库单',
    stock_out_order_code varchar(50)                        null comment '出库单号',
    status               int                                null comment '订单状态 10 待支付 20 待发货 24 出库驳回  25 备货中 30 已发货 40 已完成 50 已取消',
    remark               varchar(100)                       null comment '备注',
    deleted              tinyint  default 0                 null comment '删除标记',
    version              int      default 0                 null comment '数据版本号',
    ct_user_id           bigint                             null comment '创建人',
    ct_date              datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id           bigint                             null comment '最后修改人',
    lm_date              datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商品订单';

create table if not exists shop_order_product
(
    id            bigint auto_increment comment '主键'
        primary key,
    order_id      bigint                             null comment '订单id',
    product_id    bigint                             null comment '商品id',
    spec_id       bigint                             null comment '规格id',
    product_image varchar(255)                       null comment '商品图片',
    product_name  varchar(50)                        null comment '商品名称',
    spec_name     varchar(50)                        null comment '规格名称',
    sale_price    decimal(16, 2)                     null comment '售价',
    price         decimal(16, 2)                     null comment '原价',
    num           int      default 0                 null comment '数量',
    total_amount  decimal(16, 2)                     null comment '总金额',
    deleted       tinyint  default 0                 null comment '删除标记',
    version       int      default 0                 null comment '数据版本号',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '订单和商品关联表';

create table if not exists shop_product
(
    id            bigint auto_increment comment '主键'
        primary key,
    company_id    bigint                             null comment '公司id',
    type_id       bigint                             null comment '商品分类id',
    name          varchar(100)                       null comment '名称',
    norms         varchar(100)                       null comment '规格单位',
    thumbnail     varchar(255)                       null comment '缩略图',
    brief         varchar(100)                       null comment '商品简介',
    content       text                               null comment '商品详情',
    stock         int      default 0                 null comment '总库存',
    sale_num      int      default 0                 null comment '已售数',
    view_num      int      default 0                 null comment '浏览数',
    is_hot        tinyint(1)                         null comment '是否热门 1 是  0 否',
    is_limit_num  tinyint(1)                         null comment '是否限制购买数 1 是  0 否',
    limit_num     int      default 0                 null comment '限购数量',
    limit_type    int      default 0                 null comment '限购数量方式 1、按日限购 2、按月限购 3、总限购',
    is_limit_time tinyint(1)                         null comment '是否限制时间 1 是  0 否',
    start_time    datetime                           null comment '开始时间',
    end_time      datetime                           null comment '结束时间',
    is_show       tinyint(1)                         null comment '是否上架 1 是  0 否',
    show_order    int      default 0                 null comment '排序号',
    remark        varchar(100)                       null comment '备注',
    version       int      default 0                 null comment '数据版本号',
    deleted       tinyint  default 0                 null comment '删除标记',
    ct_user_id    bigint                             null comment '创建人',
    ct_date       datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id    bigint                             null comment '最后修改人',
    lm_date       datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商品管理';

create table if not exists shop_product_attachment
(
    id         bigint auto_increment comment '主键'
        primary key,
    product_id bigint       null comment '商品id',
    path       varchar(255) null comment '附件路径'
)
    comment '商品附件';

create table if not exists shop_product_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    product_id   bigint                             null comment '商品id',
    community_id bigint                             null comment '社区id',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商品与社区关联表';

create table if not exists shop_product_spec
(
    id         bigint auto_increment comment '主键'
        primary key,
    product_id bigint                             null comment '商品id',
    name       varchar(50)                        null comment '规格名称',
    thumbnail  varchar(255)                       null comment '缩略图',
    stock      int                                null comment '库存',
    sale_num   int      default 0                 null comment '已售数',
    price      decimal(16, 2)                     null comment '原价',
    sale_price decimal(16, 2)                     null comment '售价',
    is_show    tinyint(1)                         null comment '是否显示 1 是  0 否',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商品规格';

create table if not exists shop_stock_out_order
(
    id                bigint auto_increment comment '主键'
        primary key,
    company_id        bigint                             null comment '公司id',
    code              varchar(32)                        null comment '订单编号',
    order_qty         int                                null comment '出库单订单数量',
    bom_print_time    int                                null comment '物料清单打印次数',
    order_print_time  int                                null comment '订单打印次数',
    stock_out_user_id bigint                             null comment '出库人',
    stock_out_date    datetime                           null comment '出库时间',
    status            int                                null comment '订单状态 10 出库中 20 已出库',
    remark            varchar(100)                       null comment '备注',
    deleted           tinyint  default 0                 null comment '删除标记',
    version           int      default 0                 null comment '数据版本号',
    ct_user_id        bigint                             null comment '创建人',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '最后修改人',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商品出库单';

create table if not exists shop_type
(
    id         bigint auto_increment comment '主键'
        primary key,
    name       varchar(20)                        null comment '分类名称',
    thumbnail  varchar(255)                       null comment '缩略图',
    remark     varchar(1000)                      null comment '备注',
    is_show    tinyint(1)                         null comment '是否显示 1 是  0 否',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '商品分类';

create table if not exists sys_business
(
    id                bigint auto_increment comment '主键'
        primary key,
    code              varchar(50)                        null comment '业务编号',
    name              varchar(100)                       null comment '业务名称',
    parent_id         bigint                             null comment '父亲节点id',
    icon              varchar(200)                       null comment '图标',
    vr_path           varchar(200)                       null comment '前端路径',
    vr_name           varchar(200)                       null comment '前端router name',
    vr_redirect       varchar(200)                       null comment '前端重定向地址',
    vr_component_path varchar(200)                       null comment '前端组件包路径',
    vr_keep_alive     bit                                null comment '是否缓存',
    show_in_menu      bit                                null comment '是否在菜单中显示',
    show_order        int      default 0                 null comment '显示顺序',
    version           bigint   default 0                 null comment '版本号',
    deleted           tinyint  default 0                 null comment '删除标记',
    ct_dept_id        bigint                             null comment '创建部门id',
    ct_user_id        bigint                             null comment '创建人id',
    ct_date           datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id        bigint                             null comment '修改人id',
    lm_date           datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '修改时间'
);

create table if not exists sys_business_operate
(
    id          bigint auto_increment comment '主键'
        primary key,
    business_id bigint                             null comment '业务id',
    name        varchar(100)                       null comment '操作名称',
    operate_key varchar(100)                       null comment '操作key配合业务编号,用于前端控制按钮显示隐藏',
    version     int      default 0                 null comment '版本号',
    deleted     tinyint  default 0                 null comment '删除标记',
    ct_dept_id  bigint                             null comment '创建部门id',
    ct_user_id  bigint                             null comment '创建人id',
    ct_date     datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                             null comment '修改人id',
    lm_date     datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '修改时间'
)
    comment '系统业务包含的所有操作';

create table if not exists sys_business_operate_resource
(
    id                  bigint auto_increment comment '主键'
        primary key,
    business_operate_id bigint                             null comment '业务id',
    operate_url         varchar(1000)                      null comment '操作uri',
    operate_method      varchar(100)                       null comment 'url的请求类型如GET POST PUT DELETE ALL',
    version             int      default 0                 null comment '版本号',
    deleted             tinyint  default 0                 null comment '删除标记',
    ct_dept_id          bigint                             null comment '创建部门id',
    ct_user_id          bigint                             null comment '创建人id',
    ct_date             datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id          bigint                             null comment '修改人id',
    lm_date             datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '修改时间'
)
    comment '系统操作对应的所有资源';

create table if not exists sys_code_detail
(
    id         bigint auto_increment comment '主键'
        primary key,
    type_code  varchar(20)                        null comment '字典类型编号',
    name       varchar(100)                       null comment '字典名称',
    value_int  int                                null comment '整数类型值',
    value_min  decimal(18, 4)                     null comment '最小值范围',
    value_max  decimal(18, 4)                     null comment '最大值范围',
    value_str  varchar(500)                       null comment '字符串类型值',
    object_id  bigint                             null comment '适用对象id 用于组织隔离权限id',
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    ct_dept_id bigint                             null comment '创建部门',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '字典表详情';

create index idx_sys_code_detail_type_value
    on sys_code_detail (type_code, value_int);

create table if not exists sys_code_type
(
    id           bigint auto_increment comment '主键'
        primary key,
    code         varchar(20)                        null comment '编号',
    name         varchar(100)                       null comment '名称',
    type         int                                null comment '类型 1：系统默认 2：非系统默认',
    share_status int                                null comment '备用 1：全局 2：组织隔离',
    remark       varchar(1000)                      null comment '备注',
    show_order   int      default 0                 null comment '排序号',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    ct_dept_id   bigint                             null comment '创建部门',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '字典类型表';

create table if not exists sys_param
(
    id             bigint auto_increment comment '主键'
        primary key,
    type           int                                null comment '参数类型 1 整型  2 字符串  3 富文本 4 图片',
    code           varchar(50)                        null comment '编码',
    name           varchar(100)                       null comment '参数名称',
    value_int      int                                null comment '整数值',
    value_str      varchar(500)                       null comment '字符串类型',
    value_longtext longtext                           null comment '富文本',
    remark         varchar(1000)                      null comment '备注',
    show_order     int      default 0                 null comment '排序号',
    deleted        tinyint  default 0                 null comment '删除标记',
    version        int      default 0                 null comment '数据版本号',
    ct_user_id     bigint                             null comment '创建人',
    ct_date        datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id     bigint                             null comment '最后修改人',
    lm_date        datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '系统配置参数';

create table if not exists sys_role
(
    id         bigint auto_increment comment '主键'
        primary key,
    code       varchar(20)                          null comment '角色编号',
    name       varchar(100)                         null comment '角色名称',
    available  tinyint(1) default 1                 null comment '启用状态',
    remark     varchar(1000)                        null comment '备注',
    version    int        default 0                 null comment '数据版本号',
    ct_dept_id bigint                               null comment '创建部门',
    ct_user_id bigint                               null comment '创建人',
    ct_date    datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                               null comment '最后修改人',
    lm_date    datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '系统角色';

create table if not exists sys_role_operate
(
    id                  bigint auto_increment comment '主键'
        primary key,
    role_id             bigint null comment '关联角色',
    business_operate_id bigint null comment '关联业务操作'
)
    comment '角色操作';

create table if not exists sys_user
(
    id         bigint auto_increment comment '主键'
        primary key,
    username   varchar(100)                         null comment '用户名',
    password   varchar(200)                         null comment '用户密码',
    real_name  varchar(100)                         null comment '用户真实姓名',
    email      varchar(100)                         null comment '邮箱地址',
    mobile     varchar(100)                         null comment '联系电话',
    org_id     bigint                               null comment '所属组织',
    type       int                                  null comment '用户类型',
    available  tinyint(1) default 1                 null comment '启用状态',
    avatar     varchar(100)                         null comment '头像',
    status     int                                  null comment '扩展状态字段 备用',
    remark     varchar(1000)                        null comment '备注',
    version    int        default 0                 null comment '数据版本号',
    ct_dept_id bigint                               null comment '创建部门',
    ct_user_id bigint                               null comment '创建人',
    ct_date    datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                               null comment '最后修改人',
    lm_date    datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '用户信息';

create table if not exists sys_user_company
(
    id         bigint auto_increment comment '主键'
        primary key,
    user_id    bigint                             null comment '系统用户',
    company_id bigint                             null comment '公司id',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '用户管理社区';

create table if not exists sys_user_role
(
    id      bigint auto_increment comment '主键'
        primary key,
    user_id bigint not null comment '用户',
    role_id bigint not null comment '角色'
)
    comment '用户所持有角色表';

create table if not exists vote
(
    id          bigint auto_increment comment '主键'
        primary key,
    title       varchar(100)                         null comment '标题',
    thumbnail   varchar(255)                         null comment '缩略图',
    summary     varchar(500)                         null comment '摘要',
    content     longtext                             null comment '内容',
    content_img varchar(255)                         null comment '内容主图',
    start_time  datetime                             null comment '开始时间',
    end_time    datetime                             null comment '结束时间',
    limit_type  int                                  null comment '限制类型 1:人 2:天',
    limit_num   int                                  null comment '限制数量',
    points      int        default 0                 null comment '奖励积分',
    allow_push  tinyint(1) default 0                 null comment '是否允许推送 0：不推送 1：推送',
    status      int        default 0                 null comment '状态 0：新建 1：已发布 2：已撤销',
    vote_num    int        default 0                 null comment '总投票次数',
    remark      varchar(1000)                        null comment '备注',
    show_order  int        default 0                 null comment '排序号',
    deleted     tinyint    default 0                 null comment '删除标记',
    version     int        default 0                 null comment '数据版本号',
    company_id  bigint                               null comment '公司id',
    ct_user_id  bigint                               null comment '创建人',
    ct_date     datetime   default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id  bigint                               null comment '最后修改人',
    lm_date     datetime   default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '投票活动
';

create table if not exists vote_community
(
    id           bigint auto_increment comment '主键'
        primary key,
    vote_id      bigint                             null comment '投票id',
    community_id bigint                             null comment '社区id',
    deleted      tinyint  default 0                 null comment '删除标记',
    version      int      default 0                 null comment '数据版本号',
    company_id   bigint                             null comment '公司id',
    ct_user_id   bigint                             null comment '创建人',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id   bigint                             null comment '最后修改人',
    lm_date      datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '投票关联社区';

create table if not exists vote_item
(
    id         bigint auto_increment comment '主键'
        primary key,
    vote_id    bigint                             null comment '投票id',
    title      varchar(100)                       null comment '标题',
    thumbnail  varchar(255)                       null comment '缩略图',
    content    text                               null comment '内容',
    vote_num   int                                null comment '投票次数',
    remark     varchar(1000)                      null comment '备注',
    show_order int      default 0                 null comment '排序号',
    deleted    tinyint  default 0                 null comment '删除标记',
    version    int      default 0                 null comment '数据版本号',
    company_id bigint                             null comment '公司id',
    ct_user_id bigint                             null comment '创建人',
    ct_date    datetime default CURRENT_TIMESTAMP null comment '创建时间',
    lm_user_id bigint                             null comment '最后修改人',
    lm_date    datetime default CURRENT_TIMESTAMP null on update CURRENT_TIMESTAMP comment '最后修改时间'
)
    comment '投票项目';

create table if not exists vote_record
(
    id           bigint auto_increment comment '主键'
        primary key,
    vote_id      bigint                             null comment '投票id',
    vote_item_id bigint                             null comment '项目id',
    vote_user_id bigint                             null comment '投票用户id',
    deleted      tinyint  default 0                 null comment '删除标记',
    ct_date      datetime default CURRENT_TIMESTAMP null comment '创建时间'
)
    comment '投票记录';

